<?php
/*
Plugin Name: Guest Support
Plugin URI:  https://wordpress.org/plugins/guest-support/
Description: Complete customer support ticket system for both guest and registered users.
Version:     1.2.0
Author:      RcaTheme.com
Author URI:  https://www.rcatheme.com/
License:     GPLv3
License URI: https://www.gnu.org/licenses/gpl-3.0.html
Domain Path: /languages/
Text Domain: guest-support
Tested up to: 6.5
Requires at least: 5.5
Requires PHP: 7.0
*/

defined('ABSPATH') or die;

class Guest_Support {
    /**
     * Construct
     */
    public function __construct()
    {
        // Define all the necessary constant
        $this->_define_constant();

        // Include all the required files
        $this->_includes();

        // Admin Pages
        new AdminPages();

        add_action( 'init', array($this, '_init') );

        // Load styles and scripts
        add_action( 'admin_enqueue_scripts', array($this, '_admin_assets') );
        add_action( 'template_redirect', array($this, '_front_assets') );

        add_filter( 'the_title', array($this, '_the_title'), 10, 2 );

        register_activation_hook( __FILE__, array($this, '_activation_hook') );
        add_action( 'plugins_loaded', array($this, '_plugins_loaded') );
        add_action( 'wp_enqueue_scripts', array($this, '_load_jquery') );
    }

    public function _activation_hook()
    {
        update_option('wpgs_permalinks_flushed', 0);
        GuestSupportApp::installBaseDb();
    }

    public function _plugins_loaded()
    {
        GuestSupportApp::updateBaseDb();
    }

    public function _the_title( $title, $id )
    {
        if( GuestSupportApp::getTicketPageId() == $id && !is_admin() )
        {
            return '';
        }
        return $title;
    }

    public function _init()
	{
        // Flush parmalinks
        $ticket_page_slug = GuestSupportApp::getTicketPageSlug();
        add_rewrite_tag('%id%', '([a-zA-Z0-9]+)');
        if ( $ticket_page_slug !== false )
        {
            // Rewrite ticket view page
            add_rewrite_rule('^' . $ticket_page_slug . '/([a-zA-Z0-9]+)/?', 'index.php?pagename=' . $ticket_page_slug . '&id=$matches[1]', 'top');

            if ( get_option('wpgs_permalinks_flushed') != '1' ) {
                flush_rewrite_rules(false);
                update_option('wpgs_permalinks_flushed', 1);
            }
        }

        // Load plugin textdomain
        load_plugin_textdomain( 'guest-support', false, WPGS_LANG_DIR );
        add_shortcode( 'guest-support', array($this, '_add_shortcode') );

		// Start session
		GuestSupportApp::startSession();

        // Data handler
        $handler = isset( $_POST['wpgs_handler'] ) ? sanitize_text_field( $_POST['wpgs_handler'] ) : '';
        if ( !$handler )
        {
            $handler = isset( $_GET['wpgs_handler'] ) ? sanitize_text_field( $_GET['wpgs_handler'] ) : '';
        }
        if ( $handler )
        {
            require_once WPGS_PLUGIN_DIR . 'includes/library/process-data.php';
        }

        // Setup guide
        require_once WPGS_PLUGIN_DIR . 'includes/library/setup-guide.php';
    }

    private function _define_constant()
    {
        if ( ! defined( 'WPGS_PLUGIN_DIR' ) ) define( 'WPGS_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
        if ( ! defined( 'WPGS_PLUGIN_URI' ) ) define( 'WPGS_PLUGIN_URI', plugins_url( '', __FILE__ ) );
        if ( ! defined( 'WPGS_LANG_DIR' ) ) define('WPGS_LANG_DIR', dirname(plugin_basename( __FILE__ ) ) . '/languages');
        if ( ! defined( 'WPGS_PLUGIN_NAME' ) ) define('WPGS_PLUGIN_NAME', 'Guest Support');
        if ( ! defined( 'WPGS_CUSTOM_POST_TYPE' ) ) define('WPGS_CUSTOM_POST_TYPE', 'guest-support');
        if ( ! defined( 'WPGS_PLUGIN_ADDON_DIR' ) ) define('WPGS_PLUGIN_ADDON_DIR', plugin_dir_path( __DIR__ ) . 'guest-support-addon/');
        if ( ! defined( 'WPGS_PLUGIN_ADDON_URI' ) ) define('WPGS_PLUGIN_ADDON_URI', plugin_dir_url( __DIR__ ) . 'guest-support-addon');
    }

    public function _admin_assets()
    {
        wp_enqueue_style( 'wpgs-admin-styles', WPGS_PLUGIN_URI . '/assets/admin.css', [], '1.2.0' );
        wp_enqueue_script( 'wpgs-admin-scripts', WPGS_PLUGIN_URI . '/assets/admin.js', array('jquery'), '1.2.0', false );
        wp_enqueue_script( 'wpgs-admin-dragndrop-scripts', WPGS_PLUGIN_URI . '/assets/dragndrop.js', array('jquery'), '1.2.0', false );
        if ( GuestSupportApp::isPro() )
        {
            // wp_enqueue_style( 'wpgs-addon-admin-styles', WPGS_PLUGIN_ADDON_URI . '/assets/admin.css' );
            wp_enqueue_script( 'wpgs-addon-admin-scripts', WPGS_PLUGIN_ADDON_URI . '/assets/admin.js', array('jquery'), '1.2.0', false );
        }
    }

    public function _front_assets()
    {
        wp_register_style( 'guest-support-style', WPGS_PLUGIN_URI . '/assets/styles.min.css', array(), '1.2.0' );
        wp_register_script( 'guest-support-script', WPGS_PLUGIN_URI . '/assets/scripts.min.js', array('jquery'), '1.2.0', false );
        $config = GuestSupportApp::config();
        if ( $config->recaptcha_secret_key && $config->recaptcha_site_key )
        {
            wp_register_script( 'guest-support-recaptcha-script', 'https://www.google.com/recaptcha/api.js?render=' . $config->recaptcha_site_key, array('jquery'), false, false );
        }
        if ( GuestSupportApp::isPro() )
        {
            wp_register_style( 'guest-support-addon-style', WPGS_PLUGIN_ADDON_URI . '/assets/styles.min.css', array(), '1.2.0' );
            wp_register_script( 'guest-support-addon-script', WPGS_PLUGIN_ADDON_URI . '/assets/scripts.min.js', array('jquery'), '1.2.0', false );
        }
    }

    private function _includes()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/library/class-dbquery.php';
        require_once WPGS_PLUGIN_DIR . 'includes/library/class-app.php';
        if ( file_exists( WPGS_PLUGIN_ADDON_DIR . 'includes/library/class-pro-dbquery.php' ) )
        {
            require_once WPGS_PLUGIN_ADDON_DIR . 'includes/library/class-pro-dbquery.php';
        }
        if ( file_exists( WPGS_PLUGIN_ADDON_DIR . 'includes/library/class-pro-app.php' ) )
        {
            require_once WPGS_PLUGIN_ADDON_DIR . 'includes/library/class-pro-app.php';
        }
        require_once WPGS_PLUGIN_DIR . 'includes/admin/class-adminpages.php';
    }

    /**
     * Load jQuery
     */
    public function _load_jquery(){
        wp_enqueue_script('jquery');
    }

    /**
     * Shortcode
     */
    public function _add_shortcode( $atts ) {
        $options = shortcode_atts( array(
            'view' => '',
            'id' => 0
        ), $atts );

        // Load frontend styles and scripts
        wp_enqueue_style( 'guest-support-style' );
        wp_enqueue_script( 'guest-support-script' );

        if ( GuestSupportApp::isPro() )
        {
            wp_enqueue_style( 'guest-support-addon-style' );
            wp_enqueue_script( 'guest-support-addon-script' );
        }

        // Process view
        if( !$options['view'] )
        {
            $errorMessage = '<p>' . esc_html__( 'Guest Support: Invalid shortcode. Please follow the documentation for correct shortcode.', 'guest-support' ) . '</p>';
            return $errorMessage;
        }
        else
        {
            ob_start();
                require_once WPGS_PLUGIN_DIR . 'includes/process-shortcode.php';
            return ob_get_clean();
        }
    }
}

new Guest_Support();