<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;

class AdminPages
{
    public function __construct()
    {
        add_action( 'admin_menu', array($this,'_admin_menu') );
    }

    /**
     * Dashboard Menu and submenus
     */
    public function _admin_menu()
    {
        $paginationArgs = GuestSupportApp::paginationArgs();
        $openTickets = $paginationArgs !== false ? $paginationArgs->open : 0;
        $ispro = GuestSupportApp::isPro();

        add_menu_page(
          esc_html__( 'Guest Support', 'guest-support' ),
          esc_html__( 'G. Support','guest-support') . ( $openTickets ? ' <span class="wpgs-menu-tickets-count"><span>' . $openTickets . '</span></span>' : '' ),
          'manage_options',
          'guest-support',
          array($this,'tickets'),
          '',
          25
        );
        add_submenu_page(
            'guest-support',
            esc_html__( 'Ticket List', 'guest-support' ),
            esc_html__( 'All Tickets', 'guest-support' ),
            'manage_options',
            'guest-support',
            array($this,'tickets')
        );
        add_submenu_page(
            'guest-support',
            esc_html__( 'Departments - Guest Support', 'guest-support' ),
            esc_html__( 'Departments', 'guest-support' ),
            'manage_options',
            'guest-support-departments',
            array($this,'_router_departments')
        );
        add_submenu_page(
            'guest-support',
            esc_html__( 'Forms - Guest Support', 'guest-support' ),
            esc_html__( 'Forms', 'guest-support' ),
            'manage_options',
            'guest-support-forms',
            array($this,'_router_forms')
        );
        add_submenu_page(
            'guest-support',
            esc_html__( 'Settings - Guest Support', 'guest-support' ),
            esc_html__( 'Settings', 'guest-support' ),
            'manage_options',
            'guest-support-settings',
            array($this,'_settings')
        );
        add_submenu_page(
            'guest-support',
            esc_html__( 'Email Settings - Guest Support', 'guest-support' ),
            esc_html__( 'Email Settings', 'guest-support' ),
            'manage_options',
            'guest-support-email-settings',
            array($this,'_email_settings')
        );
		add_submenu_page(
            'guest-support',
            esc_html__( 'Email Templates - Guest Support', 'guest-support' ),
            esc_html__( 'Email Templates', 'guest-support' ),
            'manage_options',
            'guest-support-email-templates',
            array($this,'_email_templates')
        );
        add_submenu_page(
            'guest-support',
            esc_html__( 'Shortcodes - Guest Support', 'guest-support' ),
            esc_html__( 'Shortcodes', 'guest-support' ),
            'manage_options',
            'guest-support-shortcodes',
            array($this,'_shortcodes')
        );
        if ( $ispro )
        {
            add_submenu_page(
                'guest-support',
                esc_html__( 'Subscription API Key - Guest Support', 'guest-support' ),
                esc_html__( 'Subscription Key', 'guest-support' ),
                'manage_options',
                'guest-support-subscription-key',
                array($this,'_subscription_key')
            );
        }
		add_submenu_page(
            'guest-support',
            esc_html__( 'Documentation & Support - Guest Support', 'guest-support' ),
            esc_html__( 'Documentation & Support', 'guest-support' ),
            'manage_options',
            'guest-support-documentation',
            array($this,'_documentation')
        );
    }

    /**
     * Show list of tickets
     */
    public function tickets()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/tickets-router.php';
    }

    /**
     * Departments router
     */
    public function _router_departments()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/departments-router.php';
    }

    /**
     * Forms router
     */
    public function _router_forms()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/forms-router.php';
    }

    /**
     * Settings
     */
    public function _settings()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/settings.php';
    }

    /**
     * Email Settings
     */
    public function _email_settings()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/email-settings.php';
    }

	/**
     * Email templates
     */
    public function _email_templates()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/email-templates-router.php';
    }

    /**
     * Shortcodes
     */
    public function _shortcodes()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/shortcodes.php';
    }

    /**
     * PRO Addon Subscription API Key
     */
    public function _subscription_key()
    {
        require_once WPGS_PLUGIN_ADDON_DIR . 'includes/admin/subscription-key.php';
    }

	/**
     * Documentation and support
     */
    public function _documentation()
    {
        require_once WPGS_PLUGIN_DIR . 'includes/admin/documentation.php';
    }
}