<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$data = new GuestSupportDbquery();
$departments = $data->listDepartments();
$ispro = GuestSupportApp::isPro();
?>

<div class="wrap">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Support Departments - Guest Support', 'guest-support' ); ?></h1>
    <a href="<?php echo $ispro ? esc_url( admin_url( 'admin.php?page=guest-support-departments&type=add-new' ) ) : 'javascript:;'; ?>" class="page-title-action"<?php echo !$ispro ? ' id="wpgs_open_modal" data-modal="add_department_info"' : ''; ?>><?php esc_html_e( 'Add New', 'guest-support' ); ?></a>

    <hr class="wp-header-end">
    <?php echo wp_kses_post( GuestSupportApp::getNotice() ); ?>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>

    <div class="wt-grid">
        <div class="wt-block size-<?php echo $ispro ? '100' : '70'; ?>">
            <?php if ( !empty( $departments ) ) : ?>
                <table class="wp-list-table widefat fixed striped table-view-list departments">
                    <thead>
                        <tr>
                            <th scope="col" id="title" class="manage-column column-title"><?php esc_html_e( 'Department Name:', 'guest-support' ); ?></th>
                            <th scope="col" id="agent-name" class="manage-column column-agent-name wpgs-hide-small-mobile"><?php esc_html_e( 'Agent Name:', 'guest-support' ); ?></th>
                            <th scope="col" id="emails" class="manage-column column-emails wpgs-hide-mobile"><?php esc_html_e( 'Additional Emails:', 'guest-support' ); ?></th>
                            <th scope="col" id="emails-type" class="manage-column column-emails-type wpgs-hide-mobile"><?php esc_html_e( 'Additional Notifications Type:', 'guest-support' ); ?></th>
                        </tr>
                    </thead>
                    <tbody id="the-list">
                        <?php foreach ($departments as $item) : ?>
                            <tr id="dept-<?php echo esc_attr( $item->id ); ?>" class="dept-<?php echo esc_attr( $item->id ); ?> type-departments">
                                <td class="title column-title has-row-actions dept-title">
                                    <strong>
                                        <a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-departments&type=edit&dept_id=' . $item->id ) ); ?>" class="row-title"><?php echo esc_html( $item->name ); ?></a>
                                    </strong>
                                    <div class="row-actions">
                                        <span class="edit"><a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-departments&type=edit&dept_id=' . $item->id ) ); ?>"><?php esc_html_e( 'Edit', 'guest-support' ); ?></a> </span>
                                        <?php if ( $item->type != 'core' ) : ?>
                                            <span class="delete">
                                                | <a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-departments&wpgs_handler=delete_departments&type=delete&dept_id=' . $item->id ) ); ?>" id="wpgs-delete"><?php esc_html_e( 'Delete', 'guest-support' ); ?></a>
                                            </span>
                                        <?php endif; ?>
                                    </div>
                                </td>
                                <td class="wpgs-hide-small-mobile">
                                    <a href="<?php echo esc_url( admin_url( 'user-edit.php?user_id=' . $item->agent_id ) ); ?>">
                                        <?php
                                            $userdata = get_userdata($item->agent_id);
                                            echo esc_html( $userdata->display_name );
                                            echo '<div>' . esc_html( $userdata->user_email ) . '</div>';
                                        ?>
                                    </a>
                                </td>
                                <td class="wpgs-hide-mobile">
                                    <?php echo nl2br( esc_textarea( $item->additional_emails ) ); ?>
                                </td>
                                <td class="wpgs-hide-mobile">
                                    <?php
                                        $emails_type = $item->additional_emails_type;
                                        $show_type = '';
                                        if ( $emails_type == 'both' )
                                        {
                                            $show_type = esc_html__( 'Both new Tickets and Replies', 'guest-support' );
                                        }
                                        elseif ( $emails_type == 'tickets' )
                                        {
                                            $show_type = esc_html__( 'New Tickets only', 'guest-support' );
                                        }
                                        elseif ( $emails_type == 'replies' )
                                        {
                                            $show_type = esc_html__( 'Replies only', 'guest-support' );
                                        }
                                        echo esc_html( $show_type );
                                    ?>
                                </td>
                            </tr>
                        <?php endforeach; ?>
                    </tbody>
                    <tfoot>
                        <tr>
                            <th scope="col" id="title" class="manage-column column-title"><?php esc_html_e( 'Department Name:', 'guest-support' ); ?></th>
                            <th scope="col" id="agent-name" class="manage-column column-agent-name wpgs-hide-small-mobile"><?php esc_html_e( 'Agent Name:', 'guest-support' ); ?></th>
                            <th scope="col" id="emails" class="manage-column column-emails wpgs-hide-mobile"><?php esc_html_e( 'Additional Emails:', 'guest-support' ); ?></th>
                            <th scope="col" id="emails-type" class="manage-column column-emails-type wpgs-hide-mobile"><?php esc_html_e( 'Additional Notifications Type:', 'guest-support' ); ?></th>
                        </tr>
                    </tfoot>
                </table>
            <?php else : ?>
                <h4><?php esc_html_e( 'No department found.', 'guest-support' ); ?></h4>
            <?php endif; ?>
        </div>
        <?php if ( !$ispro ) : ?>
            <div class="wt-block size-5">
                <div class="wpgs-wp-heading-spacer">&nbsp;</div>
            </div>
            <div class="wt-block size-25">
                <div class="wpgs-addon-info">
                    <h3><?php esc_html_e( 'Need more departments?', 'guest-support' ); ?></h3>
                    <p><?php esc_html_e( 'You can add/edit/delete unlimited departments using the PRO addon.', 'guest-support' ); ?></p>
                    <p><a href="https://www.rcatheme.com/item/guest-support-addon-for-wordpress" target="_blank"><?php esc_html_e( 'View all PRO features or purchase here', 'guest-support' ); ?></a>.</p>
                </div>
            </div>
        <?php endif; ?>
    </div>
</div>
<!-- Edit field modal content -->
<div id="add_department_info" class="wpgs-modal">
    <div class="wpgs-modal-wrapper">
        <div class="wpgs-modal-container">
            <div class="wpgs-modal-container-wrapper">
                <div class="wpgs-modal-header">
                    <h3><?php esc_html_e( 'PRO addon required.', 'guest-support' ); ?></h3>
                </div>
                <div class="wpgs-modal-content">
                    <?php echo wp_kses( sprintf( __( 'You can add one Department in the free version. If you need more departments, %sbuy PRO addon here%s.', 'guest-support' ), '<a href="https://www.rcatheme.com/item/guest-support-addon-for-wordpress" target="_blank">', '</a>' ), GuestSupportApp::allowedKsesTags() ); ?>
                </div>
                <div class="wpgs-modal-footer">
                    <a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( 'Close', 'guest-support' ); ?></a>
                </div>
            </div>
        </div>
    </div>
</div>
<!-- END Edit field modal content -->
<!-- Setup guide -->
<?php
    $showguide = isset( $_GET['showguide'] ) ? (int) sanitize_text_field( $_GET['showguide'] ) : 0;
?>
<?php if ( $showguide ) : ?>
    <div id="modal_setup_guide_department" class="wpgs-modal wpgs-modal-active wpgs-setup-guide">
        <div class="wpgs-modal-wrapper">
            <div class="wpgs-modal-container">
                <div class="wpgs-modal-container-wrapper">
                    <div class="wpgs-modal-content">
                        <h4><?php esc_html_e( "I am your 'Guest Support' plugin setup guide, and I will guide you through all the necessary steps to configure this plugin.", "guest-support" ); ?></h4>
                        <div class="wpgs-divider"></div>
                        <h3><?php esc_html_e( "Setup Guide: Department", "guest-support" ); ?></h3>
                        <p><?php esc_html_e( "This is where you assign a support agent to an specific Department for receiving new Tickets and Replies email notifications. You could also add additional email addresses.", "guest-support" ); ?></p>
                        <p><?php _e( "Click on the default department <strong>Customer Support</strong> to get started.", "guest-support" ); ?></p>
                        <p><em><?php esc_html_e( "You must save the edited department to go to next step.", "guest-support" ); ?></em></p>
                        <p><a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( "Let's do it", "guest-support" ); ?></a></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
<?php endif; ?>
<!-- END Setup guide -->