<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$config = GuestSupportApp::config();
$ispro = GuestSupportApp::isPro();
// $mail = GuestSupportApp::sendEmail('srsifur@gmail.com', 'Testing', 'This is a test message');
?>
<div class="wrap">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Email Settings - Guest Support', 'guest-support' ); ?></h1>

    <hr class="wp-header-end">
    <?php echo wp_kses_post( GuestSupportApp::getNotice() ); ?>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>

    <div class="form-wrap wpgs-settings-form">
        <form id="wpgs_settings_form" method="post" action="" class="validate">
            <p class="submit">
                <input type="submit" class="button button-primary" value="<?php esc_html_e( 'Save Settings', 'guest-support' ) ?>">
            </p>

            <?php wp_nonce_field( '_wpnonce_settings_form_action', '_wpnonce_settings_form' ); ?>

            <div class="wpgs-settings-form-wrapper">
                <div class="wpgs-settings-form-content">
                    <div class="wpgs-settings-form-fields wt-grid">
                        <div class="wt-block size-50">
                            <div class="wpgs-settings-fields-block">
                                <ul class="wpgs-settings-fields-list">
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h2><?php esc_html_e( "Email Settings") ; ?></h2>
                                            <div class="form-field form-required">
                                                <label for="mailer"><?php esc_html_e( 'Mailer:', 'guest-support' ); ?></label>
                                                <select name="mailer" id="wpgs_select_mailer">
                                                    <option value="wp"<?php echo $config->mailer == 'wp' ? ' selected' : ''; ?>>WordPress Default</option>
                                                    <option value="smtp"<?php echo $config->mailer == 'smtp' ? ' selected' : ''; ?>>SMTP</option>
                                                </select>
                                            </div>
                                            <div class="form-field">
                                                <label for="mail_from_name"><?php esc_html_e( 'Email From Name:', 'guest-support' ); ?></label>
                                                <input name="mail_from_name" id="mail_from_name" type="text" value="<?php echo esc_attr( $config->mail_from_name ); ?>" size="40" placeholder="Example Site Name">
                                            </div>
                                            <div class="form-field">
                                                <label for="mail_from_email"><?php esc_html_e( 'Email From Address:', 'guest-support' ); ?></label>
                                                <input name="mail_from_email" id="mail_from_email" type="email" value="<?php echo esc_attr( $config->mail_from_email ); ?>" size="40" placeholder="noreply@example.com">
                                            </div>
                                            <div id="wpgs_settings_smtp_credentials" class="wpgs-smtp-credentials"<?php echo $config->mailer != 'smtp' ? ' style="display:none;"' : ''; ?>>
                                                <hr class="wpgs-settings-hr">
                                                <div class="form-field">
                                                    <label for="smtp_username"><?php esc_html_e( 'SMTP Username:', 'guest-support' ); ?></label>
                                                    <input name="smtp_username" id="smtp_username" type="text" value="<?php echo esc_attr( $config->smtp_username ); ?>" size="40">
                                                </div>
                                                <div class="form-field">
                                                    <label for="smtp_password"><?php esc_html_e( 'SMTP Password:', 'guest-support' ); ?></label>
                                                    <input name="smtp_password" id="smtp_password" type="password" value="<?php echo esc_attr( $config->smtp_password ); ?>" size="40">
                                                </div>
                                                <div class="form-field">
                                                    <label for="smtp_host"><?php esc_html_e( 'SMTP Host:', 'guest-support' ); ?></label>
                                                    <input name="smtp_host" id="smtp_host" type="text" value="<?php echo esc_attr( $config->smtp_host ); ?>" size="40" placeholder="smtp.gmail.com">
                                                </div>
                                                <div class="form-field">
                                                    <label for="smtp_port"><?php esc_html_e( 'SMTP Port:', 'guest-support' ); ?></label>
                                                    <input name="smtp_port" id="smtp_port" type="number" value="<?php echo esc_attr( $config->smtp_port ); ?>" size="40" placeholder="465">
                                                </div>
                                                <div class="form-field">
                                                    <label for="smtp_security"><?php esc_html_e( 'SMTP Security:', 'guest-support' ); ?></label>
                                                    <select name="smtp_security" id="smtp_security">
                                                        <option value="none"<?php echo $config->smtp_security == 'none' ? ' selected' : ''; ?>><?php esc_html_e( 'None', 'guest-support' ); ?></option>
                                                        <option value="ssl"<?php echo $config->smtp_security == 'ssl' ? ' selected' : ''; ?>><?php esc_html_e( 'SSL/TLS', 'guest-support' ); ?></option>
                                                        <option value="tls"<?php echo $config->smtp_security == 'tls' ? ' selected' : ''; ?>><?php esc_html_e( 'STARTTLS', 'guest-support' ); ?></option>
                                                    </select>
                                                </div>
                                            </div>
                                        </div>
                                    </li>
                                </ul>
                            </div>
                        </div>
                    </div>
                </div>
            </div>

            <p class="submit">
                <input type="hidden" name="wpgs_handler" value="wpgs_settings_form">
                <input type="hidden" name="wpgs_handler_type" value="wpgs_email_settings_form">
                <input type="submit" id="submit" class="button button-primary" value="<?php esc_html_e( 'Save Settings', 'guest-support' ) ?>">
            </p>
        </form>
    </div>
</div>

<!-- Setup guide -->
<?php
    $showguide = isset( $_GET['showguide'] ) ? (int) sanitize_text_field( $_GET['showguide'] ) : 0;
?>
<?php if ( $showguide ) : ?>
    <div id="modal_setup_guide_department" class="wpgs-modal wpgs-modal-active wpgs-setup-guide">
        <div class="wpgs-modal-wrapper">
            <div class="wpgs-modal-container">
                <div class="wpgs-modal-container-wrapper">
                    <div class="wpgs-modal-content">
                        <h4><?php esc_html_e( "I am your 'Guest Support' plugin setup guide, and I will guide you through all the necessary steps to configure this plugin.", "guest-support" ); ?></h4>
                        <div class="wpgs-divider"></div>
                        <h3><?php esc_html_e( "Setup Guide: Email Settings", "guest-support" ); ?></h3>
                        <p><?php esc_html_e( "This is where you update email settings and email templates for this plugin.", "guest-support" ); ?></p>
                        <p><em><?php _e( "You must click on <strong>Save Settings</strong> button to go to next step.", "guest-support" ); ?></em></p>
                        <p><a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( "Let's do it", "guest-support" ); ?></a></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
<?php endif; ?>
<!-- END Setup guide -->