<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$handler = '';
$title = '';
$submit_button = esc_html__('Save Template', 'guest-support');
$cancel_button = esc_html__('Close', 'guest-support');

if ( $type == 'edit' && $template_id )
{
    $handler = 'edit_email_template';
    $title = 'Edit Template: ' . $values['title'];
    $submit_button = esc_html__('Save Template', 'guest-support');
}

?>
<div class="wrap">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Email Templates', 'guest-support' ); ?></h1>

    <hr class="wp-header-end">
    <?php echo wp_kses_post( GuestSupportApp::getNotice() ); ?>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>

    <div class="form-wrap wpgs-templates-form">
        <h2><?php esc_html_e( $title, 'guest-support' ); ?></h2>
        <form id="wpgs_email_templates_form" method="post" action="" class="validate">
            <?php wp_nonce_field( '_wpnonce_email_template_form_action', '_wpnonce_email_template_form' ); ?>

			<div class="wt-grid">
				<div class="wt-block size-60">
					<div class="form-field form-required template-title-wrap">
						<label for="template-title"><?php esc_html_e( 'Template Title*', 'guest-support' ); ?></label>
						<input name="title" id="template-title" type="text" value="<?php echo esc_attr( $values['title'] ); ?>" size="40" placeholder="" aria-required="true" required>
					</div>

					<div class="form-field form-required template-subject-wrap">
						<label for="template-subject"><?php esc_html_e( 'Email Subject*', 'guest-support' ); ?></label>
						<input name="subject" id="template-subject" type="text" value="<?php echo esc_attr( $values['subject'] ); ?>" size="40" placeholder="" aria-required="true" required>
					</div>
					<div class="form-field">
						<label for="template"><?php esc_html_e( 'Email Message:*', 'guest-support' ); ?></label>
						<?php
							$content = stripcslashes( $values['template'] );
							$custom_editor_id = 'template';
							$custom_editor_name = 'template';
							$args = array(
								'media_buttons' => false,
								'textarea_name' => $custom_editor_name,
								'textarea_rows' => get_option('default_post_edit_rows', 10),
								'quicktags' => true
							);
							wp_editor( $content, $custom_editor_id, $args );
						?>
					</div>

					<div class="form-field form-required dept-name-wrap">
						<label for="template_active"><?php esc_html_e( 'Active:', 'guest-support' ); ?></label>
						<select name="active" id="template_active">
							<option value="1"<?php echo (int) $values['active'] === 1 ? ' selected' : '';?>><?php esc_html_e( 'Yes', 'guest-support' ); ?></option>
							<option value="0"<?php echo (int) $values['active'] === 0 ? ' selected' : '';?>><?php esc_html_e( 'No', 'guest-support' ); ?></option>
						</select>
						<p><?php esc_html_e( "If you don't want to send this type of emails, select 'No'.", "guest-support" ); ?></p>
					</div>

					<p class="submit">
						<input type="hidden" name="template_type" value="<?php echo $values['type']; ?>">
						<input type="hidden" name="wpgs_handler" value="<?php echo esc_attr( $handler ); ?>">
						<input type="submit" id="submit" class="button button-primary" value="<?php echo esc_attr( $submit_button ); ?>">
						<a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-email-templates' ) ); ?>" class="button button-cancel"><?php echo esc_attr( $cancel_button ); ?></a>
					</p>
				</div>
				<div class="wt-block size-5"></div>
				<div class="wt-block size-35">
					<div class="form-field wpgs-settings-placeholders">
						<h4><?php esc_html_e( 'Available Placeholders:', 'guest-support' ); ?></h4>
						<?php if ( $values['type'] === 'otp_email' ) : ?>
							<p><span>{email}</span> = <?php esc_html_e( 'Email Address of the Ticket Submitter', 'guest-support' ); ?></p>
							<p><span>{otp}</span> = <?php esc_html_e( 'The random and unique OTP will be sent.', 'guest-support' ); ?></p>
							<p><span>{sitename}</span> = <?php esc_html_e( "WordPress 'Site Title' from the General Settings.", 'guest-support' ); ?></p>
							<p><span>{siteurl}</span> = <?php esc_html_e( "WordPress 'Site Address (URL)' from the General Settings.", 'guest-support' ); ?></p>
						<?php elseif ( in_array( $values['type'], ['user_pre_ticket_close_message', 'agent_pre_ticket_close_message', 'ticket_close_message'] ) ) : ?>
							<p><span>{subject}</span> = <?php esc_html_e( 'Ticket Subject', 'guest-support' ); ?></p>
							<p><span>{ticket_id}</span> = <?php esc_html_e( 'Ticket ID', 'guest-support' ); ?></p>
							<p><span>{short_ticket_id}</span> = <?php esc_html_e( 'Short Ticket ID: A numeric ticket ID rather than the alphanumeric ticket ID.', 'guest-support' ); ?></p>
							<p><span>{ticket_link}</span> = <?php esc_html_e( 'Ticket Link Url', 'guest-support' ); ?></p>
							<p><span>{department}</span> = <?php esc_html_e( 'Department Name of the Submitted Ticket', 'guest-support' ); ?></p>
							<p><span>{form}</span> = <?php esc_html_e( 'Form Name of the Submitted Ticket', 'guest-support' ); ?></p>
							<p><span>{name}</span> = <?php esc_html_e( 'Full Name of the Ticket Submitter', 'guest-support' ); ?></p>
							<p><span>{email}</span> = <?php esc_html_e( 'Email Address of the Ticket Submitter', 'guest-support' ); ?></p>
							<p><span>{agent_name}</span> = <?php esc_html_e( 'Full Name of the Agent', 'guest-support' ); ?></p>
							<p><span>{agent_email}</span> = <?php esc_html_e( 'Email address of the Agent', 'guest-support' ); ?></p>
							<p><span>{sitename}</span> = <?php esc_html_e( "WordPress 'Site Title' from the General Settings.", 'guest-support' ); ?></p>
							<p><span>{siteurl}</span> = <?php esc_html_e( "WordPress 'Site Address (URL)' from the General Settings.", 'guest-support' ); ?></p>
						<?php else : ?>
							<p><span>{subject}</span> = <?php esc_html_e( 'Ticket Subject', 'guest-support' ); ?></p>
							<p><span>{message}</span> = <?php esc_html_e( 'Message of the New Ticket/Reply', 'guest-support' ); ?></p>
							<p><span>{ticket_id}</span> = <?php esc_html_e( 'Ticket ID', 'guest-support' ); ?></p>
							<p><span>{short_ticket_id}</span> = <?php esc_html_e( 'Short Ticket ID: A numeric ticket ID rather than the alphanumeric ticket ID.', 'guest-support' ); ?></p>
							<p><span>{ticket_link}</span> = <?php esc_html_e( 'Ticket Link Url', 'guest-support' ); ?></p>
							<p><span>{custom_fields}</span> = <?php esc_html_e( 'Custom Fields for the New Ticket/Reply', 'guest-support' ); ?></p>
							<p><span>{attachments}</span> = <?php esc_html_e( 'Links to Download Uploaded Files', 'guest-support' ); ?></p>
							<p><span>{department}</span> = <?php esc_html_e( 'Department Name of the Submitted Ticket', 'guest-support' ); ?></p>
							<p><span>{form}</span> = <?php esc_html_e( 'Form Name of the Submitted Ticket', 'guest-support' ); ?></p>
							<p><span>{name}</span> = <?php esc_html_e( 'Full Name of the Ticket Submitter', 'guest-support' ); ?></p>
							<p><span>{email}</span> = <?php esc_html_e( 'Email Address of the Ticket Submitter', 'guest-support' ); ?></p>
							<p><span>{is_registered_user}</span> = <?php esc_html_e( "Display 'Yes' or 'No'", 'guest-support' ); ?></p>
							<p><span>{user_id}</span> = <?php esc_html_e( "Display the WordPress user ID. Display 0 if the user is not registered.", 'guest-support' ); ?></p>
							<p><span>{ip_address}</span> = <?php esc_html_e( "Display the IP address of the ticket or reply submitter, whether it is a user or an agent.", 'guest-support' ); ?></p>
							<p><span>{agent_name}</span> = <?php esc_html_e( 'Full Name of the Agent', 'guest-support' ); ?></p>
							<p><span>{agent_email}</span> = <?php esc_html_e( 'Email address of the Agent', 'guest-support' ); ?></p>
							<p><span>{sitename}</span> = <?php esc_html_e( "WordPress 'Site Title' from the General Settings.", 'guest-support' ); ?></p>
							<p><span>{siteurl}</span> = <?php esc_html_e( "WordPress 'Site Address (URL)' from the General Settings.", 'guest-support' ); ?></p>
						<?php endif; ?>
					</div>
				</div>
			</div>
        </form>
    </div>
</div>