<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$db = new GuestSupportDbquery();
$templates = $db->listEmailTemplates();
$ispro = GuestSupportApp::isPro();
?>

<div class="wrap">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Email Templates - Guest Support', 'guest-support' ); ?></h1>

    <hr class="wp-header-end">
    <?php echo wp_kses_post( GuestSupportApp::getNotice() ); ?>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>

    <div class="wt-grid">
        <div class="wt-block size-100">
            <?php if ( !empty( $templates ) ) : ?>
                <table class="wp-list-table widefat fixed striped table-view-list templates">
                    <thead>
                        <tr>
                            <th scope="col" id="title" class="manage-column column-title"><?php esc_html_e( 'Template Title:', 'guest-support' ); ?></th>
                            <th scope="col" id="subject" class="manage-column column-subject wpgs-hide-mobile"><?php esc_html_e( 'Email Subject:', 'guest-support' ); ?></th>
                            <th scope="col" id="active" class="manage-column column-active"><?php esc_html_e( 'Active:', 'guest-support' ); ?></th>
                        </tr>
                    </thead>
                    <tbody id="the-list">
                        <?php foreach ($templates as $item) : ?>
                            <tr id="template-<?php echo esc_attr( $item->id ); ?>" class="template-<?php echo esc_attr( $item->id ); ?> type-templates">
                                <td class="title column-title has-row-actions template-title">
                                    <strong>
                                        <a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-email-templates&type=edit&template_id=' . $item->id ) ); ?>" class="row-title"><?php echo esc_html( $item->title ); ?></a>
                                    </strong>
                                    <div class="row-actions">
                                        <span class="edit"><a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-email-templates&type=edit&template_id=' . $item->id ) ); ?>"><?php esc_html_e( 'Edit', 'guest-support' ); ?></a> </span>
                                    </div>
                                </td>
                                <td>
									<?php echo esc_html( $item->subject ); ?>
                                </td>
                                <td>
									<?php echo (int) $item->active === 1 ? esc_html( 'Yes', 'guest-support' ) : esc_html( 'No', 'guest-support' ); ?>
                                </td>
                            </tr>
                        <?php endforeach; ?>
                    </tbody>
                    <tfoot>
						<tr>
                            <th scope="col" id="title" class="manage-column column-title"><?php esc_html_e( 'Template Title:', 'guest-support' ); ?></th>
                            <th scope="col" id="subject" class="manage-column column-subject wpgs-hide-mobile"><?php esc_html_e( 'Email Subject:', 'guest-support' ); ?></th>
                            <th scope="col" id="active" class="manage-column column-active"><?php esc_html_e( 'Active:', 'guest-support' ); ?></th>
                        </tr>
                    </tfoot>
                </table>
            <?php else : ?>
                <h4><?php esc_html_e( 'No template found.', 'guest-support' ); ?></h4>
            <?php endif; ?>
        </div>
    </div>
</div>
<!-- Setup guide -->
<?php
    $showguide = isset( $_GET['showguide'] ) ? (int) sanitize_text_field( $_GET['showguide'] ) : 0;
?>
<?php if ( $showguide ) : ?>
    <div id="modal_setup_guide_email_template" class="wpgs-modal wpgs-modal-active wpgs-setup-guide">
        <div class="wpgs-modal-wrapper">
            <div class="wpgs-modal-container">
                <div class="wpgs-modal-container-wrapper">
                    <div class="wpgs-modal-content">
                        <h4><?php esc_html_e( "I am your 'Guest Support' plugin setup guide, and I will guide you through all the necessary steps to configure this plugin.", "guest-support" ); ?></h4>
                        <div class="wpgs-divider"></div>
                        <h3><?php esc_html_e( "Setup Guide: Email Templates", "guest-support" ); ?></h3>
                        <p><?php esc_html_e( "All emails sent by this plugin use email templates listed on this page. You can customize them as needed.", "guest-support" ); ?></p>
                        <p><em><?php esc_html_e( "You must edit and save an email template to proceed to the next step.", "guest-support" ); ?></em></p>
                        <p><a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( "Let's do it", "guest-support" ); ?></a></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
<?php endif; ?>
<!-- END Setup guide -->