<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;

function customInputs($type)
{
    $label = '';
    $description = '';
    $placeholder_name = '';
    $placeholder_label = '';
    $placeholder_placeholder = '';

    switch ($type) {
        case 'text':
            $label = esc_html__( 'Textbox Name (Unique)*', 'guest-support' );
            $placeholder_name = 'product_name';
            $placeholder_label = 'Product Name';
            $placeholder_placeholder = 'Enter product name';
            break;
        case 'email':
            $label = esc_html__( 'Email Textbox Name (Unique)*', 'guest-support' );
            $placeholder_name = 'email';
            $placeholder_label = 'Email Address';
            $placeholder_placeholder = 'user@example.com';
            break;
        case 'number':
            $label = esc_html__( 'Number Field Name (Unique)*', 'guest-support' );
            $placeholder_name = 'code';
            $placeholder_label = 'Code Number';
            $placeholder_placeholder = '123456';
            break;
        case 'textarea':
            $label = esc_html__( 'Textarea Name (Unique)*', 'guest-support' );
            $placeholder_name = 'summery';
            $placeholder_label = 'Summery of your message';
            $placeholder_placeholder = 'Short info about your message';
            break;
        case 'select':
            $label = esc_html__( 'Select Name (Unique)*', 'guest-support' );
            $placeholder_name = 'colors';
            $placeholder_label = 'Choose a color';
            break;
        case 'radio':
            $label = esc_html__( 'Radio Group Name (Unique)*', 'guest-support' );
            $placeholder_name = 'gender';
            $placeholder_label = 'Gender';
            break;
        case 'checkbox':
            $label = esc_html__( 'Checkbox Group Name (Unique)*', 'guest-support' );
            $placeholder_name = 'buylist';
            $placeholder_label = 'I want to buy';
            break;
        case 'date':
            $label = esc_html__( 'Date Field Name (Unique)*', 'guest-support' );
            $placeholder_name = 'orderdate';
            $placeholder_label = 'Select order date';
            break;
        case 'datetime':
            $label = esc_html__( 'Date and Time Field Name (Unique)*', 'guest-support' );
            $placeholder_name = '';
            $placeholder_label = '';
            break;
        case 'departments':
            $placeholder_label = 'Department';
            break;

        default:
            $placeholder_name = '';
            $placeholder_label = '';
            $placeholder_placeholder = '';
            break;
    }

    $html = '<ul id="wpgs_field_type_' . $type . '" class="wpgs-field-contents">';

    if ( $type == 'hidden' )
    {
        // Name
        $label = esc_html__( 'Hidden Field Name*', 'guest-support' );
        $description = esc_html__( 'Lowercase and without spaces. This should be unique and should not match with another field name.', 'guest-support' );
        $html .= customInputsFields( 'text', 'name', $label, 'ticket_page', $description );

        // Label
        $label = esc_html__( 'Label', 'guest-support' );
        $placeholder_label = esc_html__( 'Ticket Page', 'guest-support' );
        $html .= customInputsFields( 'text', 'label', $label, $placeholder_label );

        // Value
        $label = esc_html__( 'Hidden Field Value*', 'guest-support' );
        $html .= customInputsFields( 'text', 'value', $label, 'Contact page' );
    }
    elseif ( $type == 'content' )
    {
        // Name
        $label = esc_html__( 'Content Field Name*', 'guest-support' );
        $description = esc_html__( 'Lowercase and without spaces. This should be unique and should not match with another field name.', 'guest-support' );
        $html .= customInputsFields( 'text', 'name', $label, 'privacy', $description );

        // Value
        $label = esc_html__( 'Add Content (Support HTML)*', 'guest-support' );
        $html .= customInputsFields( 'textarea', 'content', $label, 'Before submitting this ticket, please take a moment to read our <a href=&quot;privacy-policy&quot;>privacy policy here</a>.' );
    }
    elseif ( $type == 'departments' )
    {
        // Label
        $label = esc_html__( 'Label', 'guest-support' );
        $html .= customInputsFields( 'text', 'label', $label, $placeholder_label );

        // Show on form
        $options = array(
            '1' => esc_html__( 'Yes', 'guest-support' ),
            '0' => esc_html__( 'No', 'guest-support' ),
        );
        $label = esc_html__( 'Show on Form?', 'guest-support' );
        $html .= customInputsFields( 'real_radio', 'showonform_departments', $label, '', '', '', $options );


        // Show list of departments
        $label = esc_html__( 'Select Default Department', 'guest-support' );
        $description = esc_html__( 'The selected department will be used to assign an agent and send email notifications based on the department settings.', 'guest-support' );
        $html .= customInputsFields( 'real_departments', 'default_department', $label, '', $description );
    }
    else
    {
        // Name
        $description = esc_html__( 'Lowercase and without spaces. This should be unique and should not match with another field name.', 'guest-support' );
        $html .= customInputsFields( 'text', 'name', $label, $placeholder_name, $description, 'wpgs_field_list_name' );

        // Label
        $label = esc_html__( 'Label', 'guest-support' );
        $html .= customInputsFields( 'text', 'label', $label, $placeholder_label );

        // Placeholder
        if ( $type == 'text' || $type == 'email' || $type == 'number' || $type == 'textarea' )
        {
            $label = esc_html__( 'Placeholder (optional)', 'guest-support' );
            $html .= customInputsFields( 'text', 'placeholder', $label, $placeholder_placeholder );
        }

        if ( $type == 'select' )
        {
            $label = esc_html__( 'Enter Select OPTIONS', 'guest-support' );
            $placeholder = '--Select a color--&#10;Blue Color&#10;Red Color&#10;No extra color';
            $description = esc_html__( 'Enter each option on a new line.', 'guest-support' );
            $html .= customInputsFields( 'textarea', 'options', $label, $placeholder, $description );
        }

        if ( $type == 'radio' )
        {
            $label = esc_html__( 'Enter Radio OPTIONS', 'guest-support' );
            $placeholder = 'Male&#10;Female';
            $description = esc_html__( 'Enter each option on a new line.', 'guest-support' );
            $html .= customInputsFields( 'textarea', 'options', $label, $placeholder, $description );
        }

        if ( $type == 'checkbox' )
        {
            $label = esc_html__( 'Enter Checkbox OPTIONS', 'guest-support' );
            $placeholder = 'T-Shirt&#10;Sunglass&#10;Shoes&#10;Watch';
            $description = esc_html__( 'Enter each option on a new line.', 'guest-support' );
            $html .= customInputsFields( 'textarea', 'options', $label, $placeholder, $description );
        }

        // Add minimum and maximum option for number field
        if ( $type == 'number' )
        {
            // Minimum allowed
            $label = esc_html__( 'Allow Minimum Value', 'guest-support' );
            $html .= customInputsFields( 'number', 'min', $label, '1' );

            // Maximum allowed
            $label = esc_html__( 'Allow Maximum Value', 'guest-support' );
            $html .= customInputsFields( 'number', 'max', $label, '99' );
        }

        // Required?
        $html .= customInputsFields( 'required' );

        // Description
        $description = esc_html__( "If necessary, add a description for this field.", "guest-support" );
        $html .= customInputsFields( 'description', '', '', '', $description );

        // Error Message
        $description = esc_html__( "Error message to display when the user submits the ticket without filling in this field, if the 'Required setting above is set to 'Yes'.", "guest-support" );
        $html .= customInputsFields( 'error_message', '', '', '', $description );
    }

    // Input type
    $html .= '<li class="wpgs_field_list_input_type wpgs_field_list_nopadding">';
    $html .= '<input type="hidden" id="wpgs_this_field_input_type" value="' . $type . '">';
    $html .= '</li>';

    // Column size
    $label = esc_html__( "Column Size", "guest-support" );
    $html .= customInputsFields( 'colsize', 'colsize', $label );

    // Field to Form
    $label = esc_html__( "Add this field on", "guest-support" );
    $html .= customInputsFields( 'fieldtoform', 'fieldtoform', $label );

    if ( $type != 'content' )
    {
        // Show to users
        $label = esc_html__( "Display the label and the submitted value on the ticket view page.", "guest-support" );
        $html .= customInputsFields( 'showonticketview', 'showonticketview', $label );

        // Attach to email message
        $label = esc_html__( "Attach to email message", "guest-support" );
        $description = esc_html__( "It will only attach to the specific email if this field is attached to the specific (ticket/reply) form. See above 'Add this field on' option.", "guest-support" );
        $html .= customInputsFields( 'addtoemail', 'addtoemail', $label, '', $description );

		// Login required
        $label = esc_html__( "Login required?", "guest-support" );
        $description = esc_html__( "If set to 'Yes', the agent or the user needs to log in to view content. Otherwise, a message will be displayed saying: 'Login to view this message.' This feature is useful for collecting sensitive information, such as personal details or website access.", "guest-support" );
        $html .= customInputsFields( 'login_required', 'login_required', $label, '', $description );
    }

    $html .= '</ul>';
    return $html;
}
function customInputsFields($type = 'text', $for = '', $label = '', $placeholder = '', $description = '', $class = '', $options = array())
{
    $html = '';
    if ( $type == 'required' )
    {
        $for = 'required';
    }
    elseif ( $type == 'error_message' ) {
        $for = 'error_message';
    }
    $html .= '<li class="wpgs_field_list_' . $for . '">';
    if ( $type == 'text' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<input type="text" name="data[]" id="field_' . $for . '" size="40" placeholder="' . $placeholder . '">';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'number' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<input type="number" name="data[]" id="field_' . $for . '" size="40" placeholder="' . $placeholder . '">';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'textarea' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<textarea name="data[]" id="field_' . $for . '" placeholder="' . $placeholder . '" rows="5" cols="40"></textarea>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'required' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_required">' . esc_html__( 'Required?', 'guest-support' ) . '</label>';
        $html .= '<select name="data[]" id="field_required">';
        $html .= '<option value="1">' . esc_html__( 'Yes', 'guest-support' ) . '</option>';
        $html .= '<option value="0">' . esc_html__( 'No', 'guest-support' ) . '</option>';
        $html .= '</select>';
        $html .= '</div>';
    }
    elseif ( $type == 'description' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_description">' . esc_html__( 'Description (optional)', 'guest-support' ) . '</label>';
        $html .= '<textarea name="data[]" id="field_description" placeholder="' . esc_html__( 'Add description for this field if necessary. This will show on frontend.', 'guest-support' ) . '" rows="3" cols="40"></textarea>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'error_message' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_error_message">' . esc_html__( 'Error Message (optional)', 'guest-support' ) . '</label>';
        $html .= '<textarea name="data[]" id="field_error_message" placeholder="" rows="3" cols="40"></textarea>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'real_radio' )
    {
        $html .= '<div class="form-field wt-grid">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>&nbsp;&nbsp;&nbsp;';
        $html .= '<ul class="wpgs-field-checkbox">';
        if ( !empty( $options ) )
        {
            foreach ($options as $key => $value) {
                $html .= '<li><input type="radio" name="' . $for . '" id="' . $for . '_' . $key . '" value="' . $key . '"><label for="' . $for . '" class="wpgs-field-checkbox-label"> ' . $value . '</label></li>';
            }
        }
        $html .= '</ul>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'real_checkbox' )
    {
        $html .= '<div class="form-field wt-grid">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>&nbsp;&nbsp;&nbsp;';
        $html .= '<ul class="wpgs-field-checkbox">';
        if ( !empty( $options ) )
        {
            foreach ($options as $key => $value) {
                $html .= '<li><input type="checkbox" name="' . $for . '" id="' . $for . '_' . $key . '" value="' . $key . '"><label for="' . $for . '_' . $key . '" class="wpgs-field-checkbox-label"> ' . $value . '</label></li>';
            }
        }
        $html .= '</ul>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'real_departments' )
    {
		$form_id = isset( $_GET['form_id'] ) ? (int) $_GET['form_id'] : 0;
        $departments = GuestSupportApp::getFormDepartments( $form_id );
        $html .= '<div class="form-field list-departments" style="display:none;">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<select name="' . $for . '" id="field_' . $for . '">';
        if ( !empty( $departments ) )
        {
            foreach ($departments as $item) {
                $html .= '<option value="' . $item->id . '">' . $item->name . '</option>';
            }
        }
        $html .= '</select>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'colsize' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<select id="wpgs_field_list_colsize">';
        $html .= '<option value="100">100%</option>';
        $html .= '<option value="50">50%</option>';
        $html .= '<option value="33-3">33.333%</option>';
        $html .= '</select>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'fieldtoform' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<select id="wpgs_field_list_fieldtoform">';
        $html .= '<option value="ticket">' . esc_html__( 'Ticket Form', 'guest-support' ) . '</option>';
        $html .= '<option value="reply">' . esc_html__( 'Reply Form', 'guest-support' ) . '</option>';
        $html .= '<option value="both">' . esc_html__( 'Both Ticket and Reply Form', 'guest-support' ) . '</option>';
        $html .= '</select>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'showonticketview' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<select id="wpgs_field_list_showonticketview">';
        $html .= '<option value="hide">' . esc_html__( 'Hide', 'guest-support' ) . '</option>';
        $html .= '<option value="users">' . esc_html__( 'To Users only', 'guest-support' ) . '</option>';
        $html .= '<option value="agents">' . esc_html__( 'To Agents only', 'guest-support' ) . '</option>';
        $html .= '<option value="both">' . esc_html__( 'To both Users and Agents', 'guest-support' ) . '</option>';
        $html .= '</select>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    elseif ( $type == 'addtoemail' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<select id="wpgs_field_list_addtoemail">';
        $html .= '<option value="ticket">' . esc_html__( 'New ticket email only', 'guest-support' ) . '</option>';
        $html .= '<option value="reply">' . esc_html__( 'Ticket reply email only', 'guest-support' ) . '</option>';
        $html .= '<option value="both" selected>' . esc_html__( 'Both ticket and reply email', 'guest-support' ) . '</option>';
        $html .= '<option value="none">' . esc_html__( "No, Don't attach to email.", "guest-support" ) . '</option>';
        $html .= '</select>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
	elseif ( $type == 'login_required' )
    {
        $html .= '<div class="form-field">';
        $html .= '<label for="field_' . $for . '">' . $label . '</label>';
        $html .= '<select id="wpgs_field_list_login_required">';
        $html .= '<option value="0" selected>' . esc_html__( 'No', 'guest-support' ) . '</option>';
        $html .= '<option value="1">' . esc_html__( 'Yes', 'guest-support' ) . '</option>';
        $html .= '</select>';
        if ( $description )
        {
            $html .= '<p>' . $description . '</p>';
        }
        $html .= '</div>';
    }
    $html .= '</li>';
    return $html;
}
