<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$ispro = GuestSupportApp::isPro();
?>

<div class="wrap guest-support-forms">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Ticket Submissions Forms - Guest Support', 'guest-support' ); ?></h1>
    <a href="<?php echo $ispro ? esc_url( admin_url( 'admin.php?page=guest-support-forms&type=add-new' ) ) : 'javascript:;'; ?>" class="page-title-action"<?php echo !$ispro ? ' id="wpgs_open_modal" data-modal="add_form_info"' : ''; ?>><?php esc_html_e( 'Add New', 'guest-support' ); ?></a>

    <hr class="wp-header-end">
    <?php echo wp_kses_post( GuestSupportApp::getNotice() ); ?>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>
    <div class="wt-grid">
        <div class="wt-block size-<?php echo $ispro ? '100' : '70'; ?>">
            <?php if ( !empty( $forms ) ) : ?>
                <table class="wp-list-table widefat fixed striped table-view-list forms">
                    <thead>
                        <tr>
                            <th scope="col" id="form-id" class="manage-column column-id"><?php esc_html_e( 'Form Id:', 'guest-support' ); ?></th>
                            <th scope="col" id="form-name" class="manage-column column-title"><?php esc_html_e( 'Form Name:', 'guest-support' ); ?></th>
                            <th scope="col" id="open-tickets" class="manage-column column-emails-type wpgs-hide-mobile"><?php esc_html_e( "Open Tickets:", 'guest-support' ); ?></th>
                            <th scope="col" id="total-tickets" class="manage-column column-emails wpgs-hide-mobile"><?php esc_html_e( 'Total Tickets:', 'guest-support' ); ?></th>
                            <th scope="col" id="shortcode" class="manage-column column-agent-name wpgs-hide-small-mobile"><?php esc_html_e( 'Shortcode:', 'guest-support' ); ?></th>
                        </tr>
                    </thead>
                    <tbody id="the-list">
                        <?php if ( $forms && !empty( $forms ) ) : ?>
                            <?php foreach ( $forms as $form ) : ?>
                                <tr id="form-<?php echo esc_attr( $form->id ); ?>" class="form-<?php echo esc_attr( $form->id ); ?> type-form">
                                    <td class="form-id"><?php echo esc_html( $form->id ); ?></td>
                                    <td class="has-row-actions form-name">
                                        <strong>
                                            <a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-forms&type=edit&form_id=' . $form->id ) ); ?>" class="row-title"><?php echo esc_html( $form->form_name ); ?></a>    
                                        </strong>
                                        <div class="row-actions">
                                            <span class="edit"><a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-forms&type=edit&form_id=' . $form->id ) ); ?>"><?php esc_html_e( 'Edit', 'guest-support' ); ?></a></span>
                                            <?php if ( $form->form_type != 'core' ) : ?>
                                                <span class="delete">
                                                    | <a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support-forms&wpgs_handler=delete_form&type=delete&form_id=' . $form->id ) ); ?>" id="wpgs-delete"><?php esc_html_e( 'Delete', 'guest-support' ); ?></a>
                                                </span>
                                            <?php endif; ?>
                                        </div>
                                    </td>
                                    <td class="open-tickets wpgs-hide-small-mobile">
                                        <?php echo esc_html( $form->open_tickets ); ?>
                                    </td>
                                    <td class="total-tickets wpgs-hide-mobile">
                                        <?php echo esc_html( $form->total_tickets ); ?>
                                    </td>
                                    <td class="shortcode wpgs-hide-mobile">
                                        [guest-support view=form id=<?php echo esc_attr( $form->id ); ?>]
                                    </td>
                                </tr>
                            <?php endforeach; ?>
                        <?php endif; ?>
                    </tbody>
                    <tfoot>
                        <tr>
                            <th scope="col" id="form-id" class="manage-column column-id"><?php esc_html_e( 'Form Id:', 'guest-support' ); ?></th>
                            <th scope="col" id="form-name" class="manage-column column-title"><?php esc_html_e( 'Form Name:', 'guest-support' ); ?></th>
                            <th scope="col" id="open-tickets" class="manage-column column-emails-type wpgs-hide-mobile"><?php esc_html_e( "Open Tickets:", 'guest-support' ); ?></th>
                            <th scope="col" id="total-tickets" class="manage-column column-emails wpgs-hide-mobile"><?php esc_html_e( 'Total Tickets:', 'guest-support' ); ?></th>
                            <th scope="col" id="shortcode" class="manage-column column-agent-name wpgs-hide-small-mobile"><?php esc_html_e( 'Shortcode:', 'guest-support' ); ?></th>
                        </tr>
                    </tfoot>
                </table>
            <?php else : ?>
                <h4><?php esc_html_e( 'No form found.', 'guest-support' ); ?></h4>
            <?php endif; ?>
        </div>
        <?php if ( !$ispro ) : ?>
            <div class="wt-block size-5">
                <div class="wpgs-wp-heading-spacer">&nbsp;</div>
            </div>
            <div class="wt-block size-25">
                <div class="wpgs-addon-info">
                    <h3><?php esc_html_e( 'Need more forms?', 'guest-support' ); ?></h3>
                    <p><?php esc_html_e( 'You can add/edit/delete unlimited forms using the PRO addon.', 'guest-support' ); ?></p>
                    <p><a href="https://www.rcatheme.com/item/guest-support-addon-for-wordpress" target="_blank"><?php esc_html_e( 'View all PRO features or purchase here', 'guest-support' ); ?></a>.</p>
                </div>
            </div>
        <?php endif; ?>
    </div>
</div>
<!-- Edit field modal content -->
<div id="add_form_info" class="wpgs-modal">
    <div class="wpgs-modal-wrapper">
        <div class="wpgs-modal-container">
            <div class="wpgs-modal-container-wrapper">
                <div class="wpgs-modal-header">
                    <h3><?php esc_html_e( 'PRO addon required.', 'guest-support' ); ?></h3>
                </div>
                <div class="wpgs-modal-content">
                    <?php echo wp_kses( sprintf( __( 'You can add one Form in free version. If you need more forms with unlimited form fields, %sbuy PRO addon here%s.', 'guest-support' ), '<a href="https://www.rcatheme.com/item/guest-support-addon-for-wordpress" target="_blank">', '</a>' ), GuestSupportApp::allowedKsesTags() ); ?>
                </div>
                <div class="wpgs-modal-footer">
                    <a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( 'Close', 'guest-support' ); ?></a>
                </div>
            </div>
        </div>
    </div>
</div>
<!-- END Edit field modal content -->
<!-- Setup guide -->
<?php
    $showguide = isset( $_GET['showguide'] ) ? (int) sanitize_text_field( $_GET['showguide'] ) : 0;
?>
<?php if ( $showguide ) : ?>
    <div id="modal_setup_guide_department" class="wpgs-modal wpgs-modal-active wpgs-setup-guide">
        <div class="wpgs-modal-wrapper">
            <div class="wpgs-modal-container">
                <div class="wpgs-modal-container-wrapper">
                    <div class="wpgs-modal-content">
                        <h4><?php esc_html_e( "I am your 'Guest Support' plugin setup guide, and I will guide you through all the necessary steps to configure this plugin.", "guest-support" ); ?></h4>
                        <div class="wpgs-divider"></div>
                        <h3><?php esc_html_e( "Setup Guide: Ticket Submissions Forms", "guest-support" ); ?></h3>
                        <p><?php esc_html_e( "This is where you add/edit/show/hide/re-order ticket form fields and other ticket submission page options.", "guest-support" ); ?></p>
                        <p><?php _e( "Click on the <strong>Default Form</strong> to get started.", "guest-support" ); ?></p>
                        <p><em><?php esc_html_e( "You must save the form (with/without modification) to go to next step.", "guest-support" ); ?></em></p>
                        <p><a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( "Let's do it", "guest-support" ); ?></a></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
<?php endif; ?>
<!-- END Setup guide -->