<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$config = GuestSupportApp::config();
$ispro = GuestSupportApp::isPro();
?>
<div class="wrap">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Settings - Guest Support', 'guest-support' ); ?></h1>

    <hr class="wp-header-end">
    <?php echo wp_kses_post( GuestSupportApp::getNotice() ); ?>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>

    <div class="form-wrap wpgs-settings-form">
        <form id="wpgs_settings_form" method="post" action="" class="validate">
            <p class="submit">
                <input type="submit" class="button button-primary" value="<?php esc_html_e( 'Save Settings', 'guest-support' ) ?>">
            </p>
            <?php wp_nonce_field( '_wpnonce_settings_form_action', '_wpnonce_settings_form' ); ?>

            <div class="wpgs-settings-form-wrapper">
                <div class="wpgs-settings-form-content">
                    <div class="wpgs-settings-form-fields wt-grid">
                        <div class="wt-block size-50">
                            <div class="wpgs-settings-fields-block">
                                <ul class="wpgs-settings-fields-list">
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "reCAPTCHA Version 3 Options", 'guest-support' ) ; ?></h3>
                                            <div class="form-field form-required">
                                                <label for="recaptcha_site_key"><?php esc_html_e( 'reCAPTCHA Site Key:', 'guest-support' ); ?></label>
                                                <input name="recaptcha_site_key" id="recaptcha_site_key" type="text" value="<?php echo esc_attr( $config->recaptcha_site_key ); ?>" size="40">
                                            </div>
                                            <div class="form-field">
                                                <label for="recaptcha_secret_key"><?php esc_html_e( 'reCAPTCHA Secret Key:', 'guest-support' ); ?></label>
                                                <input name="recaptcha_secret_key" id="recaptcha_secret_key" type="text" value="<?php echo esc_attr( $config->recaptcha_secret_key ); ?>" size="40">
                                            </div>
                                            <div class="form-field">
                                                <label for="recaptcha_score"><?php esc_html_e( 'Allow Minimum SPAM Score (0.0 - 1):', 'guest-support' ); ?></label>
                                                <input name="recaptcha_score" id="recaptcha_score" type="number" min="0" max="1" step="0.1" value="<?php echo esc_attr( $config->recaptcha_score ); ?>" size="40">
                                            </div>
                                        </div>
                                    </li>
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Knowledge Base/Documentation Suggestions Options", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <?php if ( !$ispro ) : ?>
                                                    <p class="wpgs-color-red"><strong><?php esc_html_e( 'This feature requires the PRO Addon to work.', 'guest-support' ); ?></strong></p>
                                                <?php endif; ?>
                                                <label for="kbs_ignore_words"><?php esc_html_e( 'Ignore Words (separated by comma):', 'guest-support' ); ?></label>
                                                <textarea name="kbs_ignore_words" id="kbs_ignore_words" class="wpgs-textarea" rows="3" cols="40"><?php echo esc_html( $config->kbs_ignore_words ); ?></textarea>
                                                <p><?php esc_html_e( "If you add 'how,to' and user type 'how to order' on Subject field, the system will search on Posts by the search string 'order' instead of 'how', 'to' and 'order'.", 'guest-support' ); ?></p>
                                            </div>
                                        </div>
                                    </li>
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Re-Open Closed Ticket Options", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <ul class="wpgs-field-checkbox wpgs-field-checkbox-block" >
                                                    <li><input type="radio" name="reopen_closed_ticket" id="reopen_closed_ticket_no" value="no"<?php echo $config->reopen_closed_ticket == 'no' ? ' checked' : ''; ?>><label for="reopen_closed_ticket_no" class="wpgs-field-checkbox-label"> <?php esc_html_e( 'Can\'t re-open closed ticket', 'guest-support' ); ?></label></li>
                                                    <li><input type="radio" name="reopen_closed_ticket" id="reopen_closed_ticket_agent" value="agent"<?php echo $config->reopen_closed_ticket == 'agent' ? ' checked' : ''; ?>><label for="reopen_closed_ticket_agent" class="wpgs-field-checkbox-label"> <?php esc_html_e( 'Agent can re-open closed ticket', 'guest-support' ); ?></label></li>
                                                    <li><input type="radio" name="reopen_closed_ticket" id="reopen_closed_ticket_user" value="user"<?php echo $config->reopen_closed_ticket == 'user' ? ' checked' : ''; ?>><label for="reopen_closed_ticket_user" class="wpgs-field-checkbox-label"> <?php esc_html_e( 'User can re-open closed ticket', 'guest-support' ); ?></label></li>
                                                    <li><input type="radio" name="reopen_closed_ticket" id="reopen_closed_ticket_both" value="both"<?php echo $config->reopen_closed_ticket == 'both' ? ' checked' : ''; ?>><label for="reopen_closed_ticket_both" class="wpgs-field-checkbox-label"> <?php esc_html_e( 'Both Agent and User can re-open closed ticket', 'guest-support' ); ?></label></li>
                                                </ul>
                                            </div>
                                        </div>
                                    </li>
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Create Ticket URL", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <input type="text" name="create_ticket_url" id="create_ticket_url" value="<?php echo esc_attr( $config->create_ticket_url ); ?>" placeholder="https://example.com/submit-ticket" >
                                                <p><?php esc_html_e( 'This URL will be used to display a Create Ticket button on the frontend Ticket Listing page on PRO Addon.', 'guest-support' ); ?></p>
                                            </div>
                                        </div>
                                    </li>
									<li>
                                        <div class="wpgs-settings-fields-content">
											<?php if ( !$ispro ) : ?>
												<p class="wpgs-color-red"><strong><?php esc_html_e( 'This feature requires the PRO Addon to work.', 'guest-support' ); ?></strong></p>
											<?php endif; ?>
                                            <h3><?php esc_html_e( "Auto close tickets after", 'guest-support' ) ; ?> <span><?php esc_html_e( "(days after the last reply to the ticket)", 'guest-support' ) ; ?></span>:</h3>
                                            <div class="form-field">
												<div class="wpgs-form-field-group">
													<input type="number" name="auto_close_tickets" id="auto_close_tickets" value="<?php echo esc_attr( $config->auto_close_tickets ); ?>">
													<span>Days</span>
												</div>
												<p><?php esc_html_e( "Add 0 to disable. The ticket submitter/user will receive an email notification after the ticket is closed.", "guest-support" ); ?></p>
											</div>
											<h3><?php esc_html_e( "Send an email notification before closing tickets", 'guest-support' ) ; ?><span><?php esc_html_e( "(days after the last reply to the ticket)", 'guest-support' ) ; ?></span>:</h3>
                                            <div class="form-field">
												<div class="wpgs-form-field-group">
													<input type="number" name="pre_close_email" id="pre_close_email" value="<?php echo esc_attr( $config->pre_close_email ); ?>">
													<span>Days</span>
												</div>
												<p><?php esc_html_e( 'Add 0 to disable. Depending on the ticket status, either the Agent or the User will receive an email notification before the ticket is closed.', 'guest-support' ); ?></p>
                                            </div>
											<div class="form-field">
												<div><?php _e( '<strong>Note</strong>: This automation uses the WordPress core cron system, and the WordPress core cron executes when there is a visit to your website. For more precise automation, please refer to', 'guest-support' ); ?> <a href="https://www.rcatheme.com/docs/wordpress-plugins/guest-support-for-wordpress/setup-cron-job-to-send-pre-close-email-notifications-and-auto-close-tickets" target="_blank"><?php esc_html_e( 'this documentation', 'guest-support' ); ?></a>.</div>
											</div>
                                        </div>
                                    </li>
									<li>
                                        <div class="wpgs-settings-fields-content">
											<?php if ( !$ispro ) : ?>
												<p class="wpgs-color-red"><strong><?php esc_html_e( 'This feature requires the PRO Addon to work.', 'guest-support' ); ?></strong></p>
											<?php endif; ?>
                                            <h3><?php esc_html_e( "Login and Registration Page", 'guest-support' ) ; ?> <span><?php esc_html_e( "(Displays on the list of tickets page for non-logged-in users.)", 'guest-support' ) ; ?></span>:</h3>
                                            <div class="form-field">
                                                <h4><?php esc_html_e( 'Display login form:', 'guest-support' ); ?></h4>
                                                <select name="display_login_form">
                                                    <option value="yes"<?php echo $config->display_login_form == 'yes' ? ' selected' : ''; ?>><?php esc_html_e( 'Yes', 'guest-support' ); ?></option>
                                                    <option value="no"<?php echo $config->display_login_form == 'no' ? ' selected' : ''; ?>><?php esc_html_e( 'No', 'guest-support' ); ?></option>
                                                </select>
                                                <h4><?php esc_html_e( 'Display registration form:', 'guest-support' ); ?></h4>
                                                <select name="display_registration_form">
                                                    <option value="yes"<?php echo $config->display_registration_form == 'yes' ? ' selected' : ''; ?>><?php esc_html_e( 'Yes', 'guest-support' ); ?></option>
                                                    <option value="no"<?php echo $config->display_registration_form == 'no' ? ' selected' : ''; ?>><?php esc_html_e( 'No', 'guest-support' ); ?></option>
                                                </select>
												<label for="login_page_message"><h4><?php esc_html_e( 'Message to display when both the login and registration forms are disabled:', 'guest-support' ); ?></h4></label>
                                                <textarea name="login_page_message" id="login_page_message" class="wpgs-textarea" rows="3" cols="40"><?php echo esc_html( $config->login_page_message ); ?></textarea>
                                            </div>
                                        </div>
                                    </li>
                                </ul>
                            </div>
                        </div>
                        <div class="wt-block size-50">
                            <div class="wpgs-settings-fields-block">
                                <ul class="wpgs-settings-fields-list">
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Delete Tickets And Messages Options", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <h4><?php esc_html_e( 'Can Agents Delete Tickets?', 'guest-support' ); ?></h4>
                                                <select name="can_delete_tickets">
                                                    <option value="yes"<?php echo $config->can_delete_tickets == 'yes' ? ' selected' : ''; ?>><?php esc_html_e( 'Yes', 'guest-support' ); ?></option>
                                                    <option value="no"<?php echo $config->can_delete_tickets == 'no' ? ' selected' : ''; ?>><?php esc_html_e( 'No', 'guest-support' ); ?></option>
                                                </select>
                                                <h4><?php esc_html_e( 'Can Agents Delete Ticket Replies?', 'guest-support' ); ?></h4>
                                                <select name="can_delete_replies">
                                                    <option value="yes"<?php echo $config->can_delete_replies == 'yes' ? ' selected' : ''; ?>><?php esc_html_e( 'Yes', 'guest-support' ); ?></option>
                                                    <option value="no"<?php echo $config->can_delete_replies == 'no' ? ' selected' : ''; ?>><?php esc_html_e( 'No', 'guest-support' ); ?></option>
                                                </select>
                                                <p><?php esc_html_e( '(WordPress Admins still can Delete if set to No)', 'guest-support' ); ?></p>
                                            </div>
                                        </div>
                                    </li>
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Edit Ticket Messages Options", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <h4><?php esc_html_e( 'Who Can Edit Ticket Replies', 'guest-support' ); ?></h4>
                                                <select name="can_edit_replies">
                                                    <option value="none"<?php echo $config->can_edit_replies == 'none' ? ' selected' : ''; ?>><?php esc_html_e( 'No One (disable)', 'guest-support' ); ?></option>
                                                    <option value="agent"<?php echo $config->can_edit_replies == 'agent' ? ' selected' : ''; ?>><?php esc_html_e( 'Only Agents', 'guest-support' ); ?></option>
                                                    <option value="user"<?php echo $config->can_edit_replies == 'user' ? ' selected' : ''; ?>><?php esc_html_e( 'Only Users', 'guest-support' ); ?></option>
                                                    <option value="both"<?php echo $config->can_edit_replies == 'both' ? ' selected' : ''; ?>><?php esc_html_e( 'Both Agents and Users', 'guest-support' ); ?></option>
                                                </select>
                                                <h4><?php esc_html_e( 'Can Edit', 'guest-support' ); ?></h4>
                                                <select name="edit_reply_type">
                                                    <option value="all"<?php echo $config->edit_reply_type == 'all' ? ' selected' : ''; ?>><?php esc_html_e( 'All Replies', 'guest-support' ); ?></option>
                                                    <option value="last"<?php echo $config->edit_reply_type == 'last' ? ' selected' : ''; ?>><?php esc_html_e( 'Own Last Reply Only', 'guest-support' ); ?></option>
                                                </select>
                                                <h4><?php esc_html_e( 'Can Agents Globally Edit All Replies? (WordPress Admins still can edit if set to No)', 'guest-support' ); ?></h4>
                                                <select name="edit_replies_globally">
                                                    <option value="yes"<?php echo $config->edit_replies_globally == 'yes' ? ' selected' : ''; ?>><?php esc_html_e( 'Yes', 'guest-support' ); ?></option>
                                                    <option value="no"<?php echo $config->edit_replies_globally == 'no' ? ' selected' : ''; ?>><?php esc_html_e( 'No', 'guest-support' ); ?></option>
                                                </select>
                                            </div>
                                        </div>
                                    </li>
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Default Signature", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <h4><?php esc_html_e( 'Enable default signature for', 'guest-support' ); ?></h4>
                                                <select name="enable_signature">
                                                    <option value="both"<?php echo $config->enable_signature == 'both' ? ' selected' : ''; ?>><?php esc_html_e( 'Both Agents and Users', 'guest-support' ); ?></option>
                                                    <option value="agent"<?php echo $config->enable_signature == 'agent' ? ' selected' : ''; ?>><?php esc_html_e( 'Agents Only', 'guest-support' ); ?></option>
                                                    <option value="user"<?php echo $config->enable_signature == 'user' ? ' selected' : ''; ?>><?php esc_html_e( 'Users Only', 'guest-support' ); ?></option>
                                                    <option value="none"<?php echo $config->enable_signature == 'none' ? ' selected' : ''; ?>><?php esc_html_e( 'Disable', 'guest-support' ); ?></option>
                                                </select>
                                                <p><?php esc_html_e( 'If enabled, Agents and Users will be able to set default signature from frontend tickets list page.', 'guest-support' ); ?></p>
                                            </div>
                                        </div>
                                    </li>
                                    <li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Number of Tickets Per Page", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <input type="number" name="tickets_per_page" id="tickets_per_page" value="<?php echo esc_attr( $config->tickets_per_page ); ?>">
                                            </div>
                                        </div>
                                    </li>
									<li>
                                        <div class="wpgs-settings-fields-content">
                                            <h3><?php esc_html_e( "Actions After Ticket Creation:", 'guest-support' ) ; ?></h3>
                                            <div class="form-field">
                                                <h4><?php esc_html_e( 'After New Ticket Created:', 'guest-support' ); ?></h4>
                                                <select name="after_new_ticket_created">
                                                    <option value="redirect"<?php echo $config->after_new_ticket_created == 'redirect' ? ' selected' : ''; ?>><?php esc_html_e( 'Redirect to Ticket View Page', 'guest-support' ); ?></option>
                                                    <option value="show_message"<?php echo $config->after_new_ticket_created == 'show_message' ? ' selected' : ''; ?>><?php esc_html_e( 'Show Confirmation Message', 'guest-support' ); ?></option>
                                                </select>
                                            </div>
											<h3><?php esc_html_e( "New Ticket Confirmation Message:", 'guest-support' ); ?></h3>
											<p><?php esc_html_e( "Applicable if you have selected 'Show Confirmation Message' above.", 'guest-support' ); ?></p>
											<div class="form-field wpgs-settings-placeholders">
											<h4><?php esc_html_e( 'Available Placeholders:', 'guest-support' ); ?></h4>
												<p><span>{subject}</span> = <?php esc_html_e( 'Ticket Subject', 'guest-support' ); ?></p>
												<p><span>{ticket_id}</span> = <?php esc_html_e( 'Ticket ID', 'guest-support' ); ?></p>
												<p><span>{short_ticket_id}</span> = <?php esc_html_e( 'Short Ticket ID: A numeric ticket ID rather than the alphanumeric ticket ID.', 'guest-support' ); ?></p>
                                                <p><span>{ticket_link}</span> = <?php esc_html_e( 'Ticket Link Url', 'guest-support' ); ?></p>
                                                <p><span>{department}</span> = <?php esc_html_e( 'Department Name of the Submitted Ticket', 'guest-support' ); ?></p>
												<p><span>{form}</span> = <?php esc_html_e( 'Form Name of the Submitted Ticket', 'guest-support' ); ?></p>
                                                <p><span>{name}</span> = <?php esc_html_e( 'Full Name of the Ticket Submitter', 'guest-support' ); ?></p>
                                                <p><span>{email}</span> = <?php esc_html_e( 'Email Address of the Ticket Submitter', 'guest-support' ); ?></p>
                                                <p><span>{is_registered_user}</span> = <?php esc_html_e( "Display 'Yes' or 'No'", 'guest-support' ); ?></p>
                                                <p><span>{user_id}</span> = <?php esc_html_e( "Display the WordPress user ID. Display 0 if the user is not registered.", 'guest-support' ); ?></p>
                                                <p><span>{ip_address}</span> = <?php esc_html_e( "Display the IP address of the ticket or reply submitter, whether it is a user or an agent.", 'guest-support' ); ?></p>
                                                <p><span>{agent_name}</span> = <?php esc_html_e( 'Full Name of the Agent', 'guest-support' ); ?></p>
                                                <p><span>{agent_email}</span> = <?php esc_html_e( 'Email address of the Agent', 'guest-support' ); ?></p>
                                                <p><span>{sitename}</span> = <?php esc_html_e( "WordPress 'Site Title' from the General Settings.", 'guest-support' ); ?></p>
                                                <p><span>{siteurl}</span> = <?php esc_html_e( "WordPress 'Site Address (URL)' from the General Settings.", 'guest-support' ); ?></p>
                                            </div>
                                            <div class="form-field">
                                                <?php
                                                    $content = stripcslashes( $config->new_ticket_confirmation_message );
                                                    $custom_editor_id = 'new_ticket_confirmation_message';
                                                    $custom_editor_name = 'new_ticket_confirmation_message';
                                                    $args = array(
                                                        'media_buttons' => false,
                                                        'textarea_name' => $custom_editor_name,
                                                        'textarea_rows' => get_option('default_post_edit_rows', 10),
                                                        'quicktags' => true
                                                    );
                                                    wp_editor( $content, $custom_editor_id, $args );
                                                ?>
                                            </div>
                                        </div>
                                    </li>
                                </ul>
                            </div>
                        </div>
                    </div>
                </div>
            </div>

            <p class="submit">
                <input type="hidden" name="wpgs_handler" value="wpgs_settings_form">
                <input type="submit" id="submit" class="button button-primary" value="<?php esc_html_e( 'Save Settings', 'guest-support' ) ?>">
            </p>
        </form>
    </div>
</div>

<!-- Setup guide -->
<?php
    $showguide = isset( $_GET['showguide'] ) ? (int) sanitize_text_field( $_GET['showguide'] ) : 0;
?>
<?php if ( $showguide ) : ?>
    <div id="modal_setup_guide_department" class="wpgs-modal wpgs-modal-active wpgs-setup-guide">
        <div class="wpgs-modal-wrapper">
            <div class="wpgs-modal-container">
                <div class="wpgs-modal-container-wrapper">
                    <div class="wpgs-modal-content">
                        <h4><?php esc_html_e( "I am your 'Guest Support' plugin setup guide, and I will guide you through all the necessary steps to configure this plugin.", "guest-support" ); ?></h4>
                        <div class="wpgs-divider"></div>
                        <h3><?php esc_html_e( "Setup Guide: Settings", "guest-support" ); ?></h3>
                        <p><?php esc_html_e( "Here you will be able to update global settings for this plugin.", "guest-support" ); ?></p>
                        <p><em><?php _e( "You must click on <strong>Save Settings</strong> button to go to next step.", "guest-support" ); ?></em></p>
                        <p><a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( "Let's do it", "guest-support" ); ?></a></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
<?php endif; ?>
<!-- END Setup guide -->