<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$default_form_id = GuestSupportApp::getDefaultFormId();
?>
<div class="wrap">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'Shortcodes - Guest Support', 'guest-support' ); ?></h1>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>

    <div class="form-wrap wpgs-shortcodes">
        <div class="wpgs-settings-fields-content">
            <h3><?php esc_html_e( 'Submit Ticket', 'guest-support' ); ?></h3>
            <div class="wpgs-shortcode-item">[guest-support view=form id=<?php echo esc_attr( $default_form_id ); ?>]</div>
            <p>
                <?php
                    echo wp_kses( sprintf( __( 'Change id with the Form Id. You can get Form Id from %shere%s.', 'guest-support' ), '<a href="' . esc_url( admin_url( 'admin.php?page=guest-support-forms' ) ) . '">', '</a>' ), GuestSupportApp::allowedKsesTags() );
                ?>
            </p>
            <div class="wpgs-divider"></div>
            <h3><?php esc_html_e( 'View Single Ticket', 'guest-support' ); ?></h3>
            <div class="wpgs-shortcode-item">[guest-support view=ticket]</div>
            <div class="wpgs-divider"></div>
            <h3><?php esc_html_e( 'View List of Tickets on Frontend', 'guest-support' ); ?></h3>
            <div class="wpgs-shortcode-item">[guest-support view=tickets]</div>
        </div>
    </div>
</div>

<!-- Setup guide -->
<?php
    $showguide = isset( $_GET['showguide'] ) ? (int) sanitize_text_field( $_GET['showguide'] ) : 0;
    $shortcode = isset( $_GET['shortcode'] ) ? sanitize_text_field( $_GET['shortcode'] ) : 'form';
?>
<?php if ( $showguide ) : ?>
    <div id="modal_setup_guide_department" class="wpgs-modal wpgs-modal-active wpgs-setup-guide">
        <div class="wpgs-modal-wrapper">
            <div class="wpgs-modal-container">
                <div class="wpgs-modal-container-wrapper">
                    <div class="wpgs-modal-content">
                        <h4><?php esc_html_e( "I am your 'Guest Support' plugin setup guide, and I will guide you through all the necessary steps to configure this plugin.", "guest-support" ); ?></h4>
                        <div class="wpgs-divider"></div>
                        <h3><?php esc_html_e( "Setup Guide: Shortcode", "guest-support" ); ?></h3>
                        <p>
                            <?php
                                if ( $shortcode == 'form' )
                                {
                                    echo '<strong>' . esc_html__( 'Create Ticket Submissions Form Page:', "guest-support" ) . '</strong>';
                                    echo '<br><br>';
                                    echo sprintf( esc_html__( 'Copy below Shortcode, create a new page and paste the shortcode %s', "guest-support" ), '<br><br>[guest-support view=form id=' . esc_html( $default_form_id ) . ']' );
                                    echo '<br><br>';
                                }
                                elseif ( $shortcode == 'ticket' )
                                {
                                    echo '<strong>' . esc_html__( 'Crate Single Ticket View Page:', "guest-support" ) . '</strong>';
                                    echo '<br><br>';
                                    echo sprintf( esc_html__( 'Copy below Shortcode, create a new page and paste the shortcode %s', "guest-support" ), '<br><br>[guest-support view=ticket]' );
                                    echo '<br><br>';
                                }
                            ?>
                        </p>
                        <p><?php esc_html_e( "After creating the Page, come back here to continue.", "guest-support" ); ?></p>
                        <p><a href="<?php echo esc_url( admin_url( 'post-new.php?post_type=page' ) ) ; ?>" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( "Let's do it", "guest-support" ); ?></a></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
<?php endif; ?>
<!-- END Setup guide -->