<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;
// Check if user is in Admin area and can administrator
GuestSupportApp::verifyAdmin();

$db = new GuestSupportDbquery();
$tickets = $db->getTickets();
$departments = GuestSupportApp::getDepartments();
$forms = GuestSupportApp::getForms();
$isagent = GuestSupportApp::isAgent();
$agent_id_enc = GuestSupportApp::AgentIdEncrypt();
$ispro = GuestSupportApp::isPro();
$ticket_page = GuestSupportApp::getTicketPageUrl();
$return_url = base64_encode( admin_url( 'admin.php?page=guest-support' ) );
$datetime_formate = get_option('date_format') . ' ' . get_option('time_format');

$paged = isset( $_GET['paged'] ) ? abs( (int) sanitize_text_field( $_GET['paged'] ) ) : 1;
$ticket_status = isset( $_GET['ticket_status'] ) ? sanitize_text_field( $_GET['ticket_status'] ) : '';
$sortby = isset( $_GET['sortby'] ) ? sanitize_text_field( $_GET['sortby'] ) : '';
$status_filter = isset( $_GET['status_filter'] ) ? sanitize_text_field( $_GET['status_filter'] ) : '';
$searchstring = isset( $_GET['s'] ) ? sanitize_text_field( $_GET['s'] ) : '';
$departmentId = isset( $_GET['department'] ) ? (int) sanitize_text_field( $_GET['department'] ) : 0;
$formId = isset( $_GET['form_id'] ) ? (int) sanitize_text_field( $_GET['form_id'] ) : 0;
$ticketCounts = GuestSupportApp::paginationArgs();
$status_text = '';
if ( $ticket_status == 'open' )
{
    $status_text = esc_html__( 'Open', 'guest-support' );
}
elseif ( $ticket_status == 'pending' )
{
    $status_text = esc_html__( 'Pending', 'guest-support' );
}
elseif ( $ticket_status == 'closed' )
{
    $status_text = esc_html__( 'Closed', 'guest-support' );
}

?>

<div class="wrap wpgs-tickets">
    <h1 class="wp-heading-inline"><?php esc_html_e( 'All Tickets - Guest Support', 'guest-support' ); ?></h1>
    <a href="<?php echo $ispro ? esc_url( admin_url( 'admin.php?page=guest-support&type=create-ticket' ) ) : 'javascript:;'; ?>" class="page-title-action"<?php echo !$ispro ? ' id="wpgs_open_modal" data-modal="wpgs_modal_create_ticket_by_agent"' : ''; ?>><?php esc_html_e( 'Create Ticket', 'guest-support' ); ?></a>

    <hr class="wp-header-end">
    <?php echo wp_kses_post( GuestSupportApp::getNotice() ); ?>

	<?php if ( GuestSupportApp::askForReview() === true ) : ?>
		<div class="notice notice-success">
			<h3>Do you like this plugin?</h3>
			<p>We'd love to hear your feedback! Please consider leaving a review on the WordPress Plugins directory to share your great experiences with our plugin. Your review can make a real difference. Leave a review at: <a href="https://wordpress.org/plugins/guest-support/" target="_blank">https://wordpress.org/plugins/guest-support/</a>.</p>
			<p><a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support&wpgs_handler=askforreview' ) ); ?>">Don't show again</a>.</p>
		</div>
	<?php endif; ?>

    <div class="wpgs-wp-heading-spacer">&nbsp;</div>

    <div class="wpgs-filter">
        <ul>
            <?php
                echo '<li><a href="' . esc_url( admin_url( 'admin.php?page=guest-support' ) ) . '" class="' . ( !$ticket_status ? 'current' : '' ) . '">' . esc_html__( 'All', 'guest-support' ) . ' <span>(' . esc_html( $ticketCounts->total ) . ')</span></a>&nbsp;|&nbsp;</li>';
                echo '<li><a href="' . esc_url( admin_url( 'admin.php?page=guest-support&ticket_status=open' ) ) . '" class="' . ( $ticket_status == 'open' ? 'current' : '' ) . '">' . esc_html__( 'Open', 'guest-support' ) . ' <span>(' . esc_html( $ticketCounts->open ) . ')</span></a>&nbsp;|&nbsp;</li>';
                echo '<li><a href="' . esc_url( admin_url( 'admin.php?page=guest-support&ticket_status=pending' ) ) . '" class="' . ( $ticket_status == 'pending' ? 'current' : '' ) . '">' . esc_html__( 'Pending', 'guest-support' ) . ' <span>(' . esc_html( $ticketCounts->pending ) . ')</span></a>&nbsp;|&nbsp;</li>';
                echo '<li><a href="' . esc_url( admin_url( 'admin.php?page=guest-support&ticket_status=closed' ) ) . '" class="' . ( $ticket_status == 'closed' ? 'current' : '' ) . '">' . esc_html__( 'Closed', 'guest-support' ) . ' <span>(' . esc_html( $ticketCounts->closed ) . ')</span></a>&nbsp;|&nbsp;</li>';
                echo '<li><a href="javascript:;" class="guest-support-mass-delete">' . esc_html__( 'Delete selected tickets', 'guest-support' ) . '</a></li>';
            ?>
        </ul>
        <form action="<?php echo esc_url( admin_url( 'admin.php?page=guest-support' ) ); ?>" method="get">
            <input type="hidden" name="page" value="guest-support">
            <input type="hidden" name="paged" value="1">
            <div class="wt-grid">
                <div class="wt-filter-option">
                    <select name="sortby" id="wpgs-ticket-sortby">
                        <option value=""><?php esc_html_e( '--Sort By--', 'guest-support' ); ?></option>
                        <option value="created_asc"<?php echo $sortby == 'created_asc' ? ' selected' : ''; ?>><?php esc_html_e( 'Ticket Created Date A-Z', 'guest-support' ); ?></option>
                        <option value="created_desc"<?php echo $sortby == 'created_desc' ? ' selected' : ''; ?>><?php esc_html_e( 'Ticket Created Date Z-A', 'guest-support' ); ?></option>
                        <option value="updated_asc"<?php echo $sortby == 'updated_asc' ? ' selected' : ''; ?>><?php esc_html_e( 'Last Reply Date A-Z', 'guest-support' ); ?></option>
                        <option value="updated_desc"<?php echo $sortby == 'updated_desc' ? ' selected' : ''; ?>><?php esc_html_e( 'Last Reply Date Z-A', 'guest-support' ); ?></option>
                    </select>
                </div>
				<div class="wt-filter-option">
                    <select name="status_filter" id="wpgs-ticket-sortby">
                        <option value=""><?php esc_html_e( '--Filter By Status--', 'guest-support' ); ?></option>
                        <option value="open"<?php echo $status_filter == 'open' ? ' selected' : ''; ?>><?php esc_html_e( 'Open', 'guest-support' ); ?></option>
                        <option value="pending"<?php echo $status_filter == 'pending' ? ' selected' : ''; ?>><?php esc_html_e( 'Pending', 'guest-support' ); ?></option>
                        <option value="closed"<?php echo $status_filter == 'closed' ? ' selected' : ''; ?>><?php esc_html_e( 'Closed', 'guest-support' ); ?></option>
                    </select>
                </div>
                <?php if ( $ispro ) : ?>
                    <div class="wt-filter-option">
                        <select name="department" id="wpgs-ticket-department">
                            <option value=""><?php esc_html_e( '--Filter By Department--', 'guest-support' ); ?></option>
                            <?php
                                if ( $departments !== false )
                                {
                                    foreach ($departments as $item) {
                                        echo '<option value="' . esc_attr( $item->id ) . '"' . ( $departmentId == $item->id ? ' selected' : '' ) . '>' . esc_html( $item->name ) . '</option>';
                                    }
                                }
                            ?>
                        </select>
                    </div>
					<div class="wt-filter-option">
                        <select name="form_id" id="wpgs-ticket-form">
                            <option value=""><?php esc_html_e( '--Filter By Form--', 'guest-support' ); ?></option>
                            <?php
                                if ( $forms !== false )
                                {
									echo '<option value="-1"' . ( $formId === -1 ? ' selected' : '' ) . '>' . esc_html__( 'Tickets by agents', 'guest-support' ) . '</option>';
                                    foreach ($forms as $item) {
                                        echo '<option value="' . esc_attr( $item->id ) . '"' . ( $formId == $item->id ? ' selected' : '' ) . '>' . esc_html( $item->form_name ) . '</option>';
                                    }
                                }
                            ?>
                        </select>
                    </div>
                <?php endif; ?>
                <div class="wt-filter-option">
                    <input type="text" name="s" id="wpgs-ticket-searchstring" placeholder="<?php esc_html_e( 'Search: ticket id/subject/email', 'guest-support' ); ?>" value="<?php echo esc_attr( $searchstring ); ?>">
                </div>
                <div class="wt-filter-option">
                    <input type="submit" id="wpgs-filter-submit" class="button" value="<?php esc_html_e( 'Apply', 'guest-support' ); ?>">
                </div>
            </div>
            <?php if ( $searchstring || $departmentId || $formId || $status_filter ) : ?>
                <div class="wpgs-filter-result">
                    <h4><?php echo esc_html( sprintf( __( 'Search Result: %s Tickets found.', 'guest-support' ), $ticketCounts->num_rows ) ); ?></h4>
                </div>
            <?php endif; ?>
        </form>
    </div>

    <?php if ( !empty( $tickets ) ) : ?>
        <table class="wp-list-table widefat fixed striped table-view-list wpgs-tickets-table">
            <thead>
                <tr>
					<th scope="col" id="selectall" class="column-select"><input type="checkbox" id="guest_support_tickets_select_all"></th>
                    <th scope="col" id="subject" class="column-subject"><?php esc_html_e( 'Subject:', 'guest-support' ); ?></th>
                    <th scope="col" id="status" class="column-status wpgs-hide-mobile"><?php esc_html_e( 'Status:', 'guest-support' ); ?></th>
                    <th scope="col" id="last-update-date" class="column-last-update-date wpgs-hide-mobile"><?php esc_html_e( 'Last Replied Date:', 'guest-support' ); ?></th>
                    <th scope="col" id="created-by" class="column-created-by wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Created By:', 'guest-support' ); ?></th>
                    <?php if ( $ispro ) : ?>
						<th scope="col" id="form-name" class="column-department wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Form:', 'guest-support' ); ?></th>
						<th scope="col" id="department" class="column-department wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Department:', 'guest-support' ); ?></th>
					<?php endif; ?>
					<th scope="col" id="created-date" class="column-created-date wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Created Date:', 'guest-support' ); ?></th>
                </tr>
            </thead>
            <tbody id="the-list">
                <?php foreach ($tickets as $item) : ?>
                    <tr id="ticket-<?php echo esc_attr( $item->id ); ?>" class="ticket-<?php echo esc_attr( $item->id ); ?> type-ticket">
                        <td class="column-select"><input type="checkbox" name="ticketid[]" value="<?php echo esc_attr( $item->id ); ?>" id="guest_support_tickets_select"></td>
						<td class="subject column-subject has-row-actions">
                            <strong>
                                <a href="<?php echo esc_url( $ticket_page . $item->ticket_id . '/?auth=' . $item->ticket_token . '&agent=' . $agent_id_enc ); ?>" target="_blank" class="row-title"><?php echo stripslashes( esc_html( $item->subject ) ); ?> <span class="wpgs-replies-count">(<?php echo ( isset( $item->total_replies ) ? esc_html( $item->total_replies ) : 0 ) . ' ' . esc_html__( 'replies', 'guest-support' ); ?>)</span></a>    
                            </strong>
                            <div class="row-actions">
                                <span class="view"><a href="<?php echo esc_url( $ticket_page . $item->ticket_id . '/?auth=' . $item->ticket_token . '&agent=' . $agent_id_enc ); ?>" target="_blank"><?php esc_html_e( 'View/Reply', 'guest-support' ); ?></a> | </span>
                                <?php
                                    if ( $item->status == 'closed' )
                                    {
                                        echo '<span class="reopen-ticket"><a href="' . esc_url( admin_url( 'admin.php?page=guest-support&wpgs_handler=reopenticket&id=' . esc_html( $item->ticket_id ) . '&auth=' . esc_html( $item->ticket_token ) . '&return=' . $return_url ) ) . '">' . esc_html__( 'Re-Open Ticket', 'guest-support' ) . '</a> | </span>';
                                    }
                                    else
                                    {
                                        echo '<span class="close-ticket"><a href="' . esc_url( admin_url( 'admin.php?page=guest-support&wpgs_handler=closeticket&id=' . esc_html( $item->ticket_id ) . '&auth=' . esc_html( $item->ticket_token ) . '&return=' . $return_url ) ) . '">' . esc_html__( 'Close Ticket', 'guest-support' ) . '</a> | </span>';
                                    }
                                ?>
                                <span class="delete">
                                    <a href="<?php echo esc_url( admin_url( 'admin.php?page=guest-support&wpgs_handler=deleteticket&id=' . $item->ticket_id . '&auth=' . $item->ticket_token . '&return=' . $return_url ) ); ?>" id="wpgs-delete"><?php esc_html_e( 'Delete', 'guest-support' ); ?></a>
                                </span>
                            </div>
                        </td>
                        <td class="column-status wpgs-hide-mobile">
                            <?php echo ucfirst( esc_html( $item->status ) ); ?>
                        </td>
                        <td class="column-last-update-date wpgs-hide-mobile">
                            <?php echo wp_date( $datetime_formate, esc_html( $item->last_update_date ) ); ?>
                        </td>
                        <td class="column-created-by wpgs-hide-mobile wpgs-hide-tablet">
                            <?php echo esc_html( $item->name ) . '<br>' . esc_html( $item->email ); ?>
                        </td>
						<?php if ( $ispro ) : ?>
							<td class="column-form-name wpgs-hide-mobile wpgs-hide-tablet">
                            	<?php echo esc_html( $item->form_name ); ?>
                        	</td>
							<td class="column-department wpgs-hide-mobile wpgs-hide-tablet">
                            	<?php echo esc_html( $item->department_name ); ?>
                        	</td>
						<?php endif; ?>
                        <td class="column-created-date wpgs-hide-mobile wpgs-hide-tablet">
                            <?php echo wp_date( $datetime_formate, esc_html( $item->created_date ) ); ?>
                        </td>
                    </tr>
                <?php endforeach; ?>
            </tbody>
            <tfoot>
                <tr>
                    <th scope="col" id="selectall" class="column-select"></th>
                    <th scope="col" id="subject" class="column-subject"><?php esc_html_e( 'Subject:', 'guest-support' ); ?></th>
                    <th scope="col" id="status" class="column-status wpgs-hide-mobile"><?php esc_html_e( 'Status:', 'guest-support' ); ?></th>
                    <th scope="col" id="last-update-date" class="column-last-update-date wpgs-hide-mobile"><?php esc_html_e( 'Last Replied Date:', 'guest-support' ); ?></th>
                    <th scope="col" id="created-by" class="column-created-by wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Created By:', 'guest-support' ); ?></th>
					<?php if ( $ispro ) : ?>
						<th scope="col" id="form-name" class="column-department wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Form:', 'guest-support' ); ?></th>
						<th scope="col" id="department" class="column-department wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Department:', 'guest-support' ); ?></th>
					<?php endif; ?>
                    <th scope="col" id="created-date" class="column-created-date wpgs-hide-mobile wpgs-hide-tablet"><?php esc_html_e( 'Created Date:', 'guest-support' ); ?></th>
                </tr>
            </tfoot>
        </table>
        <div class="wpgs-pagination">
            <?php
                $currentTotal = (int) $ticketCounts->num_rows;
                if ( $currentTotal > $ticketCounts->items_per_page )
                {
                    $offset = ( (int) $ticketCounts->offset + (int) $ticketCounts->items_per_page );
                    if ( $offset > $currentTotal )
                    {
                        $offset = $currentTotal;
                    }
                    echo '<span class="wpgs-total-num">' . esc_html( ( (int) $ticketCounts->offset + 1 ) . ' - ' . $offset  . ' ' . esc_html__( 'of', 'guest-support' ) . ' ' . $currentTotal . ' ' . $status_text . ' ' . esc_html__( 'Tickets', 'guest-support' ) ) . '</span>';
                    echo '<span class="wpgs-pagination-links">';
                        echo paginate_links( array(
                            'base' => add_query_arg( 'paged', '%#%' ),
                            'format' => '',
                            'prev_text' => __( '&lsaquo;', 'guest-support' ),
				            'next_text' => __( '&rsaquo;', 'guest-support' ),
                            'total' => ceil( $currentTotal / (int) $ticketCounts->items_per_page ),
                            'current' => (int) $ticketCounts->current_page
                        ));
                    echo '</span>';
                }
            ?>
        </div>
    <?php else : ?>
        <h4><?php esc_html_e( 'No Ticket found.', 'guest-support' ); ?></h4>
    <?php endif; ?>
</div>

<div id="wpgs_modal_create_ticket_by_agent" class="wpgs-modal">
	<div class="wpgs-modal-wrapper">
		<div class="wpgs-modal-container">
			<div class="wpgs-modal-container-wrapper">
				<div class="wpgs-modal-header">
					<h3><?php esc_html_e( 'Create new Ticket for a registered user', 'guest-support' ); ?></h3>
				</div>
				<div class="wpgs-modal-content">
                    <?php esc_html_e( "You need PRO addon to create tickets for (send message to) other registered users.", "guest-support" ); ?>
				</div>
				<div class="wpgs-modal-footer">
					<a href="javascript:;" id="wpgs_modal_close" class="button button-cancel"><?php esc_html_e( 'Close', 'guest-support' ); ?></a>
				</div>
			</div>
		</div>
	</div>
</div>