<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;

// $request = isset( $_POST['request'] ) ? sanitize_text_field( $_POST['request'] ) : '';
$returnData = '';

// Early return if request was not found
if ( !$request )
{
    echo json_encode($returnData);
    exit;
}

// Get users
if ( $request == 'get_users' )
{
    $searchterm = isset( $_POST['searchterm'] ) ? sanitize_text_field( $_POST['searchterm'] ) : '';
    if ( $searchterm )
    {
        $searchterm = str_replace( array( '=', '_', '%', '>', '<', '$', '|', '"', '\'' ), '', $searchterm );
        $db = new GuestSupportDbquery();
        $results = $db->getUsersByEmail( $searchterm );

        if ( $results === false )
        {
            $returnData = '<div class="item message">' . esc_html__( 'No user found.', 'guest-support' ) . '</div>';
        }
        else
        {
            foreach ($results as $user) {
                $returnData .= '<div class="item" data-value="' . esc_attr( $user->ID ) . '">' . esc_html( $user->display_name ) . ' (' . esc_html( $user->user_email ) . ')</div>';
            }
        }
    }
}
elseif ( $request == 'edit_reply' )
{
    $message = isset( $_POST['message'] ) ? trim( wp_kses_post( $_POST['message'] ) ) : '';
    $ticket_id = isset( $_POST['ticket_id'] ) ? sanitize_text_field( $_POST['ticket_id'] ) : '';
    $ticket_auth = isset( $_POST['ticket_auth'] ) ? sanitize_text_field( $_POST['ticket_auth'] ) : '';

    $enc_message_id = isset( $_POST['message_id'] ) ? GuestSupportApp::sanitizeBase64( $_POST['message_id'] ) : '';
    $message_id = GuestSupportApp::decrypt( $enc_message_id );

    if ( !$message || !$ticket_id || !$ticket_auth || !$message_id )
    {
        $returnData = esc_html__( "Enter necessary data to continue.", "guest-support" );
    }
    else
    {
        $config = GuestSupportApp::config();
        $isagent = GuestSupportApp::isAgent();
        $db = new GuestSupportDbquery();
        $canedit = 0;
        if ( $config->can_edit_replies == 'both' || ( $config->can_edit_replies == 'agent' && $isagent ) || ( $config->can_edit_replies == 'user' && !$isagent ) )
        {
            if ( ( $isagent && $config->edit_replies_globally == 'yes' ) || $config->edit_reply_type == 'all' )
            {
                $canedit = 1;
            }
            elseif ( $config->edit_reply_type == 'last' )
            {
                $lastMessage = $db->getLastMessage( $ticket_id, $ticket_auth );
                if ( $lastMessage !== false && (int) $lastMessage->id === (int) $message_id && ( ( $lastMessage->status == 'pending' && $isagent ) || ( $lastMessage->status == 'open' && !$isagent ) ) )
                {
                    $canedit = 1;
                }
            }
        }
        else
        {
            $canedit = 0;
        }

        if ( $canedit )
        {
            // Prepare data
            $updateData = array(
                'message' => $message
            );
            $updateDataCondition = array(
                'id' => $message_id,
                'ticket_id' => $ticket_id,
                'ticket_token' => $ticket_auth
            );
            $updateDataType = array(
                '%s'
            );
            $updateDataConditionType = array(
                '%d',
                '%s',
                '%s'
            );

            $update = $db->editMessage( $updateData, $updateDataCondition, $updateDataType, $updateDataConditionType );
            if ( $update === true )
            {
                $returnData = 'success';
            }
            else
            {
                $returnData = esc_html__( "The message wasn't updated, Reload this page and try again.", "guest-support" );
            }
        }
        else
        {
            $returnData = esc_html__( "You're not allowed to edit message.", "guest-support" );
        }
    }
}
elseif ( $request == 'delete_tickets' )
{
	/**
	 * Delete mass tickets
	 */
	$ticketids = isset($_POST['ids']) ? $_POST['ids'] : [];
	if ( !is_array( $ticketids ) || empty( $ticketids ) )
	{
		$returnData = esc_html__( "Please select one or more tickets to delete.", "guest-support" );
		GuestSupportApp::setNotice( $returnData, 'error' );
	}
	else
	{
		// Sanitize
		$cleanIds = [];
		foreach ($ticketids as $key => $value) {
			if ( (int) $value )
			{
				$cleanIds[] = (int) $value;
			}
		}

		if ( empty( $cleanIds ) )
		{
			$returnData = esc_html__( "Please select one or more tickets to delete.", "guest-support" );
			GuestSupportApp::setNotice( $returnData, 'error' );
		}
		else
		{
			// Delete selected tickets
			$db = new GuestSupportDbquery();
			$deleteresult = $db->deleteMassTickets( $cleanIds );

			$returnData = esc_html__( "Selected tickets have been deleted successfully.", "guest-support" );
			GuestSupportApp::setNotice( $returnData, 'success' );
		}
	}
}

echo json_encode($returnData);
exit;