<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com
 */
defined('ABSPATH') or die;

class GuestSupportApp
{
	private static $noticeId = '';
	private static $current_url = '';
	private static $isagent = 'undefined';
	private static $ticketpageid = 'undefined';
    private static $ticketpageurl = 'undefined';
    private static $ticketpageslug = 'undefined';
    private static $ispro = 'undefined';
    private static $uploadDirectory = 'undefined';
    private static $is_error_notice = 'undefined';
    private static $_ip_address = 'undefined';
    private static $_session_id = 'undefined';

	/**
	 * Store Notices
	 */
	public static function setNotice($msg, $type = "error")
	{
		$noticeid = self::noticeId();
		$data = array(
			'id' => $noticeid,
			'message' => $msg,
			'type' => $type
		);

		$dbdata = new GuestSupportDbquery();
		$dbdata->storeNotice( $data );

		// Add this notice id to session
		self::createSession( 'guest_support_notice_id', $noticeid );
	}

	/**
	 * Return Notices
	 */
	public static function getNotice()
	{
		// Get notice id from session
		// Add this notice id to session
		$id = sanitize_text_field( self::getSession( 'guest_support_notice_id' ) );

		if ( !$id )
		{
			return '';
		}

		$dbdata = new GuestSupportDbquery();
		$results = $dbdata->getNotices( $id );
		$html = '';

		if ( is_admin() )
		{
			$errorClass = 'notice notice-';
		}
		else
		{
			$errorClass = 'wpgs-alert wpgs-alert-';
		}

		$is_error_notice = 0;

		if ( !empty( $results ) )
		{
			foreach ( $results as $item ) {
				$html .= '<div class="' . $errorClass . $item->type . ' is-dismissible">';
				$html .= '<p>' . $item->message . '</p>';
				$html .= '</div>';
				if ( $item->type == 'error' )
				{
					$is_error_notice++;
				}
			}
		}
		else
		{
			return '';
		}

		if ( $is_error_notice )
		{
			self::$is_error_notice = true;
		}

		return $html;
	}

	/**
	 * Block access to admin features to non-admin users
	 */
	public static function verifyAdmin()
	{
		if ( is_admin() && current_user_can( 'administrator' ) )
		{
			return true;
		}
		else
		{
			$notice = esc_html__("Access Denied. Only admin can access to this page.", "guest-support");
			die( $notice );
		}
	}

	/**
	 * Is Notice type error
	 */
	public static function isErrorNotice()
	{
		return self::$is_error_notice;
	}

	/**
	 * Redirect
	 */
	public static function setRedirect( $url, $notice = '', $noticeType = "error" )
	{
		if ( !$url )
		{
			return false;
		}

		if ( $notice )
		{
			self::setNotice( $notice, $noticeType );
		}

		wp_safe_redirect( $url );
		exit;
	}

	/**
	 * Check pro addon
	 */
	public static function isPro()
	{
		if ( self::$ispro != 'undefined' )
		{
			return self::$ispro;
		}
		$proapp = WPGS_PLUGIN_ADDON_DIR . 'includes/library/class-pro-app.php';
		$prodbquery = WPGS_PLUGIN_ADDON_DIR . 'includes/library/class-pro-dbquery.php';
	
		if ( in_array( 'guest-support-addon/guest-support-addon.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) && file_exists( $proapp ) && file_exists( $prodbquery ) )
		{
			self::$ispro = true;
        }
		else
		{
			self::$ispro = false;
		}
		return self::$ispro;
	}

	/**
	 * Get current page url
	 */
	public static function currentUrl()
	{
		if ( !self::$current_url )
		{
			$http_host = sanitize_text_field( $_SERVER['HTTP_HOST'] );
			$http_uri = sanitize_text_field( $_SERVER['REQUEST_URI'] );
			$request_uri = $http_host . $http_uri;
			$home_url = site_url('', 'http');
			$home_url = str_replace( array( 'http://', 'https://' ), '', $home_url );
			$current_url = str_replace( $home_url, '', $request_uri );
			self::$current_url = $current_url;
		}
		return self::$current_url;
	}

	/**
	 * Get Global Configurations
	 */
	public static function config()
	{
		$db = new GuestSupportDbquery();
		$results = $db->getConfig();
		$config = new stdClass();

		if ( $results )
		{
			foreach ($results as $item) {
				$configName = $item->name;
				$config->$configName = stripcslashes( $item->value );
			}
		}

		return $config;
	}

	/**
	 * Edit config
	 */
	public static function editConfig( $data = array() )
	{
		if ( !is_array( $data ) )
		{
			$notice = esc_html__("Enter requested data to edit settings.", "guest-support");
			$redirectUrl = self::refererUrl();
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		$redirectUrl = admin_url( 'admin.php?page=guest-support-settings' );
		if ( isset( $data['wpgs_handler_type'] ) )
		{
			$redirectUrl = admin_url( 'admin.php?page=guest-support-email-settings' );
		}

		$is_email_settings = isset( $data['wpgs_handler_type'] ) && $data['wpgs_handler_type'] == 'wpgs_email_settings_form' ? 1 : 0;

		// Unset additional inputs
		unset( 
			$data['_wpnonce_settings_form'], 
			$data['_wp_http_referer'], 
			$data['wpgs_handler'],
			$data['wpgs_handler_type']
		);

		// Check pre ticket close and ticket close days
		$auto_close_tickets = isset( $data['auto_close_tickets'] ) ? (int) $data['auto_close_tickets'] : 0;
		$pre_close_email = isset( $data['pre_close_email'] ) ? (int) $data['pre_close_email'] : 0;
		if ( $auto_close_tickets && $pre_close_email )
		{
			if ( $pre_close_email >= $auto_close_tickets )
			{
				$notice = esc_html__("The 'Pre-close' email notification days should be less than the ticket 'auto close' days.", "guest-support");
				self::setRedirect( $redirectUrl, $notice, 'error' );
			}
		}

		// Sanitize inputs
		foreach ( $data as $name => $value ) {
			$email_message = array(
				'ticket_email_user_message',
				'ticket_email_agent_message',
				'ticket_email_additional_message',
				'reply_email_user_message',
				'reply_email_agent_message',
				'reply_email_additional_message',
				'new_ticket_confirmation_message',
			);

			if ( in_array( $name, $email_message ) )
			{
				$data[$name] = trim( wp_kses_post( $value ) );
			}
			else
			{
				$data[$name] = $value ? trim( sanitize_textarea_field( $value ) ) : '';
			}
		}

		$config = self::config();
		$db = new GuestSupportDbquery();
		$error = 0;

		// Process edits
		foreach ($data as $name => $value) {
			if ( isset( $config->$name ) && $config->$name != $value )
			{
				$todb_data = array( 'value' => $value );
				$todb_condition = array( 'name' => $name );
				$todb_dataType = array( '%s' );
				$todb_conditionType = array( '%s' );

				$edit = $db->editConfig( $todb_data, $todb_condition, $todb_dataType, $todb_conditionType );
				if ( $edit === false )
				{
					$error++;
				}
			}
		}

		if ( $error )
		{
			$notice = esc_html__("Something wen't wrong and one or more settings wasn't saved.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		else
		{
			// Setup guide
			$wpgs_setup_guide = get_option('wpgs_setup_guide');
			if ( $is_email_settings && $wpgs_setup_guide == 'email_settings' )
			{
				update_option('wpgs_setup_guide', 'email_templates');
			}
			elseif ( !$is_email_settings && $wpgs_setup_guide == 'settings' )
			{
				update_option('wpgs_setup_guide', 'email_settings');
			}

			// Redirect with success message
			$notice = esc_html__("Settings has been updated successfully.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
	}

	public static function allowedKsesTags()
	{
		$allowed_kses_tags = array(
			'a' => array(
				'href' => array(),
				'title' => array(),
				'class' => array(),
				'target' => array()
			),
			'br' => array(),
			'em' => array(),
			'strong' => array(),
			'i' => array(),
			'span' => array()
		);
		return $allowed_kses_tags;
	}

	/**
	 * Edit departments
	 */
	public static function editDepartment( $data = array() )
	{
		if ( !is_array( $data ) )
		{
			$notice = esc_html__("Enter requested data to edit Department.", "guest-support");
			$redirectUrl = admin_url( 'admin.php?page=guest-support-departments' );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		// Check if another department with same name already exist
		$db = new GuestSupportDbquery();
		$checkrecords = $db->editDepartmentCheck( $data['dept_name'], $data['dept_id'] );
		if ( !empty( $checkrecords ) )
		{
			$notice = esc_html__("Another Department with same name already exist, try again with another Department name.", "guest-support");
			$redirectUrl = admin_url( 'admin.php?page=guest-support-departments&type=edit&dept_id=' . $data['dept_id'] );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Everything okay, now try to edit
		$edit = $db->editDepartments( $data );
		if ( $edit === true )
		{
			// Setup guide
			$wpgs_setup_guide = get_option('wpgs_setup_guide');
			if ( !$wpgs_setup_guide )
			{
				update_option('wpgs_setup_guide', 'forms');
			}

			// Redirect with success message
			$notice = esc_html__("Department successfully updated.", "guest-support");
			$redirectUrl = admin_url( 'admin.php?page=guest-support-departments&type=edit&dept_id=' . $data['dept_id'] );
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
		else
		{
			$notice = esc_html__("Something went wrong and the department wasn't updated. Please try again.", "guest-support");
			$redirectUrl = admin_url( 'admin.php?page=guest-support-departments&type=edit&dept_id=' . $data['dept_id'] );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
	}

	/**
	 * Get Departments
	 */
	public static function getDepartments()
	{
		$db = new GuestSupportDbquery();
		return $db->listDepartments();
	}

	/**
	 * Get Form Departments
	 * @param int $form_id The viewing form id
	 */
	public static function getFormDepartments( $form_id = 0 )
	{
		$db = new GuestSupportDbquery();
		return $db->listFormDepartments( $form_id );
	}

	/**
	 * Get default form id
	 */
	public static function getDefaultFormId()
	{
		$db = new GuestSupportDbquery();
		return $db->getDefaultFormId();
	}

	/**
	 * Edit Email Template
	 */
	public static function editEmailTemplate( $data = array() )
	{
		$formUrl = admin_url( 'admin.php?page=guest-support-email-templates&type=edit&template_id=' . $data['id'] );
		$listUrl = admin_url( 'admin.php?page=guest-support-email-templates' );
		if ( !is_array( $data ) )
		{
			$notice = esc_html__("Enter requested data to edit Email Template.", "guest-support");
			self::setRedirect( $formUrl, $notice, 'error' );
		}
		// Check if another department with same name already exist
		$db = new GuestSupportDbquery();

		$edit = $db->editEmailTemplateQuery( $data );
		if ( $edit === true )
		{
			// Setup guide
			$wpgs_setup_guide = get_option('wpgs_setup_guide');
			if ( $wpgs_setup_guide == 'email_templates' )
			{
				update_option('wpgs_setup_guide', 'shortcode');
			}

			// Redirect with success message
			$notice = esc_html__("The email template has been successfully updated.", "guest-support");
			self::setRedirect( $formUrl, $notice, 'success' );
		}
		else
		{
			$notice = esc_html__("An error occurred, and the email template was not updated. Please try again.", "guest-support");
			self::setRedirect( $listUrl, $notice, 'error' );
		}
	}

	/**
	 * Get form
	 */
	public static function getForm( $form_id )
	{
		$db = new GuestSupportDbquery();
		return $db->getForm( $form_id );
	}

	/**
	 * Get forms
	 */
	public static function getForms()
	{
		$db = new GuestSupportDbquery();
		return $db->getForms();
	}

	/**
	 * Edit Forms
	 */
	public static function editForms( $form_id, $data = array() )
	{
		if ( !is_array( $data ) || !$form_id )
		{
			$notice = esc_html__("Enter requested data to edit the Form.", "guest-support");
			$redirectUrl = admin_url( 'admin.php?page=guest-support-forms' );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Get inputs except fields
		$form_name = isset( $_POST['form_name'] ) ? sanitize_text_field( $_POST['form_name'] ) : '';
		$recaptcha = isset( $data['recaptcha'] ) ? sanitize_text_field( $data['recaptcha'] ) : 1;
		$submit_text = isset( $data['submit_text'] ) ? sanitize_text_field( $data['submit_text'] ) : 'Create Ticket';
		$verify_otp_text = isset( $data['verify_otp_text'] ) ? sanitize_text_field( $data['verify_otp_text'] ) : 'Continue';
		$submit_class = isset( $data['submit_class'] ) ? sanitize_text_field( $data['submit_class'] ) : 'wpgs-submit-button';
		$input_class = isset( $data['input_class'] ) ? sanitize_text_field( $data['input_class'] ) : 'wpgs-input';

		$fileupload = isset( $data['fileupload'] ) ? sanitize_text_field( $data['fileupload'] ) : 0;
		$upload_filesize = isset( $data['upload_filesize'] ) ? sanitize_text_field( $data['upload_filesize'] ) : 0;
		$allowed_filetypes = isset( $data['allowed_filetypes'] ) ? sanitize_text_field( $data['allowed_filetypes'] ) : '';
		$verify_email = isset( $data['verify_email'] ) ? sanitize_text_field( $data['verify_email'] ) : 0;
		$upload_filelimit = isset( $data['upload_filelimit'] ) ? sanitize_text_field( $data['upload_filelimit'] ) : 0;
		$suggest_docs = isset( $data['suggest_docs'] ) ? sanitize_text_field( $data['suggest_docs'] ) : 0;
		$suggest_docs_cats_arr = isset( $data['suggest_docs_cats'] ) ? $data['suggest_docs_cats'] : '';
		$fileupload_label = isset( $data['fileupload_label'] ) ? sanitize_text_field( $data['fileupload_label'] ) : '';
		$fileupload_help = isset( $data['fileupload_help'] ) ? sanitize_textarea_field( $data['fileupload_help'] ) : '';

		// Get params
		$params = $data['params'] ?? [];
		$params_data = '';
		// Sanitize params
		if ( !empty( $params ) )
		{
			foreach ( $params as $key => $value ) {
				$params[$key] = wp_kses_post( $value );
			}
			$params_data = serialize( $params );
		}

		// Process suggest_docs_cats
		$suggest_docs_cats = serialize( $suggest_docs_cats_arr );

		// Unset above inputs to get form input fields only
		unset( 
			$data['_wpnonce_forms_form'], 
			$data['_wp_http_referer'], 
			$data['form_name'], 
			$data['wpgs_handler'],
			$data['recaptcha'],
			$data['submit_text'],
			$data['verify_otp_text'],
			$data['submit_class'],
			$data['input_class'],
			$data['fileupload'],
			$data['upload_filesize'],
			$data['allowed_filetypes'],
			$data['verify_email'],
			$data['upload_filelimit'],
			$data['suggest_docs'],
			$data['suggest_docs_cats'],
			$data['fileupload_label'],
			$data['fileupload_help'],
			$data['params']
		);

		// Set form fields
		$form_fields = serialize( $data );

		// Prepare data
		$updateData = array(
			'form_name' => $form_name,
			'form_fields' => $form_fields,
			'recaptcha' => $recaptcha,
			'input_class' => $input_class,
			'submit_text' => $submit_text,
			'verify_otp_text' => $verify_otp_text,
			'submit_class' => $submit_class,
			'fileupload' => $fileupload,
			'upload_filesize' => $upload_filesize,
			'allowed_filetypes' => $allowed_filetypes,
			'verify_email' => $verify_email,
			'upload_filelimit' => $upload_filelimit,
			'suggest_docs' => $suggest_docs,
			'suggest_docs_cats' => $suggest_docs_cats,
			'fileupload_label' => $fileupload_label,
			'fileupload_help' => $fileupload_help,
			'params' => $params_data
		);
		$updateDataCondition = array(
			'id' => $form_id
		);
		$updateDataType = array(
			'%s',
			'%s',
			'%d',
			'%s',
			'%s',
			'%s',
			'%s',
			'%d',
			'%d',
			'%s',
			'%d',
			'%d',
			'%d',
			'%s',
			'%s',
			'%s',
			'%s'
		);
		$updateDataConditionType = array(
			'%d'
		);

		// Get db
		$db = new GuestSupportDbquery();

		// Everything okay, now try to edit
		$edit = $db->editForms( $updateData, $updateDataCondition, $updateDataType, $updateDataConditionType );
		if ( $edit === true )
		{
			// Setup guide
			$wpgs_setup_guide = get_option('wpgs_setup_guide');
			if ( $wpgs_setup_guide == 'forms' )
			{
				update_option('wpgs_setup_guide', 'settings');
			}

			// Redirect with success message
			$notice = esc_html__("Form successfully updated.", "guest-support");
			$redirectUrl = admin_url( 'admin.php?page=guest-support-forms&type=edit&form_id=' . $form_id );
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
		else
		{
			$notice = esc_html__("Something went wrong and the Form wasn't updated. Please try again.", "guest-support");
			$redirectUrl = admin_url( 'admin.php?page=guest-support-forms&type=edit&form_id=' . $form_id );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
	}

	/**
	 * Verify reCAPTCHA V3
	 */
	public static function verifyReCaptcha( $action, $token )
	{
		$config = self::config();
		$minscore = (float) $config->recaptcha_score;
		if ( $minscore < 0.1 || $minscore > 1 )
		{
			$minscore = 0.5;
		}

		$args = array(
			'secret'	=> $config->recaptcha_secret_key,
			'response'	=> $token
		);

		$response = wp_remote_get( add_query_arg( $args, 'https://www.google.com/recaptcha/api/siteverify' ) );
		$result = json_decode( $response['body'] );

		// Check for errors
		if( is_wp_error( $response ) || empty( $response['body'] ) || !$result ) {
			return FALSE;
		}

		// Verify the response
		if( $result->success == '1' && $result->action == $action && $result->score >= $minscore ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Create Ticket
	 */
	public static function createTicket( $data = array() )
	{
		$redirectUrl = sanitize_text_field( $_SERVER['HTTP_REFERER'] );
		if ( !is_array( $data ) )
		{
			$notice = esc_html__("Invalid request.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Get handler
		$form_id = isset( $data['wpgs_handler_id'] ) ? sanitize_text_field( $data['wpgs_handler_id'] ) : 0;
		$form_url = isset( $data['wpgs_handler_url'] ) ? sanitize_text_field( $data['wpgs_handler_url'] ) : '';
		$ticket_timezone = isset( $data['wpgs_handler_timezone'] ) ? sanitize_text_field( $data['wpgs_handler_timezone'] ) : '';

		if ( !$form_id || !$form_url )
		{
			$notice = esc_html__("Invalid request.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		$redirectUrl = site_url( $form_url );

		if ( $ticket_timezone == 'none' )
		{
			$ticket_timezone = wp_timezone_string();
		}

		// Get form fields
		$db = new GuestSupportDbquery();
		$form = self::getForm( $form_id );
		$formFields = array();

		if( !empty( $form ) )
		{
			$formFields = unserialize( $form->form_fields );
		}
		else
		{
			// Incase the form id was modified
			$notice = esc_html__("Invalid form id. Can't submit the ticket. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Verify reCAPTCHA
		$config = self::config();

		if ( $form->recaptcha && $config->recaptcha_secret_key && $config->recaptcha_site_key )
		{
			$recaptcha_action = isset( $data['_recaptcha_action'] ) ? sanitize_text_field( $data['_recaptcha_action'] ) : '';
			$recaptcha_token = isset( $data['_recaptcha_token'] ) ? sanitize_text_field( $data['_recaptcha_token'] ) : '';

			if ( !$recaptcha_action || !$recaptcha_token )
			{
				$notice = esc_html__("reCAPTCHA verification wasn't complete. Please try again.", "guest-support");
				self::setRedirect( $redirectUrl, $notice, 'error' );
			}
			$verify_recaptcha = self::verifyReCaptcha( $recaptcha_action, $recaptcha_token );

			if ( $verify_recaptcha === false )
			{
				$notice = esc_html__("Sorry! but our system identifying you as spammer and can't submit your ticket at this moment.", "guest-support");
				self::setRedirect( $redirectUrl, $notice, 'error' );
			}
		}

		// Check OTP verification
		if ( self::isPro() && (int) $form->verify_email && !is_user_logged_in() )
		{
			if ( self::getSession( 'guest_support_otp_verified' ) !== true )
			{
				$notice = esc_html__("We're sorry, but it appears that your email verification through OTP is pending. Please ensure that JavaScript is enabled in your browser to complete the verification process.", "guest-support");
				self::setRedirect( $redirectUrl, $notice, 'error' );
			}
		}

		// Get attachments
		$attachments = '';
		if ( $form->fileupload )
		{
			$attachments = self::getFiles( 'attachments' );
		}

		// Check ticket view page
		$ticket_page = self::getTicketPageUrl();
		if ( $ticket_page === false )
		{
			$notice = sprintf( __( "No ticket view Page found. Create a new Page and add shortcode %s", "guest-support" ), "<br><strong>[guest-support view=ticket]</strong>" );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Unset unnecessary input
		unset(
			$data['_wpgs_submit_ticket_form'],
			$data['_wp_http_referer'],
			$data['wpgs_handler'],
			$data['wpgs_handler_url'],
			$data['wpgs_handler_id'],
			$data['wpgs_handler_timezone'],
			$data['attachments'],
			$data['_recaptcha_action'],
			$data['_recaptcha_token']
		);

		// Sanitize data
		$data = self::sanitizeAndVerify( $form_id, $redirectUrl, $formFields, $data );

		// Get user id by email
		$userby_email = false;
		if ( isset( $data['email'] ) && $data['email'] )
		{
			$userby_email = self::userIdbyEmail( $data['email'] );
		}

		// Get inputs
		if ( is_user_logged_in() )
		{
			$user = wp_get_current_user();
			$data['name'] = $user->display_name;
			$data['email'] = $user->user_email;
			$user_id = $user->ID;
			$update_user_id = 0;
		}
		elseif ( $userby_email !== false )
		{
			$data['name'] = $userby_email->display_name;
			$data['email'] = $userby_email->user_email;
			$user_id = $userby_email->ID;
			$update_user_id = 0;
		}
		else
		{
			$user_id = 0;
			$update_user_id = 1;
		}

		// Ticket info
		$ticket_id = self::ticketId();
		$short_ticket_id = self::simpleTicketId();
		$ticket_token = self::randomString(19);
		$ticket_email_hash = md5( $data['email'] );
		$message_type = 'ticket';
		$status = 'open';
		$autoreply = 1;
		$published = 1;
		// $last_update_date = wp_date('Y-m-d H:i:s');
		$timestamp = time();

		if ( self::isAgent() )
		{
			$user_type = 'agent';
		}
		else
		{
			$user_type = 'user';
		}

		$department_id = isset( $formFields['field_department']['value'] ) ? $formFields['field_department']['value'] : '';
		if ( self::isPro() && $formFields['field_department']['showonform'] == '1' )
		{
			$department_id = $data['department'];
		}
		elseif ( self::isPro() && $formFields['field_department']['showonform'] == '0' && $formFields['field_department']['default_department'] )
		{
			$department_id = $formFields['field_department']['default_department'];
		}

		if ( !$department_id )
		{
			$department_id = $db->getDefaultDepartment();
		}

		// Prepare inputs
		$tc_name = $data['name'];
		$tc_email = $data['email'];
		$tc_subject = $data['subject'];
		$tc_message = $data['message'];

		// Unset used inputs
		unset(
			$data['name'],
			$data['email'],
			$data['department'],
			$data['subject'],
			$data['message']
		);

		// Get custom fields
		$email_custom_fields = $data;
		$tc_custom_fields = serialize( $data );

		// Prepare ticket data
		$newTicketdata = array(
			'ticket_id' => $ticket_id,
			'short_ticket_id' => $short_ticket_id,
			'ticket_token' => $ticket_token,
			'form_id' => $form_id,
			'ticket_email_hash' => $ticket_email_hash,
			'user_id' => $user_id,
			'name' => $tc_name,
			'email' => $tc_email,
			'message_type' => $message_type,
			'department_id' => $department_id,
			'subject' => $tc_subject,
			'message' => $tc_message,
			'custom_fields' => $tc_custom_fields,
			'encrypted_message' => '',
			'status' => $status,
			'autoreply' => $autoreply,
			'published' => $published,
			'update_user_id' => $update_user_id,
			'user_type' => $user_type,
			'created_date' => $timestamp,
			'last_update_date' => $timestamp,
			'ticket_timezone' => $ticket_timezone,
			'created_by' => 'user'
		);
		$newTicketdataType = array(
			'%s',
			'%d',
			'%s',
			'%d',
			'%s',
			'%d',
			'%s',
			'%s',
			'%s',
			'%d',
			'%s',
			'%s',
			'%s',
			'%s',
			'%s',
			'%d',
			'%d',
			'%d',
			'%s',
			'%d',
			'%d',
			'%s',
			'%s'
		);

		// Check and upload attachments and early return if any errors before creating ticket
		$filedata = '';
		if ( $form->fileupload && !empty( $attachments ) )
		{
			$allowed_extensions_raw = preg_replace('/\s+/', '', $form->allowed_filetypes);
			$allowed_extensions = explode( ',', $allowed_extensions_raw );
			$filedata = self::uploadFiles( $redirectUrl, $form->upload_filesize, $form->upload_filelimit, $ticket_id, false, $attachments, $allowed_extensions );
		}

		// Insert ticket to db
		$insertId = $db->createTicket( $newTicketdata, $newTicketdataType );
		if ( $insertId !== false )
		{
			// Add attachments to database
			$filestodb = array();
			if ( $form->fileupload && !empty( $attachments ) && !empty( $filedata ) && is_array( $filedata ) )
			{
				$filestodb = self::filesToDb( $ticket_id, $insertId, $filedata );
				if ( $filestodb === false )
				{
					$notice = esc_html__("Error: Your attachment(s) was not uploaded.", "guest-support");
					self::setNotice( $notice, "error" );
				}
			}

			/**
			 * Prepare email data
			 * Send email
			 */

			// Add custom fields
			$email_body_custom_fields = '';
			if ( !empty( $email_custom_fields ) )
			{
				$count_ecf = 0;
				$html_ect = '';
				foreach ( $email_custom_fields as $name => $value ) {
					$inputname = 'field_' . $name;
					if ( $formFields[$inputname]['fieldtoform'] != 'reply' && ( $formFields[$inputname]['addtoemail'] == 'ticket' || $formFields[$inputname]['addtoemail'] == 'both' ) )
					{
						$count_ecf++;

						$eb_cf_label = '<strong>' . esc_html__( $formFields[$inputname]['label'], 'guest-support' ) . ': </strong><br>';
						if ( is_array( $value ) )
						{
							$fieldvalue = implode( ', ', $value );
						}
						else
						{
							$fieldvalue = str_replace( '__', ' ', $value);
						}
						$eb_cf_value = stripslashes( $fieldvalue );
						$html_ect .= '<p>' . $eb_cf_label . $eb_cf_value . '</p>';
					}
				}
				if ( $count_ecf > 0 )
				{
					$email_body_custom_fields = '<h3>' . esc_html__( "Additional Informations:", "guest-support" ) . '</h3>' . $html_ect;
				}
			}
			// Add Attachments links
			$email_body_attachments = '';
			if ( $filestodb !== false && !empty( $filestodb ) )
			{
				$file_url = self::getTicketPageUrl() . '?id=' . $ticket_id . '&auth=' . $ticket_token . '&wpgs_handler=download_file&file_id=';

				$email_body_attachments = '<h3>' . esc_html__("Attachments:", "guest-support") . '</h3>';
				$email_body_attachments .= '<ul>';
				foreach ($filestodb as $file) {
					$email_body_attachments .= '<li><a href="' . $file_url . self::encrypt( $file['id'] ) . '" target="_blank">' . $file['name'] . ' (' . self::bytesToReadable( $file['size'] ) . ')</a></li>';
				}
				$email_body_attachments .= '</ul>';
			}

			// Agent info
			$agent_id_enc = '';
			$agent_id = self::getAgentIdByDepartmentId( $department_id );
			if ( $agent_id )
			{
				$agentinfo = get_userdata( $agent_id );
				$agent_name = $agentinfo->display_name;
				$agent_email = $agentinfo->user_email;
				$agent_id_enc = self::encrypt( $agent_id );
			}
			else
			{
				$agent_name = '';
				$agent_email = '';
			}

			$ticket_url_user = $ticket_page . $ticket_id . '/?auth=' . $ticket_token;
			$ticket_url_agent = $ticket_page . $ticket_id . '/?auth=' . $ticket_token . '&agent=' . $agent_id_enc;

			$tc_subject = esc_html( stripslashes( $tc_subject ) );
			$tc_message = stripslashes( $tc_message );

			$departmentData = $db->getDepartment( $department_id );

			/**
			 * Send emails
			 */
			$emailTemplates = self::getEmailTemplates();

			// Send user email
			$sitename = get_bloginfo( 'name' );
			$siteurl = home_url();

			$email_body_placeholder = array(
				'{name}'				=> $tc_name,
				'{email}'				=> $tc_email,
				'{is_registered_user}'	=> $userby_email !== false ? esc_html__("Yes", "guest-support") : esc_html__("No", "guest-support"),
				'{user_id}'				=> $userby_email !== false ? $userby_email->ID : 0,
				'{ip_address}'			=> self::getUserIP(),
				'http://{ticket_link}'	=> $ticket_url_user,
				'https://{ticket_link}'	=> $ticket_url_user,
				'{ticket_link}'			=> $ticket_url_user,
				'{ticket_id}'			=> $ticket_id,
				'{short_ticket_id}'	=> $short_ticket_id,
				'{form}'				=> $form->form_name,
				'{subject}'				=> $tc_subject,
				'{message}'				=> $tc_message,
				'{ticket_message}'		=> $tc_message,
				'{reply_message}'		=> $tc_message,
				'{custom_fields}'		=> $email_body_custom_fields,
				'{attachments}'			=> $email_body_attachments,
				'{agent_name}'			=> $agent_name,
				'{agent_email}'			=> $agent_email,
				'{department}'			=> $departmentData->name,
				'{SiteName}'			=> $sitename,
				'{sitename}'			=> $sitename,
				'{siteurl}'				=> $siteurl
			);

			// Send email to user
			$emailTemplate = !empty( $emailTemplates ) && isset( $emailTemplates['ticket_email_user_message'] ) ? $emailTemplates['ticket_email_user_message'] : [];

			if ( !empty( $emailTemplate ) && isset( $emailTemplate['body'] ) )
			{
				$email_template_subject = (string) $emailTemplate['subject'];
				$email_template_body = (string) $emailTemplate['body'];

				$email_subject = strtr( $email_template_subject, $email_body_placeholder );
				$email_body = strtr( $email_template_body, $email_body_placeholder );
				$email_body = wp_kses_post( self::cleanMessageBody( $email_body ) );
				$mailtouser = self::sendEmail( $tc_email, $email_subject, $email_body );
			}

			// Change ticket url for agent and additional email notifications
			$email_body_placeholder['http://{ticket_link}'] = $ticket_url_agent;
			$email_body_placeholder['https://{ticket_link}'] = $ticket_url_agent;
			$email_body_placeholder['{ticket_link}'] = $ticket_url_agent;

			// Send email to agent
			if ( $agent_email ) {
				$emailTemplate = !empty( $emailTemplates ) && isset( $emailTemplates['ticket_email_agent_message'] ) ? $emailTemplates['ticket_email_agent_message'] : [];

				if ( !empty( $emailTemplate ) && isset( $emailTemplate['body'] ) )
				{
					$email_template_subject = (string) $emailTemplate['subject'];
					$email_template_body = (string) $emailTemplate['body'];
	
					$email_subject = strtr( $email_template_subject, $email_body_placeholder );
					$email_body = strtr( $email_template_body, $email_body_placeholder );
					$email_body = wp_kses_post( self::cleanMessageBody( $email_body ) );
					$mailtoagent = self::sendEmail( $agent_email, $email_subject, $email_body );
				}
			}

			// and additional email addresses
			if ( $departmentData !== false && !empty( $departmentData ) )
			{
				if ( $departmentData->additional_emails && ( $departmentData->additional_emails_type == 'tickets' || $departmentData->additional_emails_type == 'both' ) )
				{
					$additional_email_addresses = preg_split( "/\r\n|\n|\r/", $departmentData->additional_emails );

					$emailTemplate = !empty( $emailTemplates ) && isset( $emailTemplates['ticket_email_additional_message'] ) ? $emailTemplates['ticket_email_additional_message'] : [];

					if ( !empty( $emailTemplate ) && isset( $emailTemplate['body'] ) )
					{
						$email_template_subject = (string) $emailTemplate['subject'];
						$email_template_body = (string) $emailTemplate['body'];
		
						$email_subject = strtr( $email_template_subject, $email_body_placeholder );
						$email_body = strtr( $email_template_body, $email_body_placeholder );
						$email_body = wp_kses_post( self::cleanMessageBody( $email_body ) );
						$mailtoadditional = self::sendEmail( $additional_email_addresses, $email_subject, $email_body );
					}
				}
			}

			/**
			 * END Send emails
			 */

			/**
			 * New ticket has been created successfully
			 * What to do now? - Either redirect or show message based on settings
			 */

			$after_new_ticket_created = $config->after_new_ticket_created;
			$new_ticket_confirmation_message = $config->new_ticket_confirmation_message;
			if ( $after_new_ticket_created === 'show_message' && $new_ticket_confirmation_message )
			{
				// Change ticket url for user
				$email_body_placeholder['http://{ticket_link}'] = $ticket_url_user;
				$email_body_placeholder['https://{ticket_link}'] = $ticket_url_user;
				$email_body_placeholder['{ticket_link}'] = $ticket_url_user;

				// Show message
				self::createSession( 'wpgs_create_ticket_success', true );
				$wpgs_create_ticket_success_message = strtr( $new_ticket_confirmation_message, $email_body_placeholder );
				self::createSession( 'wpgs_create_ticket_success_message', $wpgs_create_ticket_success_message );

				// Redirect to show message
				$ticketSuccessMsgurl = site_url( $form_url );
				self::setRedirect( $ticketSuccessMsgurl, '', '' );
			}
			else
			{
				// Redirect to ticket view page
				self::setRedirect( $ticket_url_user, '', '' );
			}
		}
		else
		{
			$notice = esc_html__("Something went wrong and your ticket wasn't created. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
	}

	/**
	 * Reply Ticket
	 */
	public static function replyTicket( $data = array() )
	{
		$redirectUrl = sanitize_text_field( $_SERVER['HTTP_REFERER'] );
		if ( !is_array( $data ) )
		{
			$notice = esc_html__("Invalid request.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Get handler
		$form_id = isset( $data['wpgs_handler_id'] ) ? sanitize_text_field( $data['wpgs_handler_id'] ) : 0;
		$ticket_id = isset( $data['wpgs_ticket_id'] ) ? sanitize_text_field( $data['wpgs_ticket_id'] ) : '';
		$ticket_token = isset( $data['wpgs_ticket_auth'] ) ? sanitize_text_field( $data['wpgs_ticket_auth'] ) : '';

		// Verify if necessary data provided to process the reply
		if ( !$form_id || !$ticket_id || !$ticket_token )
		{
			$notice = esc_html__("Invalid request.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Get db
		$db = new GuestSupportDbquery();
		$agent_id_enc = self::AgentIdEncrypt();

		// Check whether or not the ticket exist
		$thisticket = $db->getTicketSubject( $ticket_id, $ticket_token );
		if ( $thisticket === false )
		{
			$notice = esc_html__("No ticket exist to process the reply.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// If ticket replied by Agent, check if the Agent really an Agent
		$isagent = self::isAgent();
		$agent_id = self::getAgentIdByDepartmentId( $thisticket->department_id );

		$agent = get_userdata( $agent_id );

		if ( $isagent && ( $agent === false || empty( $agent ) || ( $isagent != $agent_id && !current_user_can( 'administrator' ) ) ) )
		{
			$notice = esc_html__("You can't reply to this ticket as Agent, because this ticket wasn't submitted to your assigned department. If you're an Admin, login first.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		elseif ( $agent === false || empty( $agent ) )
		{
			// Verify agent user
			$notice = esc_html__("Failed to assign support agent for this ticket. Please create a new ticket.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Set new redirect url
		$redirectUrl = self::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( self::isAgent() ? '&agent=' . $agent_id_enc : '' );

		// Get form fields
		$form = self::getForm( $form_id );

		if( empty( $form ) || $form === false )
		{
			// Incase the form id was modified
			$notice = esc_html__("Invalid form id. Can't submit reply. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Get form fields
		$formFields = unserialize( $form->form_fields );

		// Get user inputs
		$message = isset( $data['message'] ) ? wp_kses_post( $data['message'] ) : '';
		$closeticket = isset( $data['closeticket'] ) ? sanitize_text_field( $data['closeticket'] ) : 0;

		if ( !$message )
		{
			$notice = esc_html__("Please enter your message to reply.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Unset used inputs
		unset(
			$data['_wpgs_submit_reply_form'],
			$data['_wp_http_referer'],
			$data['wpgs_handler'],
			$data['wpgs_ticket_id'],
			$data['wpgs_ticket_auth'],
			$data['wpgs_handler_id'],
			$data['message'],
			$data['closeticket']
		);

		// Sanitize custom fields
		if ( !empty( $data ) )
		{
			$data = self::sanitizeAndVerify( $form_id, $redirectUrl, $formFields, $data );
		}

		// Get custom fields
		$email_custom_fields = $data;
		$custom_fields = serialize( $data );

		// Get attachments
		$attachments = '';
		if ( $form->fileupload )
		{
			$attachments = self::getFiles( 'attachments' );
		}

		// Set user id, name, email and update_user_id
		$name_todb = $thisticket->name;
		$email_todb = $thisticket->email;

		$user_name_toemail = $thisticket->name;
		$user_email_toemail = $thisticket->email;

		$agent_name_toemail = $agent->display_name;
		$agent_email_toemail = $agent->user_email;

		if ( $isagent && $agent )
		{
			$name_todb = $agent->display_name;
			$email_todb = $agent->user_email;
			$user_id = $agent->ID;
			$update_user_id = 0;
			$status = 'pending';
			$user_type = 'agent';
		}
		else
		{
			if ( $thisticket->created_by == 'agent' )
			{
				$userdata = get_userdata( $thisticket->user_id );
				if ( $userdata !== false )
				{
					$name_todb = $userdata->display_name;
					$email_todb = $userdata->user_email;
					$user_name_toemail = $userdata->display_name;
					$user_email_toemail = $userdata->user_email;
				}
			}
			$user_id = $thisticket->user_id;
			$update_user_id = 0;
			$status = 'open';
			$user_type = 'user';
		}

		// Get signature - User Signature
		$user_signature = '';
		$agent_signature = '';
		$ispro = self::isPro();

		if ( $ispro && $thisticket->user_id )
		{
			$user_settings = GuestSupportAddonApp::getUserSettings( $thisticket->user_id );
			if ( $user_settings !== false && $user_settings->signature )
			{
				$user_signature = '<p>' . stripslashes( nl2br( trim( $user_settings->signature ) ) ) . '</p>';
			}
		}

		// Get signature - Agent Signature
		if ( $ispro && $agent_id )
		{
			$agent_settings = GuestSupportAddonApp::getUserSettings( $agent_id );
			if ( $agent_settings !== false && $agent_settings->signature )
			{
				$agent_signature = '<p>' . stripslashes( nl2br( trim( $agent_settings->signature ) ) ) . '</p>';
			}
		}

		// Attach signature to message
		$config = self::config();

		if ( $agent_signature && $user_type == 'agent' && ( $config->enable_signature == 'both' || $config->enable_signature == 'agent' ) )
		{
			$message .= $agent_signature;
		}
		elseif ( $user_signature && $user_type == 'user' && ( $config->enable_signature == 'both' || $config->enable_signature == 'user' ) )
		{
			$message .= $user_signature;
		}

		// Change ticket status to close if Ticket Close checked
		if ( $closeticket )
		{
			$status = 'closed';
		}

		// Ticket info
		$ticket_email_hash = md5( $email_todb );
		$message_type = 'reply';
		$department_id = $thisticket->department_id;
		$autoreply = 1;
		$published = 1;
		$timestamp = time();

		// Prepare Ticket Reply data
		$newReplydata = array(
			'ticket_id' => $ticket_id,
			'short_ticket_id' => $thisticket->short_ticket_id,
			'ticket_token' => $ticket_token,
			'form_id' => $thisticket->form_id,
			'ticket_email_hash' => $ticket_email_hash,
			'user_id' => $user_id,
			'name' => $name_todb,
			'email' => $email_todb,
			'message_type' => $message_type,
			'department_id' => $department_id,
			'subject' => '',
			'message' => $message,
			'custom_fields' => $custom_fields,
			'encrypted_message' => '',
			'status' => $status,
			'autoreply' => $autoreply,
			'published' => $published,
			'update_user_id' => $update_user_id,
			'user_type' => $user_type,
			'created_date' => $timestamp,
			'last_update_date' => $timestamp,
			'ticket_timezone' => ''
		);
		$newReplydataType = array(
			'%s',
			'%d',
			'%s',
			'%d',
			'%s',
			'%d',
			'%s',
			'%s',
			'%s',
			'%d',
			'%s',
			'%s',
			'%s',
			'%s',
			'%s',
			'%d',
			'%d',
			'%d',
			'%s',
			'%d',
			'%d',
			'%s'
		);

		// Check and upload attachments and early return if any errors before creating ticket
		$filedata = '';
		if ( $form->fileupload && !empty( $attachments ) )
		{
			$allowed_extensions_raw = preg_replace('/\s+/', '', $form->allowed_filetypes);
			$allowed_extensions = explode( ',', $allowed_extensions_raw );
			$filedata = self::uploadFiles( $redirectUrl, $form->upload_filesize, $form->upload_filelimit, $ticket_id, false, $attachments, $allowed_extensions );
		}

		// Insert ticket to db
		$insertId = $db->createReply( $newReplydata, $newReplydataType );
		if ( $insertId !== false )
		{
			// Update ticket status
			$ticketUpdateData = array(
				'status' => $status,
				'last_update_date' => $timestamp,
			);
			$ticketUpdateCondition = array(
				'id' => $thisticket->id
			);
			$ticketUpdateDataType = array(
				'%s',
				'%s'
			);
			$ticketUpdateConditionType = array(
				'%d'
			);

			$updateTicket = $db->updateTicket( $ticketUpdateData, $ticketUpdateCondition, $ticketUpdateDataType, $ticketUpdateConditionType );

			// Add attachments to database
			$filestodb = array();
			if ( $form->fileupload && !empty( $attachments ) && !empty( $filedata ) && is_array( $filedata ) )
			{
				$filestodb = self::filesToDb( $ticket_id, $insertId, $filedata );
				if ( $filestodb === false )
				{
					$notice = esc_html__("Error: Your attachment(s) was not uploaded.", "guest-support");
					self::setNotice( $notice, "error" );
				}
			}

			/**
			 * Prepare email data
			 * Send email
			 */

			// Add custom fields
			$email_body_custom_fields = '';
			if ( !empty( $email_custom_fields ) )
			{
				$count_ecf = 0;
				$html_ect = '';
				foreach ( $email_custom_fields as $name => $value ) {
					$inputname = 'field_' . $name;
					if ( $formFields[$inputname]['fieldtoform'] != 'ticket' && ( $formFields[$inputname]['addtoemail'] == 'reply' || $formFields[$inputname]['addtoemail'] == 'both' ) )
					{
						$count_ecf++;

						$eb_cf_label = '<strong>' . esc_html__( $formFields[$inputname]['label'], 'guest-support' ) . ': </strong>';
						if ( is_array( $value ) )
						{
							$fieldvalue = implode( ', ', $value );
						}
						else
						{
							$fieldvalue = str_replace( '__', ' ', $value);
						}
						$eb_cf_value = stripslashes( $fieldvalue );
						$html_ect .= '<div>' . $eb_cf_label . $eb_cf_value . '</div>';
					}
				}
				if ( $count_ecf > 0 )
				{
					$email_body_custom_fields = '<h3>' . esc_html__( "Additional Informations:", "guest-support" ) . '</h3>' . $html_ect;
				}
			}
			// Add Attachments links
			$email_body_attachments = '';
			$ticket_page = self::getTicketPageUrl();
			if ( $filestodb !== false && !empty( $filestodb ) )
			{
				$file_url = $ticket_page . '?id=' . $ticket_id . '&auth=' . $ticket_token . '&wpgs_handler=download_file&file_id=';

				$email_body_attachments = '<h3>' . esc_html__("Attachments:", "guest-support") . '</h3>';
				$email_body_attachments .= '<ul>';
				foreach ($filestodb as $file) {
					$email_body_attachments .= '<li><a href="' . $file_url . self::encrypt( $file['id'] ) . '" target="_blank">' . $file['name'] . ' (' . self::bytesToReadable( $file['size'] ) . ')</a></li>';
				}
				$email_body_attachments .= '</ul>';
			}

			// Ticket urls for user and agent
			$agent_id_enc = self::encrypt( $agent_id );

			$ticket_url_user = $ticket_page . $ticket_id . '/?auth=' . $ticket_token;
			$ticket_url_agent = $ticket_page . $ticket_id . '/?auth=' . $ticket_token . '&agent=' . $agent_id_enc;

			// Get site name
			$sitename = get_bloginfo( 'name' );
			$siteurl = home_url();

			// Get ticket subject
			$ticket_subject = esc_html( stripslashes( $thisticket->subject ) );
			$message = stripslashes( $message );

			// Is this a registered user
			$userby_email = self::userIdbyEmail( $user_email_toemail );

			$departmentData = $db->getDepartment( $department_id );

			$email_body_placeholder = array(
				'{name}'				=> $user_name_toemail,
				'{email}'				=> $user_email_toemail,
				'{is_registered_user}'	=> $userby_email !== false ? esc_html__("Yes", "guest-support") : esc_html__("No", "guest-support"),
				'{user_id}'				=> $userby_email !== false ? $userby_email->ID : 0,
				'{ip_address}'			=> self::getUserIP(),
				'http://{ticket_link}'	=> $ticket_url_user,
				'https://{ticket_link}'	=> $ticket_url_user,
				'{ticket_link}'			=> $ticket_url_user,
				'{ticket_id}'			=> $ticket_id,
				'{short_ticket_id}'		=> $thisticket->short_ticket_id,
				'{form}'				=> $form->form_name,
				'{subject}'				=> $ticket_subject,
				'{message}'				=> $message,
				'{ticket_message}'		=> $message,
				'{reply_message}'		=> $message,
				'{custom_fields}'		=> $email_body_custom_fields,
				'{attachments}'			=> $email_body_attachments,
				'{agent_name}'			=> $agent_name_toemail,
				'{agent_email}'			=> $agent_email_toemail,
				'{department}'			=> $departmentData->name,
				'{SiteName}'			=> $sitename,
				'{sitename}'			=> $sitename,
				'{siteurl}'				=> $siteurl
			);

			/**
			 * Send emails
			 */
			$emailTemplates = self::getEmailTemplates();

			if ( $user_type == 'agent' )
			{
				// Send email to user if replied by Agent
				$emailTemplate = !empty( $emailTemplates ) && isset( $emailTemplates['reply_email_user_message'] ) ? $emailTemplates['reply_email_user_message'] : [];

				if ( !empty( $emailTemplate ) && isset( $emailTemplate['body'] ) )
				{
					$email_template_subject = (string) $emailTemplate['subject'];
					$email_template_body = (string) $emailTemplate['body'];

					$email_subject = strtr( $email_template_subject, $email_body_placeholder );
					$email_body = strtr( $email_template_body, $email_body_placeholder );
					$email_body = wp_kses_post( self::cleanMessageBody( $email_body ) );
					$mailtouser = self::sendEmail( $user_email_toemail, $email_subject, $email_body );
				}
			}
			elseif ( $user_type == 'user' )
			{
				// Change ticket url for agent
				$email_body_placeholder['http://{ticket_link}'] = $ticket_url_agent;
				$email_body_placeholder['https://{ticket_link}'] = $ticket_url_agent;
				$email_body_placeholder['{ticket_link}'] = $ticket_url_agent;

				// Send email to agent if replied by User
				$emailTemplate = !empty( $emailTemplates ) && isset( $emailTemplates['reply_email_agent_message'] ) ? $emailTemplates['reply_email_agent_message'] : [];

				if ( !empty( $emailTemplate ) && isset( $emailTemplate['body'] ) )
				{
					$email_template_subject = (string) $emailTemplate['subject'];
					$email_template_body = (string) $emailTemplate['body'];

					$email_subject = strtr( $email_template_subject, $email_body_placeholder );
					$email_body = strtr( $email_template_body, $email_body_placeholder );
					$email_body = wp_kses_post( self::cleanMessageBody( $email_body ) );
					$mailtoagent = self::sendEmail( $agent_email_toemail, $email_subject, $email_body );
				}
			}

			// Send emails to additional email addresses
			if ( $departmentData !== false && !empty( $departmentData ) )
			{
				if ( $departmentData->additional_emails && ( $departmentData->additional_emails_type == 'replies' || $departmentData->additional_emails_type == 'both' ) )
				{
					$additional_email_addresses = preg_split( "/\r\n|\n|\r/", $departmentData->additional_emails );
					$emailTemplate = !empty( $emailTemplates ) && isset( $emailTemplates['reply_email_additional_message'] ) ? $emailTemplates['reply_email_additional_message'] : [];

					if ( !empty( $emailTemplate ) && isset( $emailTemplate['body'] ) )
					{
						$email_template_subject = (string) $emailTemplate['subject'];
						$email_template_body = (string) $emailTemplate['body'];

						$email_subject = strtr( $email_template_subject, $email_body_placeholder );
						$email_body = strtr( $email_template_body, $email_body_placeholder );
						$email_body = wp_kses_post( self::cleanMessageBody( $email_body ) );
						$mailtoadditional = self::sendEmail( $additional_email_addresses, $email_subject, $email_body );
					}
				}
			}

			/**
			 * END Send emails
			 */

			// Redirect back to the ticket with success message.
			$notice = esc_html__("Your reply has been added successfully.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
		else
		{
			$notice = esc_html__("Something wen't wrong and your ticket wasn't created. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
	}

	/**
	 * Update ticket status
	 */
	public static function updateTicketStatus( $ticket_id, $ticket_token, $status, $return_url = '' )
	{
		$agent_id_enc = self::AgentIdEncrypt();
		$redirectUrl = self::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( self::isAgent() ? '&agent=' . $agent_id_enc : '' );

		if ( $return_url )
		{
			$redirectUrl = base64_decode( $return_url );
		}

		if ( $status == 'closed' )
		{
			$notice = esc_html__("The ticket has been closed successfully.", "guest-support");
		}
		elseif ( $status == 'reopen' )
		{
			$notice = esc_html__("The ticket has been re-opened successfully.", "guest-support");
		}
		else
		{
			$notice = esc_html__("Invalid status.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		$db = new GuestSupportDbquery();

		// Check whether or not the ticket exist
		$thisticket = $db->getTicketSubject( $ticket_id, $ticket_token );
		if ( $thisticket === false )
		{
			$notice = esc_html__("The ticket doesn't exist to update status.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// If request is 'reopen', set status based on previous status
		if ( $status == 'reopen' )
		{
			$last_user_type = $db->lastReplyUserType( $ticket_id, $ticket_token );
			if ( $last_user_type && $last_user_type == 'user' )
			{
				$status = 'open';
			}
			elseif ( $last_user_type && $last_user_type == 'agent' ) {
				$status = 'pending';
			}
		}

		$timestamp = time();
		$data = array(
			'status' => $status,
			'last_update_date' => $timestamp,
		);
		$condition = array(
			'ticket_id' => $ticket_id,
			'ticket_token' => $ticket_token
		);
		$dataType = array(
			'%s',
			'%s'
		);
		$conditionType = array(
			'%s',
			'%s'
		);

		$updateTicket = $db->updateTicket( $data, $condition, $dataType, $conditionType );

		if ( $updateTicket === false )
		{
			$notice = esc_html__("Something wen't wrong and your request wasn't successfull. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		else
		{
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
	}

	/**
	 * Delete Ticket
	 */
	public static function deleteTicket( $ticket_id, $ticket_token, $return_url = '' )
	{
		if ( $return_url )
		{
			$redirectUrl = base64_decode( $return_url );
		}

		$db = new GuestSupportDbquery();
		$delete = $db->deleteTicket( $ticket_id, $ticket_token );

		if ( $delete === false )
		{
			$notice = esc_html__("Something went wrong and the Ticket wasn't deleted. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		elseif ( $delete === 0 )
		{
			$notice = esc_html__("The ticket wasn't found to Delete. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		else
		{
			if ( !is_admin() )
			{
				self::createSession( 'guest_support_deletedticket', $ticket_id );
			}
			$notice = esc_html__("The ticket has been successfully deleted.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
	}

	/**
	 * Delete Reply
	 */
	public static function deleteReply( $ticket_id, $ticket_token, $replyid, $return_url = '' )
	{
		if ( $return_url )
		{
			$redirectUrl = $return_url;
		}
		else
		{
			$redirectUrl = self::refererUrl();
		}

		$db = new GuestSupportDbquery();
		$delete = $db->deleteReply( $ticket_id, $ticket_token, $replyid );

		if ( $delete === false )
		{
			$notice = esc_html__("Something went wrong and the Reply wasn't deleted. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		elseif ( $delete === 0 )
		{
			$notice = esc_html__("The Reply wasn't found to Delete. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		else
		{
			$notice = esc_html__("The Reply deleted successfully.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
	}

	/**
	 * Delete Reply
	 */
	public static function deleteFile( $ticket_id, $ticket_token, $fileid, $return_url = '' )
	{
		if ( $return_url )
		{
			$redirectUrl = $return_url;
		}
		else
		{
			$redirectUrl = self::refererUrl();
		}

		// Check if file exist
		$db = new GuestSupportDbquery();
		$fileinfo = $db->getFileInfo( $ticket_id, $fileid );

		if ( $fileinfo === false )
		{
			$notice = esc_html__("Error: No file found to delete.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
			return false;
		}

		$filename = $fileinfo->file_name_enc;

		// Delete file from database
		$created_by = self::isAgent() ? 'agent' : 'user';
		$delete_record = $db->deleteFile( $ticket_id, $fileid, $created_by );

		if ( $delete_record === false )
		{
			$notice = esc_html__("Internal Error: The file wasn't deleted. Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		elseif ( $delete_record === 0 )
		{
			$notice = esc_html__("Error: No file found to delete.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}
		else
		{
			// File record deleted from Database, now delete File.
			$deletefile = self::uploadDirectory() . $filename;
			if ( file_exists( $deletefile ) )
			{
				unlink( $deletefile );
			}

			$notice = esc_html__("The File deleted successfully.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'success' );
		}
	}

	/**
	 * Get Pagination Data
	 */
	public static function paginationArgs()
	{
		$db = new GuestSupportDbquery();
		return $db->getTickets( 'paginationArgs' );
	}

	/**
	 * Sanitize data
	 */
	public static function sanitizeAndVerify( $form_id, $redirectUrl, $formFields = array(), $data = array() )
	{
		// Sanitize inputs
		foreach ( $data as $name => $value ) {
			$inputname = 'field_' . $name;
			// Sanitize
			if ( is_array( $value ) )
			{
				$data[$name] = array();
				foreach ($value as $key => $value) {
					if ( !empty( $value ) )
					{
						$data[$name][] = trim( sanitize_text_field( $value ) );
					}
				}
			}
			else
			{
				if ( $formFields[$inputname]['inputtype'] == 'email' )
				{
					$data[$name] = $value ? trim( sanitize_email( $value ) ) : '';

					if ( $name == 'email' )
					{
						$emaildomain = substr(strrchr($data[$name], "@"), 1);

						if ((bool) checkdnsrr($emaildomain, 'MX')===FALSE)
						{
							$notice = esc_html__( 'Invalid email, please use a real email address.', 'guest-support' );
							self::setRedirect( $redirectUrl, $notice, 'error' );
							break;
						}
						elseif (preg_match("/(ThrowAwayMail|DeadAddress|10MinuteMail|20MinuteMail|AirMail|Dispostable|Email Sensei|EmailThe|FilzMail|Guerrillamail|IncognitoEmail|Koszmail|Mailcatch|Mailinator|Mailnesia|MintEmail|MyTrashMail|NoClickEmail|SpamSpot|Spamavert|Spamfree24|TempEmail|Thrashmail.ws|Yopmail|EasyTrashMail|Jetable|MailExpire|MeltMail|Spambox|empomail|33Mail|E4ward|GishPuppy|InboxAlias|MailNull|Spamex|Spamgourmet|BloodyVikings|SpamControl|MailCatch|Tempomail|EmailSensei|Yopmail|Trasmail|Guerrillamail|Yopmail|boximail|ghacks|Maildrop|MintEmail|fixmail|gelitik.in|ag.us.to|mobi.web.id|fansworldwide.de|privymail.de|gishpuppy|spamevader|temp-mail.org|uroid|tempmail|soodo|deadaddress|trbvm)/i", $emaildomain))
						{
							$notice = esc_html__( 'Invalid email, please use a real email address.', 'guest-support' );
							self::setRedirect( $redirectUrl, $notice, 'error' );
							break;
						}
					}
				}
				elseif ( $formFields[$inputname]['inputtype'] == 'textarea' && $name == 'message' )
				{
					$data[$name] = $value ? trim( wp_kses_post( $value ) ) : '';
				}
				elseif ( $formFields[$inputname]['inputtype'] == 'textarea' )
				{
					$data[$name] = $value ? trim( sanitize_textarea_field( $value ) ) : '';
				}
				else
				{
					$data[$name] = $value ? trim( sanitize_text_field( $value ) ) : '';
				}
			}

			$error = 0;
			if ( isset( $formFields[$inputname]['required'] ) && $formFields[$inputname]['required'] == 1 )
			{
				if ( is_array( $data[$name] ) && empty( $data[$name] ) )
				{
					$error = 1;
				}
				elseif ( !$data[$name] && $data[$name] !== '0' )
				{
					$error = 1;
				}
			}

			// Check if any required field was empty
			if ( $error )
			{
				if ( !empty( $formFields[$inputname]['error_message'] ) )
				{
					$noticeMsg = $formFields[$inputname]['error_message'];
				}
				else
				{
					$noticeMsg = "'" . $formFields[$inputname]['label'] . "' field is required.";
				}

				$notice = esc_html__($noticeMsg, "guest-support");
				self::setRedirect( $redirectUrl, $notice, 'error' );
				break;
			}
		}
		return $data;
	}

	/**
	 * Get User Id by Department Id
	 */
	public static function getAgentIdByDepartmentId( $department_id )
	{
		$db = new GuestSupportDbquery();
		return $db->getAgentIdByDepartmentId( $department_id );
	}

	/**
	 * Get ticket subject
	 */
	public static function getTicketSubject( $ticket_id, $ticket_token )
	{
		$db = new GuestSupportDbquery();
		return $db->getTicketSubject( $ticket_id, $ticket_token );
	}

	/**
	 * Get ticket
	 */
	public static function getTicket( $ticket_id, $ticket_token )
	{
		$db = new GuestSupportDbquery();
		return $db->getTicket( $ticket_id, $ticket_token );
	}

	/**
	 * Get ticket
	 */
	public static function getOtherTickets( $ticket_id, $email, $limit )
	{
		$db = new GuestSupportDbquery();
		return $db->getOtherTickets( $ticket_id, $email, $limit );
	}

	/**
	 * Get ticket status
	 */
	public static function getStatus( $statuscode )
	{
		if ( !$statuscode )
		{
			return '';
		}
		if ( $statuscode == 'open' )
		{
			$status = esc_html__( 'Pending agent reply', 'guest-support' );
		}
		elseif ( $statuscode == 'pending' )
		{
			if ( self::isAgent() )
			{
				$status = esc_html__( 'Pending user reply', 'guest-support' );
			}
			else
			{
				$status = esc_html__( 'Pending your reply', 'guest-support' );
			}
		}
		elseif ( $statuscode == 'closed' )
		{
			$status = esc_html__( 'Closed', 'guest-support' );
		}
		return $status;
	}

	/**
	 * Get ticket form page id
	 */
	public static function getTicketFormPageId()
	{
		$db = new GuestSupportDbquery();
		return $db->getTicketFormPageId();
	}

	/**
	 * Get ticket view page id
	 */
	public static function getTicketPageId()
	{
		if ( self::$ticketpageid != 'undefined' )
		{
			return self::$ticketpageid;
		}
		$db = new GuestSupportDbquery();
		self::$ticketpageid = $db->getTicketPageId();
		return self::$ticketpageid;
	}

	/**
	 * Get ticket view page slug
	 */
	public static function getTicketPageSlug()
	{
		if ( self::$ticketpageslug != 'undefined' )
		{
			return self::$ticketpageslug;
		}
		$db = new GuestSupportDbquery();
		self::$ticketpageslug = $db->getTicketPageSlug();
		return self::$ticketpageslug;
	}

	/**
	 * Get ticket view page url
	 */
	public static function getTicketPageUrl()
	{
		if ( self::$ticketpageurl != 'undefined' )
		{
			return self::$ticketpageurl;
		}
		$db = new GuestSupportDbquery();
		self::$ticketpageurl = $db->getTicketPageUrl();
		return self::$ticketpageurl;
	}

	/**
	 * Notice id
	 */
	public static function noticeId()
	{
		if ( !self::$noticeId )
		{
			$current_time = time();
			$microseconds = microtime(true);
			$microseconds = str_replace('.', '', $microseconds);
			$uniqueid = self::otp(4);
			self::$noticeId = $current_time . $microseconds . $uniqueid;
		}
		return self::$noticeId;
	}

	/**
	 * Generate OTP
	 */
	public static function otp($length = 6)
	{
		$lengthMinus = $length - 1;
		$firstNumber = 1 . str_repeat(0, $lengthMinus);
		$secondNumber = str_repeat(9, $length);
		return random_int($firstNumber, $secondNumber);
	}

    /**
     * Generate secure password
     */
    public static function password()
    {
		$lowercase = 'abcdefghijklmnopqrstuvwxyz';
        $uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $numbers = '1234567890987654321';
        $special = '!#^*!#^**^#!';
		$keys = $lowercase . $special . $uppercase . $numbers;
		$limit = random_int(15, 19);
		return substr( str_shuffle( $keys ), 0, $limit );
    }

	/**
	 * Random string
	 */
	public static function randomString( $length, $random_lenght = true )
	{
		$chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$chars_length = strlen($chars);
		$random_string = '';

		if ( $random_lenght === true )
		{
			$minlength = (int) $length - 4;
			$length = random_int($minlength, $length);
		}

		for($i = 0; $i < $length; $i++) {
			$random_character = $chars[random_int(0, $chars_length - 1)];
			$random_string .= $random_character;
		}

		return $random_string;
	}

	/**
	 * Generate Ticket Id
	 */
	public static function ticketId()
	{
		$id = self::randomString(17);
		$db = new GuestSupportDbquery();

		// Check if the id already exist
		$checkId = $db->checkTicketId($id);
		if ( $checkId !== false )
		{
			$id = self::ticketId();
		}
		return $id;
	}

	/**
	 * Generate simple ticket id
	 */
	public static function simpleTicketId()
	{
		$db = new GuestSupportDbquery();

		// Get new inserting ticket id
		$newid = (int) $db->getNewTicketId();

		$random_length = 3;
		if ( $newid < 10 )
		{
			$newid = $newid . '00';
		}
		elseif ( $newid < 100 )
		{
			$newid = $newid . '0';
		}

		$random_numbers = self::otp( $random_length );

		return $newid . $random_numbers;
	}

	/**
	 * Check if user is agent
	 */
	public static function isAgent()
	{
		if ( self::$isagent != 'undefined' ) {
			return self::$isagent;
		}
		else
		{
			$db = new GuestSupportDbquery();
			$agents = $db->getAgents();

			// Check if user logged in and is agent
			if ( is_user_logged_in() )
			{
				$user_id = get_current_user_id();

				// Early return if it's an administrator
				if ( current_user_can( 'administrator' ) )
				{
					self::$isagent = $user_id;
					return self::$isagent;
				}
				elseif ( $agents === false )
				{
					// Retun if no agent found on the database
					self::$isagent = false;
					return self::$isagent;
				}
				elseif ( in_array( $user_id, $agents ) )
				{
					// Check if logged in user is an agent
					self::$isagent = $user_id;
					return self::$isagent;
				}
				else
				{
					// User is not agent.
					self::$isagent = false;
					return self::$isagent;
				}
			}
			else
			{
				// Get agent id from url
				$get_agentid = isset( $_GET['agent'] ) ? self::sanitizeBase64( $_GET['agent'] ) : '';

				if ( $get_agentid )
				{
					$agentid = self::decrypt( $get_agentid );

					// Check if the agen id is valid
					if ( $agentid && ( in_array( $agentid, $agents ) || user_can( $agentid, 'manage_options' ) ) )
					{
						self::$isagent = $agentid;
						return self::$isagent;
					}
					else
					{
						// Invalid agent id
						$error_msg = esc_html__("Invalid Agent Id.", "guest-support");
						die( $error_msg );
					}
				}
			}
			// User is not agent.
			self::$isagent = false;
			return self::$isagent;
		}
	}

	/**
	 * User id by email
	 */
	public static function userIdbyEmail( $email )
	{
		return get_user_by( 'email', $email );
	}

	/**
	 * Sanitize Base64
	 */
	public static function sanitizeBase64( $string )
	{
		$pattern = '/[^A-Z0-9\/+=]/i';
		$base64_string = str_replace( ' ', '+', $string );
		return (string) preg_replace($pattern, '', $base64_string);
	}

	/**
	 * KB to MB
	 */
	public static function kbtomb($kb)
	{
		$kb = (int) $kb;
		if ( $kb < 1000 )
		{
			return $kb . ' KB';
		}
		else
		{
			$mb = $kb / 1000;
			return $mb . ' MB';
		}
	}

	/**
	 * Bytes to KB and MB
	 */
	public static function bytesToReadable($bytes)
	{
		$kb = (int) $bytes / 1000;
		if ( $kb < 1000 )
		{
			return round( $kb ) . ' KB';
		}
		else
		{
			$mb = $kb / 1000;
			return round( $mb ) . ' MB';
		}
	}

	/**
	 * Upload File
	 */
	public static function filesToDb( $ticket_id, $ticket_message_id, $filedata = array() )
	{
		if ( !is_array( $filedata ) )
		{
			return false;
		}

		$time = time();
		$attach_error = 0;
		$uploadData = array();
		$count = 0;
		$created_by = self::isAgent() ? 'agent' : 'user';
		foreach ( $filedata as $file ) {
			$count++;
			$data = array(
				'file_ticket_id' => $ticket_id,
				'file_ticket_message_id' => $ticket_message_id,
				'file_name_raw' => $file['file_name_raw'],
				'file_name_enc' => $file['file_name_enc'],
				'file_size' => $file['file_size'],
				'file_created' => $time,
				'created_by' => $created_by,
			);
			$dataType = array(
				'%s',
				'%d',
				'%s',
				'%s',
				'%d',
				'%d',
				'%s'
			);

			$db = new GuestSupportDbquery();
			$inserid = $db->attachFile( $data, $dataType );

			if ( $inserid !== false )
			{
				/* $uploadData = array(
					'data' => array(
						'id'	=> $inserid,
						'name'	=> $file['file_name_raw'],
						'size'	=> $file['file_size']
					)
				); */
				$arrid = 'data_' . $count;
				$uploadData[$arrid]['id'] = $inserid;
				$uploadData[$arrid]['name'] = $file['file_name_raw'];
				$uploadData[$arrid]['size'] = $file['file_size'];
			}
			else
			{
				$attach_error++;
			}
		}

		if ( $attach_error )
		{
			return false;
		}
		return $uploadData;
	}

	public static function refererUrl()
	{
		if ( isset( $_SERVER['HTTP_REFERER'] ) && !empty( $_SERVER['HTTP_REFERER'] ) )
		{
			return sanitize_text_field( $_SERVER['HTTP_REFERER'] );
		}
		else
		{
			$error_msg = esc_html__( 'Invalid request.', 'guest-support' );
			die($error_msg);
		}
	}

	/**
	 * Download files
	 */
	public static function downloadFile( $ticket_id, $file_id )
	{
		$db = new GuestSupportDbquery();
		$fileinfo = $db->getFileInfo( $ticket_id, $file_id );

		if ( $fileinfo === false )
		{
			$error_msg = esc_html__( 'File not found.', 'guest-support' );
			die($error_msg);
		}

		$filepath = self::uploadDirectory() . $fileinfo->file_name_enc;

		if ( !file_exists( $filepath ) )
		{
			$error_msg = esc_html__( 'File not found.', 'guest-support' );
			die($error_msg);
		}

		// Download file
		@ob_end_clean();
		$filename = $fileinfo->file_name_raw;
		$user_agent = sanitize_text_field( $_SERVER["HTTP_USER_AGENT"] );
		header("Cache-Control: public, must-revalidate");
		header('Cache-Control: pre-check=0, post-check=0, max-age=0');
		if (strstr(@$user_agent, "MSIE") == false)
		{
			header("Cache-Control: no-cache");
			header("Pragma: no-cache");
		}
		header("Expires: 0");
		header("Content-Description: File Transfer");
		// header("Expires: Sat, 01 Jan 2000 01:00:00 GMT");
		header("Content-Type: application/octet-stream; charset=utf-8");
		header("Content-Length: " . (string) filesize($filepath));
		header('Content-Disposition: attachment; filename="' . $filename . '"');
		header("Content-Transfer-Encoding: binary\n");
		@readfile($filepath);
		exit;
	}

	/**
	 * Get input files
	 */
	public static function getFiles( $inputname )
	{
		$filesarray = [];

		if ( isset( $_FILES[$inputname] ) )
		{
			foreach ( $_FILES[$inputname] as $key => $value) {
				if ( is_array( $value ) ) {
					foreach ( $value as $skey => $svalue ) {
						$filesarray[$skey][$key] = $svalue;
					}
				}
			}
		}
		return $filesarray;
	}

	/**
	 * File Upload Check
	 */
	public static function uploadFiles( $redirectUrl, $allowed_total_filesize, $maximum_allowed_files, $ticket_id = '', $isprofile_picture = false, $files = array(), $allowed_extensions = array() )
	{
		if ( !is_array( $files ) || !is_array( $allowed_extensions ) )
		{
			$notice = esc_html__("Can't submit file(s). Please try again.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		$upload_dir = self::uploadDirectory();
		$filedata = [];

		if ( $isprofile_picture === true )
		{
			$upload_dir = self::profilePictureUploadDirectory();
		}

		// Check upload directory
		if ( $upload_dir === false )
		{
			$notice = esc_html__("Can't submit file(s). Upload directory isn't writable or not exist.", "guest-support");
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Check files size
		$uploaded_total_filesize = 0;
		$file_uploaded = 0;
		foreach ( $files as $file ) {
			if ($file['error'] != 4) {
				$uploaded_total_filesize += $file['size'];
				$file_uploaded++;
			}
		}

		// Check maximum allowed file uploads
		if ( $file_uploaded > $maximum_allowed_files )
		{
			$notice = esc_html( sprintf( __("Can't submit file(s). You can submit maximum %s file(s).", "guest-support"), $maximum_allowed_files ) );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		$allowed_filesize_inbyte = $allowed_total_filesize * 1000;

		if ( $uploaded_total_filesize > $allowed_filesize_inbyte ) {
			$readable_filesize = self::kbtomb( $allowed_total_filesize );
			$notice = esc_html( sprintf( __("Can't submit file(s). Maximum allowed total files size is %s.", "guest-support"), $readable_filesize ) );
			self::setRedirect( $redirectUrl, $notice, 'error' );
		}

		// Process each file
		foreach ($files as $file) {
			// proceed if has file
			if ($file['error'] != 4) {
				$file_errors = 0;
				$tmp_path = $file['tmp_name'];
				$original_name = $file['name'];
				$filename = self::safeFileName( $original_name );

				$file_name_noext = preg_replace('#\.[^.]*$#', '', $filename);
				$file_ext = self::safeFileExtension( $filename );

				// If php maximum upload limit exceded
				if ( $file['error'] == 1 )
				{
					$notice = esc_html__("Can't submit file(s). Maximum server file upload limit exceded.", "guest-support");
					self::setRedirect( $redirectUrl, $notice, 'error' );
				}

				// Check file extension after sanitization
				if ( !$file_ext )
				{
					$file_errors++;
					$notice = esc_html__("Can't submit file(s). One or more files has invalid file extension.", "guest-support");
					self::setRedirect( $redirectUrl, $notice, 'error' );
				}

				// Check if file extension is in allowed file types
				if ( !in_array( $file_ext, $allowed_extensions ) )
				{
					$file_errors++;
					$valid_extensions = implode( ', ', $allowed_extensions );
					$notice = esc_html( sprintf( __("Can't submit file(s). Allowed file types are %s.", "guest-support"), $valid_extensions ) );
					self::setRedirect( $redirectUrl, $notice, 'error' );
				}

				// Neglect other than non-alphanumeric characters, hyphens & underscores.
				$safeFileName = preg_replace(array("/[\\s]/", '/[^a-zA-Z0-9_\-]/'), array('_', ''), $file_name_noext);

				// Check if filename remains after sanitizing
				if ( !$safeFileName )
				{
					$file_errors++;
					$notice = esc_html__("Can't submit file(s). One or more files has invalid file name. Please rename your files and try again.", "guest-support");
					self::setRedirect( $redirectUrl, $notice, 'error' );
				}

				// Check for broken files
				if ($file['error'] == 3) {
					$file_errors++;
					$notice = esc_html__("Can't submit file(s). One or more files was broken. Please try again.", "guest-support");
					self::setRedirect( $redirectUrl, $notice, 'error' );
				}

				// Everything Okay, now try to upload file
				if ( $file_errors == 0 )
				{
					$file_name_raw = $safeFileName . '.' . $file_ext;
					$time = time();
					$otp = self::otp(8);

					if ( $ticket_id )
					{
						$uniqueId = $ticket_id;
					}
					else
					{
						$uniqueId = self::ticketId();
					}

					$unique_name = $safeFileName . $time . $otp  . $uniqueId . '.' . $file_ext;

                    $file_name_enc = md5( $unique_name ) . '.' . $file_ext;

					$upload_dest = $upload_dir . $file_name_enc;

					// Prepare files data
					$filedata[] = array(
						'file_name_raw' => $file_name_raw,
						'file_name_enc' => $file_name_enc,
						'file_size' => $file['size'],
					);

					if ( !move_uploaded_file( $tmp_path, $upload_dest ) )
					{
						$notice = esc_html__("Can't submit file(s) at this moment. Please try again.", "guest-support");
						self::setRedirect( $redirectUrl, $notice, 'error' );
					}
				}
			}
		}
		return $filedata;
	}

	/**
	 * Safe file extension
	 */
	public static function safeFileExtension($filename)
	{
		// String manipulation should be faster than pathinfo() on newer PHP versions.
		$dot = strrpos($filename, '.');

		if ($dot === false)
		{
			return '';
		}

		$ext = substr($filename, $dot + 1);

		// Extension cannot contain slashes.
		if (strpos($ext, '/') !== false || (DIRECTORY_SEPARATOR === '\\' && strpos($ext, '\\') !== false))
		{
			return '';
		}

		return $ext;
	}

	/**
	 * Safe file name
	 */
	public static function safeFileName($filename)
	{
		// Remove any trailing dots, as those aren't ever valid file names.
		$file = rtrim($filename, '.');

		// Try transliterating the file name using the native php function
		if (function_exists('transliterator_transliterate') && function_exists('iconv'))
		{
			// Using iconv to ignore characters that can't be transliterated
			$file = iconv("UTF-8", "ASCII//TRANSLIT//IGNORE", transliterator_transliterate('Any-Latin; Latin-ASCII', $file));
		}

		$regex = array('#(\.){2,}#', '#[^A-Za-z0-9\.\_\- ]#', '#^\.#');

		return trim(preg_replace($regex, '', $file));
	}

	/**
	 * Profile picture upload directory
	 */
	public static function profilePictureUploadDirectory()
	{
		$wp_upload_dir = wp_upload_dir();
		$upload_basedir = $wp_upload_dir['basedir'];

		$dir_path = $upload_basedir . '/guest-support-pictures';

		if ( !is_dir( $dir_path ) ) {
			$create = mkdir($dir_path, 0755, true);

			if ( $create === false )
			{
				return false;
			}
		}

		if ( is_writable( $dir_path ) ) {
			return $dir_path . '/';
		}
		else
		{
			return false;
		}
	}

	/**
	 * File Upload  path
	 */
	public static function uploadDirectory()
	{
		if ( self::$uploadDirectory != 'undefined' )
		{
			return self::$uploadDirectory;
		}

		$wp_upload_dir = wp_upload_dir();
		$upload_basedir = $wp_upload_dir['basedir'];

		$dir_path = $upload_basedir . '/guest-support';
		if ( !is_dir( $dir_path ) )
		{
			$create_dir = self::createUploadDirectory( $dir_path );
			if ( $create_dir !== false )
			{
				$upload_dir = $create_dir;
			}
			else
			{
				self::$uploadDirectory = false;
			}
		}
		else
		{
			$upload_dir = $dir_path;
		}

		// Check if upload directory is writable
		if ( is_writable( $upload_dir ) ) {
			self::$uploadDirectory = $upload_dir . '/';
		}
		else
		{
			self::$uploadDirectory = false;
		}
		return self::$uploadDirectory;
	}

	/**
	 * Create Upload Directory
	 */
	public static function createUploadDirectory( $dir_path )
	{
		// Create upload directory if not exist
		if ( !is_dir( $dir_path ) ) {
			mkdir($dir_path, 0755, true);
		}

		// Create index.php file inside upload directory
		$data = '<?php' . PHP_EOL . '//Silence is golden';

		$file = fopen($dir_path . '/index.php', 'w');
		fwrite($file, $data);
		fclose($file);

		// Create .htaccess file inside upload directory
		$data = 'Deny from all';

		$file = fopen($dir_path . '/.htaccess', 'w');
		fwrite($file, $data);
		fclose($file);

		// Check if we did it correctly, otherwise, return false.
		if ( !is_dir( $dir_path ) )
		{
			return false;
		}
		else
		{
			return $dir_path;
		}
	}

	/**
	 * Upload directory base url
	 */
	public static function uploadDirBaseurl()
	{
		$wp_upload_dir = wp_upload_dir();
		$upload_baseurl = $wp_upload_dir['baseurl'];

		if ( is_ssl() ) {
			$upload_baseurl = str_replace( 'http://', 'https://', $upload_baseurl );
		}

		return $upload_baseurl;
	}

	/**
	 * Clean Email Body
	 */
	public static function cleanMessageBody( $message )
	{
		$result = wpautop( $message );
		return stripslashes( $result );
	}

	/**
	 * Send Email
	 */
	public static function sendEmail( $to, $subject, $message, $attachments = array() )
	{
		// return false;
		$config = self::config();
		$headers = [];

		if ( $config->mailer == 'smtp' )
		{
			add_action( 'phpmailer_init', 'GuestSupportApp::smtpConfig' );
		}
		else
		{
			$headers[] = 'From: ' . $config->mail_from_name . ' <' . $config->mail_from_email . '>';
		}

		// Decode subject and message
		$subject = htmlspecialchars_decode( $subject );
		$message = htmlspecialchars_decode( $message );

		$headers[] = 'MIME-Version: 1.0';
        $headers[] = 'Content-type: text/html';
        $headers[] = 'X-Mailer: Guest Support';
		$result = wp_mail( $to, $subject, $message, $headers, $attachments );
		return $result;
	}

	/**
	 * SMTP Config
	 */
	public static function smtpConfig( $phpmailer )
	{
		$config = self::config();

		if ( ! is_object( $phpmailer ) ) {
			$phpmailer = (object) $phpmailer;
		}

		$phpmailer->Mailer     = 'smtp';
		$phpmailer->Host       = $config->smtp_host;
		$phpmailer->SMTPDebug  = 0;
		$phpmailer->SMTPAuth   = true;
		$phpmailer->Port       = $config->smtp_port;
		$phpmailer->Username   = $config->smtp_username;
		$phpmailer->Password   = $config->smtp_password;
		$phpmailer->SMTPSecure = $config->smtp_security;
		$phpmailer->From       = $config->mail_from_email;
		$phpmailer->FromName   = $config->mail_from_name;
	}

	/**
	 * Encrtypted Agent Id
	 */
	public static function AgentIdEncrypt()
	{
		if ( self::isAgent() )
		{
			$agent_id = self::isAgent();
			return self::encrypt( $agent_id );
		}
		else
		{
			return false;
		}
	}

	/**
	 * Encrtypted Agent Id
	 */
	public static function AgentIdDecrypt( $encrypted_agent_id )
	{
		return self::decrypt( $encrypted_agent_id );
	}

	/**
	 * Encrypt data
	 */
	public static function encrypt( $string )
	{
		if ( get_option('wpgs_encryption_key') === false )
		{
			$key = self::password();
			add_option( 'wpgs_encryption_key', $key );
		}

		$encryption_key = get_option('wpgs_encryption_key');

		$cipher     = 'AES-256-CBC';
        $options    = OPENSSL_RAW_DATA;
        $hash_algo  = 'sha256';
        $sha2len    = 32;
        $ivlen = openssl_cipher_iv_length($cipher);
        $iv = openssl_random_pseudo_bytes($ivlen);
        $ciphertext_raw = openssl_encrypt($string, $cipher, $encryption_key, $options, $iv);
        $hmac = hash_hmac($hash_algo, $ciphertext_raw, $encryption_key, true);
        
		$result = base64_encode( $iv . $hmac . $ciphertext_raw );
		return $result;
	}

	/**
	 * Decrypt data
	 */
	public static function decrypt( $encrypted_user_string )
	{
		$encrypted_string = base64_decode( $encrypted_user_string );
		$encryption_key = get_option('wpgs_encryption_key');

		if ( !$encryption_key )
		{
			return false;
		}

		$cipher     = 'AES-256-CBC';
        $options    = OPENSSL_RAW_DATA;
        $hash_algo  = 'sha256';
        $sha2len    = 32;
        $ivlen = openssl_cipher_iv_length($cipher);
        $iv = substr($encrypted_string, 0, $ivlen);
        $hmac = substr($encrypted_string, $ivlen, $sha2len);
        $ciphertext_raw = substr($encrypted_string, $ivlen+$sha2len);
        $original_plaintext = openssl_decrypt($ciphertext_raw, $cipher, $encryption_key, $options, $iv);
        $calcmac = hash_hmac($hash_algo, $ciphertext_raw, $encryption_key, true);

		if ( function_exists( 'hash_equals' ) ) {
            if ( hash_equals( $hmac, $calcmac ) )
			{
				return $original_plaintext;
			}
        } else {
            if ( self::verifyDecryptHash( $hmac, $calcmac ) )
			{
				return $original_plaintext;
			}
        }
	}

	// Verify decrypt hash
	private static function verifyDecryptHash($knownString, $userString)
	{
		if (function_exists('mb_strlen')) {
            $kLen = mb_strlen($knownString, '8bit');
            $uLen = mb_strlen($userString, '8bit');
        } else {
            $kLen = strlen($knownString);
            $uLen = strlen($userString);
        }
        if ($kLen !== $uLen) {
            return false;
        }
        $result = 0;
        for ($i = 0; $i < $kLen; $i++) {
            $result |= (ord($knownString[$i]) ^ ord($userString[$i]));
        }
        return 0 === $result;
	}

	/**
	 * Get time elapsed from a date time
	 */
	public static function timeElapsed( $timestring, $user_timezone = '' )
	{
		/* if ( self::isAgent() )
		{
			$timezone = wp_timezone_string();
		}
		else
		{
			$timezone = $user_timezone;
		} */

		$ticket_date = date_create(date('Y-m-d H:i:s', $timestring));
		$ticket_timestamp = $ticket_date->getTimestamp();

		// Convert the time difference to a DateInterval object for better readability
		$interval = date_diff(date_create(), date_create('@'.$ticket_timestamp));

		// Date details
		$years = $interval->format('%y');
		$months = $interval->format('%m');
		$days = $interval->format('%d');
		$hours = $interval->format('%h');
		$minutes = $interval->format('%i');

		$text_year		= esc_html__( 'year', 'guest-support' );
		$text_years		= esc_html__( 'years', 'guest-support' );
		$text_month		= esc_html__( 'month', 'guest-support' );
		$text_months	= esc_html__( 'months', 'guest-support' );
		$text_day		= esc_html__( 'day', 'guest-support' );
		$text_days		= esc_html__( 'days', 'guest-support' );
		$text_hour		= esc_html__( 'hour', 'guest-support' );
		$text_hours		= esc_html__( 'hours', 'guest-support' );
		$text_minute	= esc_html__( 'minute', 'guest-support' );
		$text_minutes	= esc_html__( 'minutes', 'guest-support' );
		$text_seconds	= esc_html__( 'Few seconds ago', 'guest-support' );
		$text_ago	= esc_html__( 'ago', 'guest-support' );

		$yearstext		= $years == 1 ? ' ' . $text_year . ' ' : ' ' . $text_years . ' ';
		$monthstext		= $months == 1 ? ' ' . $text_month . ' ' : ' ' . $text_months . ' ';
		$daystext		= $days == 1 ? ' ' . $text_day . ' ' : ' ' . $text_days . ' ';
		$hourstext		= $hours == 1 ? ' ' . $text_hour . ' ' : ' ' . $text_hours . ' ';
		$minutestext	= $minutes == 1 ? ' ' . $text_minute . ' ' : ' ' . $text_minutes . ' ';

		if ( $years < 1 )
		{
			if ( $months > 0 )
			{
				// If have month - 1 month 2 days ago
				$result = $months . $monthstext . $days . $daystext . $text_ago;
			}
			elseif ( $days > 0 )
			{
				// Doesn't have month but have days - 2 days 5 minutes ago
				$result = $days . $daystext . $hours . $hourstext . $text_ago;
			}
			elseif ( $hours > 0 )
			{
				// Doesn't have month and days but have hours - 2 hours 5 minutes ago
				$result = $hours . $hourstext . $minutes . $minutestext . $text_ago;
			}
			elseif ( $minutes > 0 )
			{
				// Doesn't have month, days and hours but have minutes - 2 minutes ago
				$result = $minutes . $minutestext . $text_ago;
			}
			else
			{
				// Doesn't have month, days, hours and minutes - Few seconds ago.
				$result = $text_seconds;
			}
		}
		else
		{
			$result = $years . $yearstext . $months . $monthstext . $text_ago;
		}

		return $result;
	}

	/**
	 * Select field placeholder
	 */
	public static function selectPlaceholder( $haystack, $needle = '--' ) {
		$length = strlen( $needle );
		$result = substr( $haystack, 0, $length ) === $needle;
		if ( $result )
		{
			$length = strlen( $needle );
			if( !$length ) {
				return true;
			}
			$result = substr( $haystack, -$length ) === $needle;
		}
		return $result;
	}

	/**
	 * Install DB Version
	 */
	public static function dbVersion() {
		return '1.0';
	}

	/**
	 * Update DB Version
	 */
	public static function dbUpdateVersion() {
		return '1.2.0';
	}

	/**
	 * Session DB Version
	 */
	public static function dbSessionVersion() {
		return '1.0';
	}

	/**
	 * Install database tables on plugin activation
	 */
	public static function installBaseDb() {
		$db_version = self::dbVersion();
		$update_db_version = self::dbUpdateVersion();
		$session_db_version = self::dbSessionVersion();
		$installed_db_version = get_option( "guest_support_db_version" );

		if ( $installed_db_version != $db_version )
		{
			$db = new GuestSupportDbquery();
			// Create necessary Tables
			$db->installBaseDbTables();

			// Insert necessary Data to created Tables
			$db->insertBaseDbData();
			update_option( 'guest_support_db_version', $db_version );
			update_option( 'guest_support_update_db_version', $update_db_version );
			update_option( 'guest_support_session_db_version', $session_db_version );
		}
	}

	/**
	 * Update database tables after this plugin updated if necessary
	 */
	public static function updateBaseDb() {
		$updated_db_version = self::dbUpdateVersion();
		$installed_db_version = get_option( "guest_support_update_db_version" );
		$session_db_version = self::dbSessionVersion();
		$installed_session_db_version = get_option( "guest_support_session_db_version" );

		if ( $installed_db_version === false )
		{
			// Update to db update 1.0
			$db = new GuestSupportDbquery();
			$db->updateBaseDbTables();
			update_option( 'guest_support_update_db_version', $updated_db_version );
			update_option( 'wpgs_setup_guide', 'email_templates' );
		}
		elseif ( $installed_db_version === '1.0' )
		{
			// Update to db update 1.2.0
			$db = new GuestSupportDbquery();
			$db->updateDatabaseV120();
			update_option( 'guest_support_update_db_version', $updated_db_version );
		}

		// Create session table
		if ( $session_db_version != $installed_session_db_version )
		{
			$db = new GuestSupportDbquery();
			$db->createSessionDbTables();
			update_option( 'guest_support_session_db_version', $installed_session_db_version );
		}
	}

	/**
	 * Setup email settings
	 */
	public static function installSettings()
	{
		$items = new stdClass();

		// Generate from email
		$site_url = site_url('', 'http');
		$site_url = str_replace( array( 'http://', 'https://', 'www.' ), '', $site_url );
		$url_part = strstr($site_url, '/', true);
		$site_domain = $url_part ? $url_part : $site_url;
		$email_address = "noreply@$site_domain";

		// Get Admin user id
		$admins = get_super_admins();
		$adminObj = get_user_by( 'login', $admins[0] );

		if ( !empty( $adminObj ) )
		{
			$admin_id = $adminObj->ID;
		}
		else
		{
			$admin_id = '';
		}

		// Set items
		$items->fromname = get_bloginfo( 'name' );
		$items->fromemail = $email_address;
		$items->admin_id = $admin_id;
		return $items;
	}

	/**
	 * Get current user's IP address
	 */
	public static function getUserIP() {
		if ( self::$_ip_address === 'undefined' )
		{
			if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
				$ip = $_SERVER['HTTP_CLIENT_IP'];
			} elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
				$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
			} else {
				$ip = $_SERVER['REMOTE_ADDR'];
			}

			// Validate the IP address
			self::$_ip_address = filter_var($ip, FILTER_VALIDATE_IP) ? $ip : '';
		}

		return self::$_ip_address;
	}

	/**
	 * Get email template by type
	 * @param string $lang Email templates language
	 */
	public static function getEmailTemplates( $lang = 'en' ) {
		$db = new GuestSupportDbquery();
		$getList = $db->listEmailTemplates();

		if ( empty( $getList ) )
		{
			return false;
		}

		$templates = [];
		foreach ( $getList as $item ) {
			if ( (int) $item->active === 1 && $item->lang === $lang )
			{
				$templates[$item->type] = array(
					'subject' => $item->subject,
					'body' => $item->template
				);
			}
		}

		return $templates;
	}

	/**
	 * Start a session using Cookies and storing the id on the database
	 */
	public static function startSession()
	{
		if ( wp_installing() === true || wp_doing_ajax() === true || wp_doing_cron() === true )
		{
			return false;
		}

		$cookie_param_name = self::cookieName();
		$cookie_param_value = self::cookieId();
		$cookie_param_expires = time() + (7 * 24 * 60 * 60);
		$cookie_param_path = '/';
		$cookie_param_domain = '';
		$cookie_param_secure = is_ssl() ? true : false;
		$cookie_param_httponly = true;

		/**
		 * Domain
		 */
		$urlhost  = parse_url( home_url(), PHP_URL_HOST );
		$cookie_domain = ltrim( $urlhost, '.' );
		// Strip leading periods, www., and port numbers from cookie domain.
		if ( strpos( $cookie_domain, 'www.' ) === 0 ) {
			$cookie_domain = substr( $cookie_domain, 4 );
		}
		$cookie_domain = explode( ':', $cookie_domain );
		$cookie_domain = '.' . $cookie_domain[0];

		if ( count( explode( '.', $cookie_domain ) ) > 2 && ! is_numeric( str_replace( '.', '', $cookie_domain ) ) ) {
			$cookie_param_domain = $cookie_domain;
		}

		$setcookie = setcookie(
			$cookie_param_name,
			$cookie_param_value,
			$cookie_param_expires,
			$cookie_param_path,
			$cookie_param_domain,
			$cookie_param_secure,
			$cookie_param_httponly
		);

		if ( $setcookie === true )
		{
			$db = new GuestSupportDbquery();
			$data = [
				'session_id' => $cookie_param_value,
				'data' => ''
			];
			$db->storeSessionId( $data );
		}
		else
		{
			$notice = esc_html__("We're sorry, but we couldn't create a cookie to process your request. This page requires the ability to store and access cookies. Please enable cookies in your browser settings and reload this page to try again.", "guest-support");
			self::setNotice( $notice, 'error' );	
		}
	}

	/**
	 * Cookie name
	 */
	public static function cookieName()
	{
		$session_name = parse_url( home_url(), PHP_URL_HOST ) . '_guest_support';
		return 'GSSESS_' . substr( hash( 'sha256', $session_name ), 0, 32 );
	}

	/**
	 * Cookie name
	 */
	public static function cookieId( $from_cookie = false )
	{
		if ( self::$_session_id !== 'undefined' )
		{
			return self::$_session_id;
		}

		$cookie_name = self::cookieName();
		if ( isset( $_COOKIE[$cookie_name] ) )
		{
			// Get the session id from cookie
			self::$_session_id = sanitize_text_field( $_COOKIE[$cookie_name] );
		}
		elseif ( $from_cookie === true )
		{
			// Return false if $from_cookie is set to true but one not found on the cookie
			self::$_session_id = false;
		}
		else
		{
			// Generate new cookie id
			$unique_number = time() . microtime(true);
			$uniqueid = self::otp(4);
			$cookie_id = $unique_number . $uniqueid . '.' . self::getUserIP();
			self::$_session_id = hash( 'sha256', $cookie_id );
		}

		return self::$_session_id;
	}

	/**
	 * Get session data in array
	 */
	public static function sessionData()
	{
		$db = new GuestSupportDbquery();
		$session_id = self::cookieId( true );

		// Return if a session was not found
		if ( !$session_id )
		{
			return [];
		}

		// Get the session data from Database
		$sessionData = $db->sessionDataQuery( $session_id );
		if ( !$sessionData )
		{
			return [];
		}

		return unserialize( $sessionData );
	}

	/**
	 * Function to create a session with a specific name and value
	 */
    public static function createSession($name, $value) {
		$session_data = self::sessionData();
		if ( empty( $session_data ) )
		{
			$session_data = [];
		}
        $session_data[$name] = $value;

		// Serialize session_data for database
		$ser_session_data = serialize( $session_data );

		$db = new GuestSupportDbquery();
		$session_id = self::cookieId( true );
		return $db->updateSessionData( $session_id, $ser_session_data );
    }

	/**
	 * Function to retrieve a session value by its name
	 */
    public static function getSession($name) {
		$session_data = self::sessionData();
        return !empty( $session_data ) && isset($session_data[$name]) ? $session_data[$name] : '';
    }

	/**
	 * Function to create a session with a specific name and value
	 */
	/* public static function createSessionForAjax($name, $value) {
		// Start session if not already started
		if (!session_id()) {
			session_start();
		}

		// Set session value
		$_SESSION[$name] = $value;

		// Close the session to ensure proper handling
		session_write_close();
	} */

	/**
	 * Function to retrieve a session value by its name
	 */
	/* public static function getSessionForAjax($name) {
		// Start session if not already started
		if (!session_id()) {
			session_start();
		}

		// Retrieve session value
		$value = isset($_SESSION[$name]) ? $_SESSION[$name] : '';

		// Close the session to ensure proper handling
		session_write_close();

		return $value;
	} */

	/**
	 * Whether to show ask for review or not
	 */
	public static function askForReview()
	{
		$config = self::config();
		$timestamp = strtotime("-7 days");
		if ( (int) $config->ask_for_review === 1 && (int) $config->install_timestamp < (int) $timestamp )
		{
			return true;
		}
		return false;
	}
	/**
	 * Disable Ask for review
	 */
	public static function disableAskForReview()
	{
		$db = new GuestSupportDbquery();

		$todb_data = array( 'value' => 0 );
		$todb_condition = array( 'name' => 'ask_for_review' );
		$todb_dataType = array( '%d' );
		$todb_conditionType = array( '%s' );

		$edit = $db->editConfig( $todb_data, $todb_condition, $todb_dataType, $todb_conditionType );
		if ( $edit === false )
		{
			return false;
		}
		return true;
	}

	/**
	 * Extended wp_kses for allowing SVG
	 */
	public static function wp_kses_extended( $content )
	{
		$kses_defaults = wp_kses_allowed_html( 'post' );
		$svg_args = array(
			'svg'   => array(
				'id'           => true,
				'class'           => true,
				'aria-hidden'     => true,
				'aria-labelledby' => true,
				'role'            => true,
				'xmlns'           => true,
				'width'           => true,
				'height'          => true,
				'viewbox'         => true // <= Must be lower case!
			),
			'use'   => array( 'href' => true ),
			'g'     => array( 'fill' => true ),
			'title' => array( 'title' => true ),
			'path'  => array( 
				'd'               => true, 
				'fill'            => true  
			)
		);

		$allowed_tags = array_merge( $kses_defaults, $svg_args );

		return wp_kses( $content, $allowed_tags );
	}
}