<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com
 */
defined('ABSPATH') or die;

class GuestSupportDbquery
{
	private $db;
	private $ticketpageid = 'undefined';
	private $ticketpageurl = 'undefined';
	private $ticketpageslug = 'undefined';

	public function __construct() {
		global $wpdb;
		$this->db = $wpdb;
	}

	/**
	 * Get global configurations
	 */
	public function getConfig()
	{
		$results = $this->db->get_results( "SELECT * FROM {$this->db->prefix}gs_tickets_config" );

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Create new configuration
	 * @param string $name Name of the configuration
	 * @param string $value Value of the configuration
	 */
	public function createNewConfiguration($name, $value)
	{
		if ( !$name || !$value )
		{
			return false;
		}

		$result = $this->db->insert( 
			$this->db->prefix . 'gs_tickets_config',
			array(
				'name' => $name,
				'value' => $value
			),
			array(
				'%s',
				'%s'
			)
		);

		if ( $result === false )
		{
			return false;
		}

		return true;
	}

	/**
	 * Edit Config
	 */
	public function editConfig( $data = array(), $condition = array(), $datatype = array(), $conditiontype = array() )
	{
		if ( !is_array( $data ) || !is_array( $condition ) || !is_array( $datatype ) || !is_array( $conditiontype ) )
		{
			return false;
		}

		$results = $this->db->update( $this->db->prefix . 'gs_tickets_config', $data, $condition, $datatype, $conditiontype );

		if ( $results === false )
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Get default form id
	 */
	public function getDefaultFormId()
	{
		$results = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT id FROM {$this->db->prefix}gs_tickets_forms 
					WHERE form_type = %s 
					ORDER BY id ASC LIMIT 1
				",
				'core'
			)
		);
		if ( !$results )
		{
			return false;
		}
		return $results;
	}

	/**
	 * Get form
	 */
	public function getForm( $id = 0 )
	{
		$sql = "SELECT * FROM {$this->db->prefix}gs_tickets_forms ";

		if ( $id )
		{
			$sql .= $this->db->prepare( "WHERE id = %d", $id );
		}
		else
		{
			$sql .= $this->db->prepare( "WHERE form_type = %s", 'core' );
		}

		$results = $this->db->get_row( $sql );

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get list of forms
	 */
	public function listForms()
	{
		$addtoquery = "";
		$limitquery = "";
		if ( !GuestSupportApp::isPro() )
		{
			$addtoquery = " WHERE a.form_type = 'core'";
			$limitquery = " LIMIT 1";
		}
		$results = $this->db->get_results( "
			SELECT 
				a.*, 
				COUNT(b.id) AS total_tickets, 
				COUNT(CASE WHEN b.status='open' THEN 1 END) AS open_tickets 
			FROM {$this->db->prefix}gs_tickets_forms a 
			LEFT JOIN {$this->db->prefix}gs_tickets b ON (b.form_id = a.id) 
			AND b.message_type = 'ticket' AND b.published = 1
			" . $addtoquery . "
			GROUP BY a.id" . $limitquery
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get list of forms without tickets counts
	 */
	public function getForms()
	{
		if ( GuestSupportApp::isPro() )
		{
			$results = $this->db->get_results( "SELECT id, form_name FROM {$this->db->prefix}gs_tickets_forms" );
		}
		else
		{
			$results = $this->db->get_results( "SELECT id, form_name FROM {$this->db->prefix}gs_tickets_forms WHERE form_type = 'core' LIMIT 1" );
		}

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get list of departments
	 */
	public function listDepartments()
	{
		if ( GuestSupportApp::isPro() )
		{
			$results = $this->db->get_results( "SELECT * FROM {$this->db->prefix}gs_tickets_departments" );
		}
		else
		{
			$results = $this->db->get_results( "SELECT * FROM {$this->db->prefix}gs_tickets_departments WHERE type = 'core' LIMIT 1" );
		}

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Form departments. Get all department assigned to this form
	 */
	public function listFormDepartments( $form_id = 0 )
	{
		if ( GuestSupportApp::isPro() )
		{
			$results = '';
			if ( $form_id )
			{
				$results = $this->db->get_results(
					$this->db->prepare(
						"
							SELECT 
								a.department_id AS id, 
								b.name, 
								b.agent_id, 
								b.additional_emails, 
								b.additional_emails_type, 
								b.type 
							FROM {$this->db->prefix}gs_tickets_department_to_forms a 
							INNER JOIN {$this->db->prefix}gs_tickets_departments b ON (b.id = a.department_id) 
							WHERE a.form_id = %d
						",
						$form_id
					)
				);
			}

			if ( !$form_id || empty( $results ) )
			{
				$results = $this->db->get_results( "SELECT * FROM {$this->db->prefix}gs_tickets_departments" );
			}
		}
		else
		{
			$results = $this->db->get_results( "SELECT * FROM {$this->db->prefix}gs_tickets_departments WHERE type = 'core' LIMIT 1" );
		}

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get department by id
	 */
	public function getDepartment( $id )
	{
		if ( !$id )
		{
			return false;
		}

		$results = $this->db->get_row(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets_departments WHERE id = %d
				",
				$id
			)
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
     * Get selected form ids for the current editing department
     */
    public function getSelectedFormidsbyDepartment( $department_id )
    {
		if ( !$department_id )
		{
			return false;
		}

		$results = $this->db->get_col(
			$this->db->prepare(
				"
					SELECT form_id FROM {$this->db->prefix}gs_tickets_department_to_forms WHERE department_id = %d
				",
				$department_id
			)
		);

        return $results;
    }

	/**
	 * Get default department id
	 */
	public function getDefaultDepartment()
	{
		$results = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT id FROM {$this->db->prefix}gs_tickets_departments WHERE type = %s LIMIT 1
				",
				'core'
			)
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Check department before edit
	 */
	public function editDepartmentCheck( $name, $id )
	{
		if ( !$id )
		{
			return false;
		}

		$results = $this->db->get_row(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets_departments
					WHERE name = %s AND id != %d
				",
				$name, $id
			)
		);
		
		if ( empty( $results ) )
		{
			return false;
		}
		
		return $results;
	}

	/**
	 * Edit departments
	 */
	public function editDepartments($data = [])
	{
		if ( !is_array( $data ) )
		{
			return false;
		}

		if ( empty( $data['dept_id'] ) )
		{
			return false;
		}

		$results = $this->db->update(
			$this->db->prefix . 'gs_tickets_departments',
			array(
				'name' => $data['dept_name'],
				'agent_id' => $data['dept_agent'],
				'additional_emails' => $data['dept_additional_emails'],
				'additional_emails_type' => $data['dept_additional_emails_type']
			),
			array( 'id' => $data['dept_id'] ),
			array(
				'%s',
				'%d',
				'%s',
				'%s'
			),
			array( '%d' )
		);

		if ( $results === false )
		{
			return false;
		}
		else
		{
			// Delete existing department to forms
			$this->db->query(
				$this->db->prepare(
					"
						DELETE FROM {$this->db->prefix}gs_tickets_department_to_forms
						WHERE department_id = %d
					",
					$data['dept_id']
				)
			);

			// Insert department to forms
			if ( isset( $data['form_ids'] ) && is_array( $data['form_ids'] ) && !empty( $data['form_ids'] ) )
			{
				foreach ( $data['form_ids'] as $form_ids ) {
					$this->db->insert(
						$this->db->prefix . 'gs_tickets_department_to_forms', 
						array(
							'department_id' => $data['dept_id'],
							'form_id' => $form_ids
						),
						array(
							'%d',
							'%d'
						)
					);
				}
			}
			return true;
		}
	}

	/**
	 * Get list of Email Templates
	 */
	public function listEmailTemplates()
	{
		$results = $this->db->get_results( "SELECT * FROM {$this->db->prefix}gs_tickets_email_templates" );

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get department by id
	 */
	public function getEmailTemplate( $id )
	{
		if ( !$id )
		{
			return false;
		}

		$results = $this->db->get_row(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets_email_templates WHERE id = %d
				",
				$id
			)
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Edit email template
	 */
	public function editEmailTemplateQuery($data = [])
	{
		if ( !is_array( $data ) )
		{
			return false;
		}

		if ( empty( $data['id'] ) )
		{
			return false;
		}

		$results = $this->db->update(
			$this->db->prefix . 'gs_tickets_email_templates',
			array(
				'title' => $data['title'],
				'subject' => $data['subject'],
				'template' => $data['template'],
				'active' => $data['active']
			),
			array( 
				'id' => $data['id'],
				'type' => $data['type']
			),
			array(
				'%s',
				'%s',
				'%s',
				'%d'
			),
			array(
				'%d',
				'%s'
			)
		);

		if ( $results === false )
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Edit forms
	 */
	public function editForms( $data = array(), $condition = array(), $datatype = array(), $conditiontype = array() )
	{
		if ( !is_array( $data ) || !is_array( $condition ) || !is_array( $datatype ) || !is_array( $conditiontype ) )
		{
			return false;
		}

		$results = $this->db->update( $this->db->prefix . 'gs_tickets_forms', $data, $condition, $datatype, $conditiontype );

		if ( $results === false )
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Store Notice
	 */
	public function storeNotice($data = array())
	{
		$created = date('U');
		$results = $this->db->insert( 
			$this->db->prefix . 'gs_tickets_notices',
			array(
				'id' => $data['id'],
				'message' => $data['message'],
				'type' => $data['type'],
				'created' => $created
			),
			array(
				'%s',
				'%s',
				'%s'
			)
		);
		return $results;
	}

	/**
	 * Get Notices
	 */
	public function getNotices($id)
	{
		if ( !$id )
		{
			return false;
		}
		
		$results = $this->db->get_results(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets_notices
					WHERE id = %s
				",
				$id
			)
		);

		// Delete displayed notices
		$validtime = date('U') - 300;

		$this->db->query(
			$this->db->prepare(
				"
					DELETE FROM {$this->db->prefix}gs_tickets_notices
					WHERE id = %s OR created < %d
				",
				$id, $validtime
			)
		);

		return $results;
	}

	/**
	 * Get last ticket id
	 */
	public function lastTicketId()
	{
		$results = $this->db->get_var("SELECT id FROM {$this->db->prefix}gs_tickets ORDER BY id DESC LIMIT 1");
		if ( !$results )
		{
			return 0;
		}
		return $results;
	}

	/**
	 * Create new ticket
	 */
	public function createTicket( $data = array(), $datatype = array() )
	{
		if ( !is_array( $data ) || !is_array( $datatype ) )
		{
			return false;
		}
		$results = $this->db->insert($this->db->prefix . 'gs_tickets', $data, $datatype);

		if ( $results )
		{
			return $this->db->insert_id;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get new inserting ticket id
	 */
	public function getNewTicketId()
	{
		$results = $this->db->get_var(
			"
				SELECT id FROM {$this->db->prefix}gs_tickets 
				ORDER BY id DESC LIMIT 1
			"
		);
		if ( !$results )
		{
			return 1;
		}
		return (int) $results + 1;
	}

	/**
	 * Create new Reply
	 */
	public function createReply( $data = array(), $datatype = array() )
	{
		if ( !is_array( $data ) || !is_array( $datatype ) )
		{
			return false;
		}
		$results = $this->db->insert($this->db->prefix . 'gs_tickets', $data, $datatype);

		if ( $results )
		{
			return $this->db->insert_id;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Create new Reply
	 */
	public function updateTicket( $data = array(), $condition = array(), $datatype = array(), $conditiontype = array() )
	{
		if ( !is_array( $data ) || !is_array( $condition ) || !is_array( $datatype ) || !is_array( $conditiontype ) )
		{
			return false;
		}
		$results = $this->db->update($this->db->prefix . 'gs_tickets', $data, $condition, $datatype, $conditiontype);

		if ( $results )
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Delete Ticket
	 */
	public function deleteTicket( $ticket_id, $ticket_token )
	{
		$ticket = $this->getTicketSubject( $ticket_id, $ticket_token );
		if ( $ticket === false )
		{
			return false;
		}

		$results = $this->db->delete( $this->db->prefix . 'gs_tickets', array( 'ticket_id' => $ticket_id ), array( '%s' ) );

		return $results;
	}

	/**
	 * Delete Ticket Reply
	 */
	public function deleteReply( $ticket_id, $ticket_token, $replyid )
	{
		$results = $this->db->delete( 
			$this->db->prefix . 'gs_tickets', 
			array( 
				'id' => $replyid,
				'ticket_id' => $ticket_id,
				'ticket_token' => $ticket_token,
			), 
			array( '%d', '%s', '%s' ) 
		);

		return $results;
	}

	/**
	 * Delete mass tickets
	 */
	public function deleteMassTickets( $ids )
	{
		$idsString = implode(",", $ids);
		$result = $this->db->query("DELETE FROM {$this->db->prefix}gs_tickets WHERE id IN ($idsString)");
		return $result;
	}

	/**
	 * Delete Ticket File
	 */
	public function deleteFile( $ticket_id, $fileid, $created_by )
	{
		if ( $created_by == 'agent' )
		{
			$results = $this->db->delete( 
				$this->db->prefix . 'gs_tickets_attachments', 
				array( 
					'file_id' => $fileid,
					'file_ticket_id' => $ticket_id
				), 
				array( '%d', '%s' )
			);
		}
		else
		{
			$results = $this->db->delete( 
				$this->db->prefix . 'gs_tickets_attachments', 
				array( 
					'file_id' => $fileid,
					'file_ticket_id' => $ticket_id,
					'created_by' => $created_by,
				), 
				array( '%d', '%s', '%s' ) 
			);
		}

		return $results;
	}

	/**
	 * Get last reply user_type
	 */
	public function lastReplyUserType( $ticket_id, $ticket_token )
	{
		$results = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT user_type FROM {$this->db->prefix}gs_tickets 
					WHERE ticket_id = %s 
					AND ticket_token = %s 
					ORDER BY id DESC LIMIT 1
				",
				$ticket_id,
				$ticket_token
			));
		if ( !$results )
		{
			return false;
		}
		return $results;
	}

	/**
	 *   Attache ticket and reply files
	 */
	public function attachFile( $data = array(), $datatype = array() )
	{
		if ( !is_array( $data ) || !is_array( $datatype ) )
		{
			return false;
		}
		$results = $this->db->insert($this->db->prefix . 'gs_tickets_attachments', $data, $datatype);

		if ( $results )
		{
			return $this->db->insert_id;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get Ticket Subject
	 */
	public function getTicketSubject( $ticket_id, $ticket_token )
	{
		if ( !$ticket_id || !$ticket_token )
		{
			return false;
		}

		$results = $this->db->get_row(
			$this->db->prepare(
				"
					SELECT 
						a.*, 
						b.name AS department_name 
					FROM {$this->db->prefix}gs_tickets a 
					LEFT JOIN {$this->db->prefix}gs_tickets_departments b ON (b.id = a.department_id) 
					WHERE a.ticket_id = %s 
					AND a.ticket_token = %s 
					AND a.message_type = %s 
					AND a.published = %d
				",
				$ticket_id,
				$ticket_token,
				'ticket',
				1
			)
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get Last Message
	 */
	public function getLastMessage( $ticket_id, $ticket_token )
	{
		if ( !$ticket_id || !$ticket_token )
		{
			return false;
		}

		$results = $this->db->get_row(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets 
					WHERE ticket_id = %s 
					AND ticket_token = %s 
					AND published = %d
					ORDER BY id DESC LIMIT 1
				",
				$ticket_id,
				$ticket_token,
				1
			)
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Edit message
	 */
	public function editMessage( $data = array(), $condition = array(), $datatype = array(), $conditiontype = array() )
	{
		if ( !is_array( $data ) || !is_array( $condition ) || !is_array( $datatype ) || !is_array( $conditiontype ) )
		{
			return false;
		}

		$results = $this->db->update( $this->db->prefix . 'gs_tickets', $data, $condition, $datatype, $conditiontype );

		if ( $results === false )
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Get Ticket
	 */
	public function getTicket( $ticket_id, $ticket_token )
	{
		if ( !$ticket_id || !$ticket_token )
		{
			return false;
		}

		// Get messages
		$messages = $this->db->get_results(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets 
					WHERE ticket_id = %s 
					AND ticket_token = %s 
					AND published = %d
				",
				$ticket_id,
				$ticket_token,
				1
			), 'OBJECT_K'
		);

		// Get files
		$files = $this->db->get_results(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets_attachments 
					WHERE file_ticket_id = %s
				",
				$ticket_id
			)
		);

		// Attach file into the message
		if ( !empty( $files ) )
		{
			foreach ( $files as $file ) {
				$message_id = $file->file_ticket_message_id;

				if ( !empty( $messages[$message_id] ) )
				{
					$message = &$messages[$message_id];

					// add the file to the array
					if ( !isset($message->files) )
					{
						$message->files = array();
					}
					$message->files[] = $file;
				}
			}
		}

		if ( !empty( $messages ) )
		{
			return $messages;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get Attachments
	 */
	public function getAttachments( $ticket_message_id )
	{
		if ( !$ticket_message_id )
		{
			return false;
		}

		$results = $this->db->get_results(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets_attachments 
					WHERE file_ticket_message_id = %d
				",
				$ticket_message_id
			)
		);

		if ( !empty( $results ) )
		{
			return $results;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get file
	 */
	public function getFileInfo( $ticket_id, $file_id )
	{
		if ( !$ticket_id || !$file_id )
		{
			return false;
		}

		$results = $this->db->get_row(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets_attachments WHERE file_id = %d AND file_ticket_id = %s
				",
				$file_id,
				$ticket_id,
			)
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get Other Ticket
	 */
	public function getOtherTickets( $ticket_id, $email, $limit = 10 )
	{
		if ( !$ticket_id || !$email )
		{
			return false;
		}

		// Get other tickets
		$sql = $this->db->prepare(
			"
				SELECT id, ticket_id, ticket_token, subject FROM {$this->db->prefix}gs_tickets 
				WHERE email = %s AND message_type = %s AND published = %d AND ticket_id != %s 
				ORDER BY created_date DESC LIMIT %d
			",
			$email,
			'ticket',
			1,
			$ticket_id,
			$limit
		);

		$results = $this->db->get_results($sql);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get tickets
	 */
	public function getTickets( $type = 'tickets', $filter = '', $order = '' )
	{
		if ( !current_user_can( 'administrator' ) )
		{
			return false;
		}

		// $select = "SELECT * FROM {$this->db->prefix}gs_tickets ";
		$query = "
			FROM {$this->db->prefix}gs_tickets AS a
			LEFT JOIN {$this->db->prefix}gs_tickets_departments AS b ON b.id = a.department_id
			LEFT JOIN {$this->db->prefix}gs_tickets_forms AS c ON c.id = a.form_id
			WHERE a.message_type = 'ticket'
			AND a.published = '1'
		";

		$config = GuestSupportApp::config();
		$items_per_page = (int) $config->tickets_per_page;
		if ( !$items_per_page )
		{
			$items_per_page = 20;
		}
		$current_page = isset( $_GET['paged'] ) ? abs( (int) sanitize_text_field( $_GET['paged'] ) ) : 1;
		$offset = ( $current_page * $items_per_page ) - $items_per_page;
		$args = new stdClass();

		// Paginstion Arguments Query
		if ( $type == 'paginationArgs' )
		{
			$total = $this->db->get_row( "
				SELECT COUNT(a.id) AS all_tickets, 
				COUNT(CASE WHEN a.status='open' THEN 1 END) AS open_tickets, 
				COUNT(CASE WHEN a.status='pending' THEN 1 END) AS pending_tickets, 
				COUNT(CASE WHEN a.status='closed' THEN 1 END) AS closed_tickets 
				{$query}" 
			);
			$args->total = $total->all_tickets;
			$args->open = $total->open_tickets;
			$args->pending = $total->pending_tickets;
			$args->closed = $total->closed_tickets;
			$args->items_per_page = $items_per_page;
			$args->current_page = $current_page;
			$args->offset = $offset;
		}

		// Get filters
		$ticket_status = isset( $_GET['ticket_status'] ) ? sanitize_text_field( $_GET['ticket_status'] ) : '';
		$sortby = isset( $_GET['sortby'] ) ? sanitize_text_field( $_GET['sortby'] ) : '';
		$searchstring = isset( $_GET['s'] ) ? sanitize_text_field( $_GET['s'] ) : '';
		$searchstring = esc_sql( $this->db->esc_like( $searchstring ) );
		$departmentId = isset( $_GET['department'] ) ? (int) sanitize_text_field( $_GET['department'] ) : 0;
		$formId = isset( $_GET['form_id'] ) ? (int) sanitize_text_field( $_GET['form_id'] ) : 0;
		$status_filter = isset( $_GET['status_filter'] ) ? sanitize_text_field( $_GET['status_filter'] ) : '';

		$parameters = [];

		// Filter by status
		if ( !$ticket_status )
		{
			$ticket_status = $status_filter;
		}
		$allowed_status = array( 'open', 'pending', 'closed' );
		if ( $ticket_status && in_array( $ticket_status, $allowed_status ) )
		{
			$query .= " AND a.status = %s";
			$parameters[] = $ticket_status;
		}

		if ( $searchstring )
		{
			$query .= " AND (a.ticket_id = %s OR a.email LIKE %s OR a.subject LIKE %s)";
			$parameters[] = $searchstring;
			$parameters[] = '%' . $searchstring . '%';
			$parameters[] = '%' . $searchstring . '%';
		}

		if ( $departmentId )
		{
			$query .= " AND a.department_id = %d";
			$parameters[] = $departmentId;
		}

		if ( $formId )
		{
			if ( $formId === -1 )
			{
				$formId = 0;
			}
			$query .= " AND a.form_id = %d";
			$parameters[] = $formId;
		}

		// Pagination Arguments
		// Create another other filter conditions above this.
		if ( $type == 'paginationArgs' )
		{
			// Get number of rows after filter
			if ( !empty( $parameters ) )
			{
				$num_rows = $this->db->get_var(
					$this->db->prepare(
						"SELECT COUNT(a.id) " . $query, $parameters
					)
				);
			}
			else
			{
				$num_rows = $this->db->get_var( "SELECT COUNT(a.id) " . $query );
			}

			$args->num_rows = $num_rows;
			return $args;
		}

		// Tickets Sort
		if ( $sortby == 'created_asc' )
		{
			$query .= " ORDER BY a.created_date ASC";
		}
		elseif ( $sortby == 'created_desc' )
		{
			$query .= " ORDER BY a.created_date DESC";
		}
		elseif ( $sortby == 'updated_asc' )
		{
			$query .= " ORDER BY a.last_update_date ASC";
		}
		elseif ( $sortby == 'updated_desc' )
		{
			$query .= " ORDER BY a.last_update_date DESC";
		}
		else
		{
			$query .= " ORDER BY 
				CASE WHEN a.status = 'open' THEN 1 
					 WHEN a.status = 'pending' THEN 2 
					 ELSE 3 
				END, 
				CASE WHEN a.status IN ('open', 'pending') THEN a.last_update_date END ASC";
		}

		// Assign params for LIMIT
		$parameters[] = $offset;
		$parameters[] = $items_per_page;

		// Tickets list query
		$tickets = $this->db->get_results(
			$this->db->prepare(
				"SELECT a.*, b.name AS department_name, c.form_name " . $query . " LIMIT %d, %d", $parameters
			), 'OBJECT_K'
		);

		/* var_dump( $parameters );
		exit; */

		if ( empty( $tickets ) )
		{
			return false;
		}

		// Get number of replies
		$replies = $this->db->get_results(
			"
				SELECT id, COUNT(*) as total_replies FROM {$this->db->prefix}gs_tickets
				WHERE published = 1
				GROUP BY ticket_id
			"
		);

		// Attach number of replies to tickets
		if ( !empty( $replies ) )
		{
			foreach ( $replies as $reply ) {
				$ticketId = $reply->id;
				$replies = (int) $reply->total_replies - 1;

				if ( isset( $tickets[$ticketId] ) )
				{
					$ticket = &$tickets[$ticketId];

					// add the file to the array
					$ticket->total_replies = $replies;
				}
			}
		}

		return $tickets;
	}

	/**
	 * Get ticket form page id
	 */
	public function getTicketFormPageId()
	{
		$pageid = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT ID FROM {$this->db->prefix}posts WHERE post_type = %s AND post_status = %s AND (post_content LIKE %s OR post_content LIKE %s OR post_content LIKE %s OR post_content LIKE %s) LIMIT 1
				",
				'page',
				'publish',
				'%[guest-support view=form]%',
				'%[guest-support view=form id=%',
				'%[guest-support view="form"]%',
				'%[guest-support view="form" id=%'
			)
		);
		if ( $pageid )
		{
			return $pageid;
		}

		return false;
	}

	/**
	 * Get ticket view page id
	 */
	public function getTicketPageId()
	{
		if ( $this->ticketpageid != 'undefined' )
		{
			return $this->ticketpageid;
		}
		$pageid = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT ID FROM {$this->db->prefix}posts WHERE post_type = %s AND post_status = %s AND (post_content LIKE %s OR post_content LIKE %s) LIMIT 1
				",
				'page',
				'publish',
				'%[guest-support view=ticket]%',
				'%[guest-support view="ticket"]%',
			)
		);
		if ( $pageid )
		{
			$this->ticketpageid = $pageid;
			return $this->ticketpageid;
		}
		else
		{
			$this->ticketpageid = false;
			return $this->ticketpageid;
		}
	}

	/**
	 * Get ticket view page slug
	 */
	public function getTicketPageSlug()
	{
		if ( $this->ticketpageslug != 'undefined' )
		{
			return $this->ticketpageslug;
		}
		$pageid = $this->getTicketPageId();

		if ( $pageid )
		{
			$page_slug = $this->db->get_var(
				$this->db->prepare(
					"
						SELECT post_name FROM {$this->db->prefix}posts WHERE ID = %d
					",
					$pageid
				)
			);
			if ( $page_slug )
			{
				$this->ticketpageslug = $page_slug;
				return $this->ticketpageslug;
			}
			else
			{
				$this->ticketpageslug = false;
				return $this->ticketpageslug;
			}
		}
		else
		{
			$this->ticketpageslug = false;
			return $this->ticketpageslug;
		}
	}

	/**
	 * Get ticket view page id
	 */
	public function getTicketPageUrl()
	{
		if ( $this->ticketpageurl != 'undefined' )
		{
			return $this->ticketpageurl;
		}
		$pageid = $this->getTicketPageId();

		if ( $pageid )
		{
			$this->ticketpageurl = get_page_link( $pageid );
			return $this->ticketpageurl;
		}
		else
		{
			$this->ticketpageurl = false;
			return $this->ticketpageurl;
		}
	}

	/**
	 * Get Agents user id
	 */
	public function getAgents()
	{
		$results = $this->db->get_col(
			"SELECT agent_id FROM {$this->db->prefix}gs_tickets_departments "
		);

		if ( empty( $results ) )
		{
			return false;
		}

		return $results;
	}

	/**
	 * Get Agent user id by Department id
	 */
	public function getAgentIdByDepartmentId( $department_id )
	{
		$results = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT agent_id FROM {$this->db->prefix}gs_tickets_departments 
					WHERE id = %d
				",
				$department_id
			)
		);
		if ( !$results )
		{
			return false;
		}
		return (int) $results;
	}

	/**
	 * Check ticket id
	 */
	public function checkTicketId($id)
	{
		if ( !$id )
		{
			return false;
		}

		$results = $this->db->get_results(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}gs_tickets
					WHERE ticket_id = %s
				",
				$id
			)
		);

		if ( !empty( $results ) )
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Start session on the Database
	 * @param array $data Provide: $data['session_id'], $data['data']
	 */
	public function storeSessionId($data = array())
	{
		// Check if the data already exist
		$dataExist = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT COUNT(*) FROM {$this->db->prefix}gs_tickets_session 
					WHERE session_id = %s 
				",
				$data['session_id']
			)
		);

		if ( (int) $dataExist > 0 )
		{
			return true;
		}

		// Insert new data
		$last_updated = time();
		$results = $this->db->insert( 
			$this->db->prefix . 'gs_tickets_session',
			array(
				'session_id' => $data['session_id'],
				'data' => $data['data'],
				'last_updated' => $last_updated
			),
			array(
				'%s',
				'%s',
				'%d'
			)
		);
		return $results;
	}

	/**
	 * Get session data for a session id
	 */
	public function sessionDataQuery( $session_id )
	{
		// Delete expired session
		$validtime = time() - (7 * 24 * 60 * 60);

		$this->db->query(
			$this->db->prepare(
				"
					DELETE FROM {$this->db->prefix}gs_tickets_session
					WHERE last_updated < %d
				",
				$validtime
			)
		);

		// Get session data
		$results = $this->db->get_var(
			$this->db->prepare(
				"
					SELECT data FROM {$this->db->prefix}gs_tickets_session 
					WHERE session_id = %s 
					LIMIT 1
				",
				$session_id
			)
		);
		if ( !$results )
		{
			return false;
		}
		return $results;
	}

	/**
	 * Update session data
	 * @param string $session_id
	 * @param string $data Serialized data
	 */
	public function updateSessionData( $session_id, $data)
	{
		if ( !$session_id || !$data )
		{
			return false;
		}
		$last_updated = time();

		$results = $this->db->update(
			$this->db->prefix . 'gs_tickets_session',
			array(
				'data' => $data,
				'last_updated' => $last_updated
			),
			array( 'session_id' => $session_id ),
			array(
				'%s',
				'%d'
			),
			array( '%s' )
		);

		if ( $results === false )
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Get Users
	 */
	public function getUsersByEmail( $email )
	{
		$email = esc_sql( $this->db->esc_like( $email ) );
		$email = trim( $email );

		$results = $this->db->get_results(
			$this->db->prepare(
				"
					SELECT * FROM {$this->db->prefix}users
					WHERE user_email LIKE %s LIMIT 5
				",
				$email . '%'
			)
		);

		if ( !empty( $results ) )
		{
			return $results;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Install plugin's base database tables
	 */
	public function installBaseDbTables()
	{
		$charset_collate = $this->db->get_charset_collate();
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';

		// Table:: gs_tickets
		$table_name = $this->db->prefix . 'gs_tickets';

		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			ticket_id varchar(100) DEFAULT '' NOT NULL,
			short_ticket_id varchar(100) DEFAULT '' NOT NULL,
			ticket_token varchar(200) DEFAULT '' NOT NULL,
			form_id int(11) NULL,
			ticket_email_hash varchar(200) DEFAULT '' NOT NULL,
			ip varchar(255) DEFAULT '',
			user_id int(11) NULL,
			name varchar(255) DEFAULT '' NOT NULL,
			email varchar(255) DEFAULT '' NOT NULL,
			message_type varchar(100) DEFAULT '' NOT NULL,
			department_id int(11) NOT NULL,
			subject varchar(500) DEFAULT '',
			message text NOT NULL,
			custom_fields text DEFAULT '' NULL,
			encrypted_message text DEFAULT '' NULL,
			status varchar(50) DEFAULT '',
			autoreply int(11) DEFAULT 0,
			published int(11) DEFAULT 1 NOT NULL,
			update_user_id int(11) DEFAULT 0,
			user_type varchar(30) DEFAULT '' NOT NULL,
			created_date varchar(30) DEFAULT '' NOT NULL,
			last_update_date varchar(30) DEFAULT '' NOT NULL,
			ticket_timezone varchar(200) DEFAULT '' NULL,
			created_by varchar(200) DEFAULT '' NULL,
			PRIMARY KEY  (id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets

		// Table:: gs_tickets_attachments
		$table_name = $this->db->prefix . 'gs_tickets_attachments';

		$sql = "CREATE TABLE $table_name (
			file_id int(11) NOT NULL AUTO_INCREMENT,
			file_ticket_id varchar(100) DEFAULT '' NOT NULL,
			file_ticket_message_id varchar(100) DEFAULT '' NOT NULL,
			file_name_raw varchar(1000) DEFAULT '' NOT NULL,
			file_name_enc varchar(100) DEFAULT '' NOT NULL,
			file_size int(11) NOT NULL,
			file_created varchar(100) DEFAULT '' NOT NULL,
			created_by varchar(50) DEFAULT '' NULL,
			PRIMARY KEY  (file_id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_attachments

		// Table:: gs_tickets_config
		$table_name = $this->db->prefix . 'gs_tickets_config';

		$sql = "CREATE TABLE $table_name (
			name varchar(200) DEFAULT '' NOT NULL,
			value text NOT NULL
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_config

		// Table:: gs_tickets_departments
		$table_name = $this->db->prefix . 'gs_tickets_departments';

		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			name varchar(200) DEFAULT '' NOT NULL,
			agent_id int(11) NOT NULL,
			additional_emails text DEFAULT '' NULL,
			additional_emails_type varchar(50) DEFAULT '' NULL,
			type varchar(100) DEFAULT '' NULL,
			PRIMARY KEY  (id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_departments

		// Table:: gs_tickets_forms
		$table_name = $this->db->prefix . 'gs_tickets_forms';

		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			form_name varchar(200) DEFAULT '' NOT NULL,
			form_fields text NOT NULL,
			form_type varchar(50) DEFAULT '' NULL,
			recaptcha int(11) NULL,
			input_class varchar(100) DEFAULT '' NULL,
			submit_text varchar(200) DEFAULT '' NULL,
			verify_otp_text varchar(200) DEFAULT '' NULL,
			submit_class varchar(200) DEFAULT '' NULL,
			fileupload int(11) NULL,
			upload_filelimit int(11) NULL,
			upload_filesize int(11) NULL,
			allowed_filetypes varchar(400) DEFAULT '' NULL,
			fileupload_label varchar(200) DEFAULT '' NULL,
			fileupload_help varchar(400) DEFAULT '' NULL,
			verify_email int(11) NULL,
			suggest_docs int(11) NULL,
			suggest_docs_cats text DEFAULT '' NULL,
			params text DEFAULT '' NULL,
			PRIMARY KEY  (id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_forms

		// Table:: gs_tickets_notices
		$table_name = $this->db->prefix . 'gs_tickets_notices';

		$sql = "CREATE TABLE $table_name (
			id varchar(100) DEFAULT '' NOT NULL,
			message text NOT NULL,
			type varchar(50) DEFAULT '' NOT NULL,
			created int(11) DEFAULT 0 NOT NULL
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_notices

		// Table:: gs_tickets_otp
		$table_name = $this->db->prefix . 'gs_tickets_otp';

		$sql = "CREATE TABLE $table_name (
			otp varchar(200) DEFAULT '' NOT NULL,
			email varchar(200) DEFAULT '' NOT NULL,
			auth varchar(200) DEFAULT '' NOT NULL,
			created int(11) DEFAULT 0 NOT NULL
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_otp

		// Table:: gs_tickets_otp_history
		$table_name = $this->db->prefix . 'gs_tickets_otp_history';

		$sql = "CREATE TABLE $table_name (
			email varchar(200) DEFAULT '' NOT NULL,
			created int(11) DEFAULT 0 NOT NULL
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_otp_history

		// Table:: gs_user_settings
		$table_name = $this->db->prefix . 'gs_user_settings';

		$sql = "CREATE TABLE $table_name (
			user_id int(11) DEFAULT 0 NOT NULL,
			picture varchar(200) DEFAULT '',
			signature varchar(400) DEFAULT ''
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_user_settings

		// Table:: gs_tickets_email_templates
		$table_name = $this->db->prefix . 'gs_tickets_email_templates';

		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			title varchar(400) DEFAULT '' NOT NULL,
			type varchar(100) DEFAULT '' NOT NULL,
			subject text DEFAULT '' NOT NULL,
			template text DEFAULT '' NOT NULL,
			lang varchar(100) DEFAULT '' NULL,
			active int(11) DEFAULT 1 NOT NULL,
			PRIMARY KEY  (id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_email_templates

		// Table:: gs_tickets_session
		$table_name = $this->db->prefix . 'gs_tickets_session';

		$sql = "CREATE TABLE $table_name (
			session_id varchar(200) DEFAULT '' NOT NULL,
			data text DEFAULT '' NULL,
			last_updated int(11) DEFAULT 0 NOT NULL,
			PRIMARY KEY  (session_id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_session

		// Table:: gs_tickets_department_to_forms
		$table_name = $this->db->prefix . 'gs_tickets_department_to_forms';

		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			department_id int(11) DEFAULT NULL,
			form_id int(11) DEFAULT NULL,
			PRIMARY KEY (id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_department_to_forms
	}

	/**
	 * Insert data to installed base database tables
	 */
	public function insertBaseDbData()
	{
		$insSettings = GuestSupportApp::installSettings();
		// Table:: gs_tickets_config
		$table_name = $this->db->prefix . 'gs_tickets_config';
		$this->db->insert( $table_name,
			array(
				'name' => 'recaptcha_secret_key',
				'value' => '',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'recaptcha_site_key',
				'value' => '',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'recaptcha_version',
				'value' => '3',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'recaptcha_score',
				'value' => '0.5',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'reopen_closed_ticket',
				'value' => 'both',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'kbs_ignore_words',
				'value' => 'how,to,what,is,can,i,do,you,add,edit',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'can_edit_replies',
				'value' => 'both',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'edit_reply_type',
				'value' => 'last',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'edit_replies_globally',
				'value' => 'no',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'can_delete_tickets',
				'value' => 'yes',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'can_delete_replies',
				'value' => 'yes',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'tickets_per_page',
				'value' => '20',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'create_ticket_url',
				'value' => '',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'mailer',
				'value' => 'wp',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'mail_from_name',
				'value' => $insSettings->fromname,
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'mail_from_email',
				'value' => $insSettings->fromemail,
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'smtp_username',
				'value' => '',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'smtp_password',
				'value' => '',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'smtp_host',
				'value' => '',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'smtp_port',
				'value' => '465',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'smtp_security',
				'value' => 'ssl',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'enable_signature',
				'value' => 'agent',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'email_signature',
				'value' => 'user',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'auto_close_tickets',
				'value' => '7',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'pre_close_email',
				'value' => '6',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'new_ticket_confirmation_message',
				'value' => "Thank you {name}, for submitting your ticket. Your Ticket ID is <strong>{short_ticket_id}</strong>. You can view your ticket and its status by following <a href=\"http://{ticket_link}\">this link</a>.",
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'after_new_ticket_created',
				'value' => 'show_message',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'ask_for_review',
				'value' => 1,
			), array( '%s', '%d' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'install_timestamp',
				'value' => time(),
			), array( '%s', '%d' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'display_login_form',
				'value' => 'yes',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'display_registration_form',
				'value' => 'yes',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'login_page_message',
				'value' => 'Log in to view your tickets.',
			), array( '%s', '%s' )
		);
		// END Table:: gs_tickets_config

		// Table:: gs_tickets_departments
		$table_name = $this->db->prefix . 'gs_tickets_departments';

		$this->db->insert( 
			$table_name, 
			array( 
				'name' => 'Customer Support',
				'agent_id' => $insSettings->admin_id,
				'additional_emails' => '',
				'additional_emails_type' => 'both',
				'type' => 'core'
			) 
		);
		// END Table:: gs_tickets_departments

		// Table:: gs_tickets_forms
		$table_name = $this->db->prefix . 'gs_tickets_forms';

		$this->db->insert( 
			$table_name, 
			array(
				'form_name' => 'Default Form',
				'form_fields' => 'a:5:{s:10:"field_name";a:12:{s:4:"name";s:4:"name";s:5:"label";s:9:"Your Name";s:11:"placeholder";s:15:"Enter your name";s:9:"inputtype";s:4:"text";s:9:"fieldtype";s:4:"core";s:7:"colsize";s:2:"50";s:7:"options";s:0:"";s:12:"hidden_value";s:0:"";s:10:"customtext";s:0:"";s:8:"required";s:1:"1";s:11:"description";s:0:"";s:13:"error_message";s:22:"Please enter your name";}s:11:"field_email";a:12:{s:4:"name";s:5:"email";s:5:"label";s:13:"Email Address";s:11:"placeholder";s:16:"user@example.com";s:9:"inputtype";s:5:"email";s:9:"fieldtype";s:4:"core";s:7:"colsize";s:2:"50";s:7:"options";s:0:"";s:12:"hidden_value";s:0:"";s:10:"customtext";s:0:"";s:8:"required";s:1:"1";s:11:"description";s:0:"";s:13:"error_message";s:31:"Please enter your email address";}s:13:"field_subject";a:12:{s:4:"name";s:7:"subject";s:5:"label";s:7:"Subject";s:11:"placeholder";s:20:"Enter ticket subject";s:9:"inputtype";s:4:"text";s:9:"fieldtype";s:4:"core";s:7:"colsize";s:3:"100";s:7:"options";s:0:"";s:12:"hidden_value";s:0:"";s:10:"customtext";s:0:"";s:8:"required";s:1:"1";s:11:"description";s:0:"";s:13:"error_message";s:27:"Please enter ticket subject";}s:16:"field_department";a:8:{s:5:"value";s:1:"1";s:4:"name";s:10:"department";s:5:"label";s:10:"Department";s:9:"inputtype";s:11:"departments";s:9:"fieldtype";s:4:"core";s:7:"colsize";s:3:"100";s:10:"showonform";s:1:"0";s:18:"default_department";s:1:"0";}s:13:"field_message";a:12:{s:4:"name";s:7:"message";s:5:"label";s:7:"Message";s:11:"placeholder";s:18:"Enter your message";s:9:"inputtype";s:8:"textarea";s:9:"fieldtype";s:4:"core";s:7:"colsize";s:3:"100";s:7:"options";s:0:"";s:12:"hidden_value";s:0:"";s:10:"customtext";s:0:"";s:8:"required";s:1:"1";s:11:"description";s:0:"";s:13:"error_message";s:25:"Please enter your message";}}',
				'form_type' => 'core',
				'recaptcha' => 0,
				'input_class' => 'wpgs-input',
				'submit_text' => 'Create Ticket',
				'verify_otp_text' => 'Continue',
				'submit_class' => 'wpgs-button wpgs-submit-button',
				'fileupload' => 1,
				'upload_filelimit' => 5,
				'upload_filesize' => 2000,
				'allowed_filetypes' => 'jpg,jpeg,png,pdf,zip',
				'fileupload_label' => 'Upload Files',
				'fileupload_help' => 'Maximum 5 files in jpg,jpeg,png,pdf,zip format and total images size not more than 2 MB.',
				'verify_email' => 0,
				'suggest_docs' => 0,
				'suggest_docs_cats' => 's:0:"";',
				'params' => 'a:11:{s:8:"top_text";s:0:"";s:21:"display_ticket_status";s:1:"1";s:18:"display_department";s:1:"0";s:28:"display_ticket_creation_date";s:1:"1";s:27:"ticket_creation_date_format";s:11:"M n, Y h:iA";s:25:"display_last_updated_date";s:1:"1";s:24:"last_updated_date_format";s:11:"M n, Y h:iA";s:21:"display_custom_fields";s:1:"1";s:21:"display_other_tickets";s:4:"both";s:18:"noof_other_tickets";s:2:"10";s:11:"bottom_text";s:0:"";}'
			)
		);
		// END Table:: gs_tickets_forms

		// Table:: gs_tickets_email_templates
		/**
		 * You can retrieve the template text from database by following:
		 * $data_with_literal = str_replace("\n", '\r\n', $values['template']);
		 * echo htmlspecialchars($data_with_literal);
		 */
		$table_name = $this->db->prefix . 'gs_tickets_email_templates';

		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a new ticket is created - Send to User',
				'type' => 'ticket_email_user_message',
				'subject' => '{subject}',
				'template' => "Hello {name}, \r\n \r\nThis is a confirmation that, we received your support ticket \"{subject}\" successfully. One of our support Agent will get back to you as soon as possible. \r\n<h4>Your Message:</h4> \r\n{ticket_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\nYour {sitename} Support Team\r\n \r\nTicket ID: {short_ticket_id}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a new ticket is created - Send to Agent',
				'type' => 'ticket_email_agent_message',
				'subject' => '{subject}',
				'template' => "Hello {agent_name}, \r\n \r\nA support ticket requires your attention. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Ticket Message:</h4> \r\n{ticket_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a new ticket is created - Send to Additional email addresses',
				'type' => 'ticket_email_additional_message',
				'subject' => '{subject}',
				'template' => "Hello, \r\n \r\nA support ticket requires your attention. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Ticket Message:</h4> \r\n{ticket_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a ticket reply - Send to User',
				'type' => 'reply_email_user_message',
				'subject' => '{subject}',
				'template' => "Hello {name}, \r\n \r\nYour support ticket \"{subject}\" has been replied to by {agent_name}. \r\n<h4>Reply Message:</h4> \r\n{reply_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\nYour {sitename} Support Team",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a ticket reply - Send to Agent',
				'type' => 'reply_email_agent_message',
				'subject' => '{subject}',
				'template' => "Hello {agent_name}, \r\n \r\nSupport ticket \"{subject}\" has been replied to by {name}. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Reply Message:</h4> \r\n{reply_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a ticket reply - Send to Additional email addresses',
				'type' => 'reply_email_additional_message',
				'subject' => '{subject}',
				'template' => "Hello, \r\n \r\nSupport ticket \"{subject}\" has been replied to by {name}. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Reply Message:</h4> \r\n{reply_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'Send pre-ticket closing notification to User',
				'type' => 'user_pre_ticket_close_message',
				'subject' => '{subject}',
				'template' => "Hello {name}, \r\n \r\nYour support ticket \"{subject}\" will close automatically after 24 hours due to inactivity if no action taken. \r\n \r\nIf your requested issue was not resolved, you can <a href=\"http://{ticket_link}\">reply here</a>. \r\n \r\n<strong>Best regards, \r\n</strong>Your {sitename} Support Team",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'Send pre-ticket closing notification to Agent',
				'type' => 'agent_pre_ticket_close_message',
				'subject' => '{subject}',
				'template' => "Dear {agent_name}, \r\n \r\nSupport ticket \"{subject}\" is still waiting for your reply. If no action taken it will close automatically after 24 hours due to inactivity. \r\n \r\nYou can <a href=\"http://{ticket_link}\">view and reply here</a>. \r\n \r\n<strong>Best regards, \r\n</strong>{sitename} - {siteurl}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'Send closure notification to User after automatically closing tickets',
				'type' => 'ticket_close_message',
				'subject' => '{subject}',
				'template' => "Hello {name}, \r\n \r\nYour support ticket \"{subject}\" has been closed automatically due to inactivity. \r\n \r\nIf your requested issue was not resolved, please submit new ticket and we will get back to you asap. \r\n \r\n<strong>Best regards,</strong> \r\nYour Support Team",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'OTP Email',
				'type' => 'otp_email',
				'subject' => 'Your {sitename} OTP',
				'template' => "Hello, \r\n \r\nYour {sitename} email verification OTP is {otp}. \r\n \r\nThis OTP is valid for the next 10 minutes. \r\n \r\nIf you didn\'t request this verification, please ignore this email. \r\n \r\n&nbsp; \r\n \r\nBest regards, \r\n \r\n{sitename} - {siteurl}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'When new user account created - Send to User',
				'type' => 'new_user_registration_email',
				'subject' => 'Your {sitename} account has been successfully created.',
				'template' => "Hello {name}, \r\n \r\nThank you for registering at {sitename}. \r\n \r\nThis is to confirm that your new account has been created successfully, and you can now log in with the following credentials: \r\n \r\nEmail: {email} \r\nPassword: {password}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		// END Table:: gs_tickets_email_templates

		// Insert data to gs_tickets_department_to_forms
		$this->insertDepartmentToForms();

		return true;
	}

	/**
	 * Update plugin's base database tables
	 * Updating DB version 1.0
	 */
	public function updateBaseDbTables()
	{
		$config = GuestSupportApp::config();
		$charset_collate = $this->db->get_charset_collate();
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		/**
		 * Update to db update 1.0
		 * Create new column
		 */
		// Table:: gs_tickets_forms
		$table_name = $this->db->prefix . 'gs_tickets_forms';
		$sql = "ALTER TABLE $table_name ADD params TEXT NULL;";
		maybe_add_column( $table_name, "params", $sql );
		// END Table:: gs_tickets_forms

		/**
		 * Update to db update 1.0
		 * CREATE NEW TABLES
		 */
		// Table:: gs_tickets_email_templates
		$table_name = $this->db->prefix . 'gs_tickets_email_templates';

		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			title varchar(400) DEFAULT '' NOT NULL,
			type varchar(100) DEFAULT '' NOT NULL,
			subject text DEFAULT '' NOT NULL,
			template text DEFAULT '' NOT NULL,
			lang varchar(100) DEFAULT '' NULL,
			active int(11) DEFAULT 1 NOT NULL,
			PRIMARY KEY  (id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_email_templates

		/**
		 * Update to db update 1.0
		 * INSERT NEW ROWS
		 */
		// Table:: gs_tickets_email_templates
		$table_name = $this->db->prefix . 'gs_tickets_email_templates';

		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a new ticket is created - Send to User',
				'type' => 'ticket_email_user_message',
				'subject' => '{subject}',
				'template' => isset( $config->ticket_email_user_message ) ? $config->ticket_email_user_message : "Hello {name}, \r\n \r\nThis is a confirmation that, we received your support ticket \"{subject}\" successfully. One of our support Agent will get back to you as soon as possible. \r\n<h4>Your Message:</h4> \r\n{ticket_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\nYour {sitename} Support Team\r\n \r\nTicket ID: {short_ticket_id}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a new ticket is created - Send to Agent',
				'type' => 'ticket_email_agent_message',
				'subject' => '{subject}',
				'template' => isset( $config->ticket_email_agent_message ) ? $config->ticket_email_agent_message : "Hello {agent_name}, \r\n \r\nA support ticket requires your attention. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Ticket Message:</h4> \r\n{ticket_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a new ticket is created - Send to Additional email addresses',
				'type' => 'ticket_email_additional_message',
				'subject' => '{subject}',
				'template' => isset( $config->ticket_email_additional_message ) ? $config->ticket_email_additional_message : "Hello, \r\n \r\nA support ticket requires your attention. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Ticket Message:</h4> \r\n{ticket_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a ticket reply - Send to User',
				'type' => 'reply_email_user_message',
				'subject' => '{subject}',
				'template' => isset( $config->reply_email_user_message ) ? $config->reply_email_user_message : "Hello {name}, \r\n \r\nYour support ticket \"{subject}\" has been replied to by {agent_name}. \r\n<h4>Reply Message:</h4> \r\n{reply_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\nYour {sitename} Support Team",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a ticket reply - Send to Agent',
				'type' => 'reply_email_agent_message',
				'subject' => '{subject}',
				'template' => isset( $config->reply_email_agent_message ) ? $config->reply_email_agent_message : "Hello {agent_name}, \r\n \r\nSupport ticket \"{subject}\" has been replied to by {name}. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Reply Message:</h4> \r\n{reply_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'After a ticket reply - Send to Additional email addresses',
				'type' => 'reply_email_additional_message',
				'subject' => '{subject}',
				'template' => isset( $config->reply_email_additional_message ) ? $config->reply_email_additional_message : "Hello, \r\n \r\nSupport ticket \"{subject}\" has been replied to by {name}. Please attend to this ticket as soon as possible. \r\n<h4>Ticket Subject:</h4> \r\n{subject}<strong> \r\n</strong> \r\n<h4>Reply Message:</h4> \r\n{reply_message} \r\n \r\n{custom_fields} \r\n \r\n{attachments} \r\n \r\n<a href=\"http://{ticket_link}\">View and reply here.</a> \r\n \r\n&nbsp; \r\n \r\n<strong>Best regards,</strong> \r\n{sitename}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'Send pre-ticket closing notification to User',
				'type' => 'user_pre_ticket_close_message',
				'subject' => '{subject}',
				'template' => "Hello {name}, \r\n \r\nYour support ticket \"{subject}\" will close automatically after 24 hours due to inactivity if no action taken. \r\n \r\nIf your requested issue was not resolved, you can <a href=\"http://{ticket_link}\">reply here</a>. \r\n \r\n<strong>Best regards, \r\n</strong>Your {sitename} Support Team",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'Send pre-ticket closing notification to Agent',
				'type' => 'agent_pre_ticket_close_message',
				'subject' => '{subject}',
				'template' => "Dear {agent_name}, \r\n \r\nSupport ticket \"{subject}\" is still waiting for your reply. If no action taken it will close automatically after 24 hours due to inactivity. \r\n \r\nYou can <a href=\"http://{ticket_link}\">view and reply here</a>. \r\n \r\n<strong>Best regards, \r\n</strong>{sitename} - {siteurl}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'Send closure notification to User after automatically closing tickets',
				'type' => 'ticket_close_message',
				'subject' => '{subject}',
				'template' => "Hello {name}, \r\n \r\nYour support ticket \"{subject}\" has been closed automatically due to inactivity. \r\n \r\nIf your requested issue was not resolved, please submit new ticket and we will get back to you asap. \r\n \r\n<strong>Best regards,</strong> \r\nYour Support Team",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'OTP Email',
				'type' => 'otp_email',
				'subject' => 'Your {sitename} OTP',
				'template' => "Hello, \r\n \r\nYour {sitename} email verification OTP is {otp}. \r\n \r\nThis OTP is valid for the next 10 minutes. \r\n \r\nIf you didn't request this verification, please ignore this email. \r\n \r\n&nbsp; \r\n \r\nBest regards, \r\n \r\n{sitename} - {siteurl}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		$this->db->insert( 
			$table_name, 
			array( 
				'title' => 'When new user account created - Send to User',
				'type' => 'new_user_registration_email',
				'subject' => 'Your {sitename} account has been successfully created.',
				'template' => "Hello {name}, \r\n \r\nThank you for registering at {sitename}. \r\n \r\nThis is to confirm that your new account has been created successfully, and you can now log in with the following credentials: \r\n \r\nEmail: {email} \r\nPassword: {password}",
				'lang' => 'en',
				'active' => 1,
			) 
		);
		// END Table:: gs_tickets_email_templates

		// Table:: gs_tickets_config
		$table_name = $this->db->prefix . 'gs_tickets_config';
		$this->db->insert( $table_name,
			array(
				'name' => 'auto_close_tickets',
				'value' => '7',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'pre_close_email',
				'value' => '6',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'new_ticket_confirmation_message',
				'value' => "Thank you {name}, for submitting your ticket. Your Ticket ID is <strong>{short_ticket_id}</strong>. You can view your ticket and its status by following <a href=\"http://{ticket_link}\">this link</a>.",
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'after_new_ticket_created',
				'value' => 'show_message',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'ask_for_review',
				'value' => 1,
			), array( '%s', '%d' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'install_timestamp',
				'value' => strtotime("-8 days"),
			), array( '%s', '%d' )
		);
		// END Table:: gs_tickets_config
		return true;
	}

	/**
	 * Update: Create session table
	 */
	public function createSessionDbTables()
	{
		$config = GuestSupportApp::config();
		$charset_collate = $this->db->get_charset_collate();
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';

		$table_name = $this->db->prefix . 'gs_tickets_session';
		$sql = "CREATE TABLE $table_name (
			session_id varchar(200) DEFAULT '' NOT NULL,
			data text DEFAULT '' NULL,
			last_updated int(11) DEFAULT 0 NOT NULL,
			PRIMARY KEY  (session_id)
		) $charset_collate;";

		dbDelta( $sql );

		return true;
	}

	/**
	 * Update plugin's base database tables
	 * Updating DB version 1.2.0
	 */
	public function updateDatabaseV120()
	{
		$config = GuestSupportApp::config();
		$charset_collate = $this->db->get_charset_collate();
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';

		/**
		 * Create new column on the gs_tickets
		 */
		// Table:: gs_tickets_forms
		$table_name = $this->db->prefix . 'gs_tickets';
		$sql = "ALTER TABLE $table_name ADD short_ticket_id varchar(100) DEFAULT '' NULL;";
		maybe_add_column( $table_name, "short_ticket_id", $sql );
		// END Table:: gs_tickets_forms

		/**
		 * Create table: gs_tickets_department_to_forms
		 */
		// Table:: gs_tickets_department_to_forms
		$table_name = $this->db->prefix . 'gs_tickets_department_to_forms';

		$sql = "CREATE TABLE $table_name (
			id int(11) NOT NULL AUTO_INCREMENT,
			department_id int(11) DEFAULT NULL,
			form_id int(11) DEFAULT NULL,
			PRIMARY KEY (id)
		) $charset_collate;";

		dbDelta( $sql );
		// END Table:: gs_tickets_department_to_forms

		// Insert data to gs_tickets_department_to_forms
		$this->insertDepartmentToForms();

		// Table:: gs_tickets_config
		$table_name = $this->db->prefix . 'gs_tickets_config';
		$this->db->insert( $table_name,
			array(
				'name' => 'display_login_form',
				'value' => 'yes',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'display_registration_form',
				'value' => 'yes',
			), array( '%s', '%s' )
		);
		$this->db->insert( $table_name,
			array(
				'name' => 'login_page_message',
				'value' => 'Log in to view your tickets.',
			), array( '%s', '%s' )
		);
		// END Table:: gs_tickets_config
		return true;
	}

	/**
	 * Insert to gs_tickets_department_to_forms
	 */
	public function insertDepartmentToForms()
	{
		// Get number of records
		$records = $this->db->get_var("SELECT COUNT(*) FROM {$this->db->prefix}gs_tickets_department_to_forms");

		if ( (int) $records < 1 )
		{
			// Get departments
			$result_departments = $this->listDepartments();

			// Get forms
			$result_forms = $this->listForms();

			// Assign departments to forms
			if ( !empty( $result_departments ) && !empty( $result_forms ) )
			{
				foreach ( $result_departments as $department ) {
					foreach ( $result_forms as $form ) {
						$this->db->insert( 
							$this->db->prefix . 'gs_tickets_department_to_forms',
							array(
								'department_id' => $department->id,
								'form_id' => $form->id,
							),
							array(
								'%d',
								'%d'
							)
						);
					}
				}
			}
		}
	}
}
