<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com
 */
defined('ABSPATH') or die;

$ispro = GuestSupportApp::isPro();
$isagent = GuestSupportApp::isAgent();
$agent_id_enc = GuestSupportApp::AgentIdEncrypt();

if ( $handler == 'add_department' || $handler == 'edit_department' )
{
    /**
     * Add or edit departments
     */
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    $redirectUrl = admin_url( 'admin.php?page=guest-support-departments' );
    if ( $handler == 'add_department' && !$ispro )
    {
        $notice = esc_html__("You need PRO addon to add new Departments.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
    // Verify nonce
	if ( !isset( $_POST['_wpnonce_department_form'] ) || !wp_verify_nonce( $_POST['_wpnonce_department_form'], '_wpnonce_department_form_action' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    // Get post data
    $type = isset( $_GET['type'] ) ? sanitize_text_field( $_GET['type'] ) : '';
    $dept_id = isset( $_GET['dept_id'] ) ? sanitize_text_field( $_GET['dept_id'] ) : '';

    $dept_name = isset( $_POST['dept_name'] ) ? sanitize_text_field( $_POST['dept_name'] ) : '';
    $applicable_forms = isset( $_POST['applicable_forms'] ) ? $_POST['applicable_forms'] : [];
	$dept_agent = isset( $_POST['dept_agent'] ) ? (int) sanitize_text_field( $_POST['dept_agent'] ) : '';
	$dept_additional_emails = isset( $_POST['dept_additional_emails'] ) ? sanitize_textarea_field( $_POST['dept_additional_emails'] ) : '';
	$dept_additional_emails_type = isset( $_POST['dept_additional_emails_type'] ) ? sanitize_text_field( $_POST['dept_additional_emails_type'] ) : '';

    $redirectUrl = admin_url( 'admin.php?page=guest-support-departments' . ( $dept_id ? '&type=edit&dept_id=' . $dept_id : '&type=add-new' ) );

    // Department name and agent id should not empty
	if ( !$dept_name )
	{
        $notice = esc_html__("Empty 'Department Name' not allowed.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}
	elseif ( empty( $applicable_forms ) ) {
		$notice = esc_html__("Please select applicable forms.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}
	elseif ( !$dept_agent )
	{
        $notice = esc_html__("Error: You must select a 'Support Agent' for new Department.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

	// Sanitize applicable form ids
	$cleanFormIds = [];
	foreach ( $applicable_forms as $formid ) {
		$int_formid = (int) $formid;
		if ( $int_formid )
		{
			$cleanFormIds[] = $int_formid;
		}
	}

    // Process the udpate
    $data = array(
        'dept_name' => $dept_name,
        'form_ids' => $cleanFormIds,
        'dept_agent' => $dept_agent,
        'dept_additional_emails' => $dept_additional_emails,
        'dept_additional_emails_type' => $dept_additional_emails_type
    );

    if ( $type == 'add-new' && $ispro )
    {
        GuestSupportAddonApp::addDepartment( $data );
    }
    elseif ( $type == 'edit' && $dept_id )
    {
        $data['dept_id'] = $dept_id;
        GuestSupportApp::editDepartment( $data );
    }
}
elseif ( $handler == 'delete_departments' )
{
    /**
     * Delete department
     */
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    $redirectUrl = admin_url( 'admin.php?page=guest-support-departments' );
    if ( $ispro )
    {
        $type = isset( $_GET['type'] ) ? sanitize_text_field( $_GET['type'] ) : '';
        $dept_id = isset( $_GET['dept_id'] ) ? sanitize_text_field( $_GET['dept_id'] ) : '';

        if ( $type == 'delete' && $dept_id )
        {
            GuestSupportAddonApp::deleteDepartment( $dept_id );
        }
    }
    else
    {
        $notice = esc_html__("You need PRO addon to delete Departments.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
}
elseif ( $handler == 'add_form' )
{
    /**
     * Add new ticket forms
     */
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    $redirectUrl = admin_url( 'admin.php?page=guest-support-forms' );
    // Verify nonce
	if ( !isset( $_POST['_wpnonce_forms_form'] ) || !wp_verify_nonce( $_POST['_wpnonce_forms_form'], '_wpnonce_forms_form_action' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    if ( !$ispro )
    {
        $notice = esc_html__("You need PRO addon to add new form.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    // Form name and id are required
    $form_name = isset( $_POST['form_name'] ) ? sanitize_text_field( $_POST['form_name'] ) : '';
    if ( !$form_name )
    {
        $notice = esc_html__("Invalid request.", "guest-support");
		$redirectUrl = admin_url( 'admin.php?page=guest-support-forms' );
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    $data = filter_var_array( $_POST );
    // Update form
    GuestSupportAddonApp::AddForm( $data );
}
elseif ( $handler == 'edit_form' )
{
    /**
     * Edit ticket forms
     */
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    // Verify nonce
	if ( !isset( $_POST['_wpnonce_forms_form'] ) || !wp_verify_nonce( $_POST['_wpnonce_forms_form'], '_wpnonce_forms_form_action' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
		$redirectUrl = admin_url( 'admin.php?page=guest-support-forms' );
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    // Form name and id are required
    $form_name = isset( $_POST['form_name'] ) ? sanitize_text_field( $_POST['form_name'] ) : '';
    $form_id = isset( $_GET['form_id'] ) ? sanitize_text_field( $_GET['form_id'] ) : '';
    if ( !$form_name || !$form_id )
    {
        $notice = esc_html__("Invalid request.", "guest-support");
		$redirectUrl = admin_url( 'admin.php?page=guest-support-forms' );
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    $data = filter_var_array( $_POST );
    // Update form
    GuestSupportApp::editForms( $form_id, $data );
}
elseif ( $handler == 'delete_form' )
{
    /**
     * Delete department
     */
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    $redirectUrl = admin_url( 'admin.php?page=guest-support-forms' );
    if ( !$ispro )
    {
        $notice = esc_html__("You need PRO addon to delete Forms.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
    $type = isset( $_GET['type'] ) ? sanitize_text_field( $_GET['type'] ) : '';
    $form_id = isset( $_GET['form_id'] ) ? sanitize_text_field( $_GET['form_id'] ) : '';

    if ( $type == 'delete' && $form_id )
    {
        GuestSupportAddonApp::deleteForm( $form_id );
    }
}
elseif ( $handler == 'wpgs_settings_form' )
{
    /**
     * Settings
     */
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    // Verify nonce
	if ( !isset( $_POST['_wpnonce_settings_form'] ) || !wp_verify_nonce( $_POST['_wpnonce_settings_form'], '_wpnonce_settings_form_action' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    $data = filter_var_array( $_POST );
    GuestSupportApp::editConfig( $data );
}
elseif ( $handler == 'guest_support_submit_ticket' )
{
    /**
     * Create new Ticket
     */
    // Verify nonce
	if ( !isset( $_POST['_wpgs_submit_ticket_form'] ) || !wp_verify_nonce( $_POST['_wpgs_submit_ticket_form'], '_wpgs_submit_ticket_form_83598274952' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    $data = filter_var_array( $_POST );
    GuestSupportApp::createTicket( $data );
}
elseif ( $handler == 'guest_support_submit_reply' )
{
    /**
     * Create Ticket Reply
     */
    // Verify nonce
	if ( !isset( $_POST['_wpgs_submit_reply_form'] ) || !wp_verify_nonce( $_POST['_wpgs_submit_reply_form'], '_wpgs_submit_reply_form_957862050274' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    $data = filter_var_array( $_POST );
    GuestSupportApp::replyTicket( $data );
}
elseif ( $handler == 'download_file' )
{
    global $wp_query;

    $ticket_id = isset( $wp_query->query_vars['id'] ) ? $wp_query->query_vars['id'] : '';
    $enc_file_id = isset( $_GET['file_id'] ) ? GuestSupportApp::sanitizeBase64( $_GET['file_id'] ) : '';

    $file_id = GuestSupportApp::decrypt( $enc_file_id );

    if ( !$ticket_id )
    {
        $ticket_id = isset( $_GET['id'] ) ? sanitize_text_field( $_GET['id'] ) : '';
    }

    if ( !$ticket_id || !$file_id )
    {
        $notice = esc_html__("Invalid request.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    GuestSupportApp::downloadFile( $ticket_id, $file_id );
}
elseif ( $handler == 'closeticket' )
{
    global $wp_query;

    $ticket_id = isset( $wp_query->query_vars['id'] ) ? $wp_query->query_vars['id'] : '';
    $ticket_token = isset( $_GET['auth'] ) ? sanitize_text_field( $_GET['auth'] ) : '';
    $return_url = isset( $_GET['return'] ) ? GuestSupportApp::sanitizeBase64( $_GET['return'] ) : '';

    if ( !$ticket_id )
    {
        $ticket_id = isset( $_GET['id'] ) ? sanitize_text_field( $_GET['id'] ) : '';
    }

    if ( !$ticket_id || !$ticket_token )
    {
        $notice = esc_html__("Invalid request here.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    if ( !$return_url )
    {
        $return_url = GuestSupportApp::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( $isagent ? '&agent=' . $agent_id_enc : '' );
        $return_url = base64_encode( $return_url );
    }

    GuestSupportApp::updateTicketStatus( $ticket_id, $ticket_token, 'closed', $return_url );
}
elseif ( $handler == 'reopenticket' )
{
    global $wp_query;

    $ticket_id = isset( $wp_query->query_vars['id'] ) ? $wp_query->query_vars['id'] : '';
    $ticket_token = isset( $_GET['auth'] ) ? sanitize_text_field( $_GET['auth'] ) : '';
    $return_url = isset( $_GET['return'] ) ? GuestSupportApp::sanitizeBase64( $_GET['return'] ) : '';

    if ( !$ticket_id )
    {
        $ticket_id = isset( $_GET['id'] ) ? sanitize_text_field( $_GET['id'] ) : '';
    }

    if ( !$ticket_id || !$ticket_token )
    {
        $notice = esc_html__("Invalid request.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    if ( !$return_url )
    {
        $return_url = GuestSupportApp::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( $isagent ? '&agent=' . $agent_id_enc : '' );
        $return_url = base64_encode( $return_url );
    }

    GuestSupportApp::updateTicketStatus( $ticket_id, $ticket_token, 'reopen', $return_url );
}
elseif ( $handler == 'deleteticket' )
{
    if ( current_user_can( 'administrator' ) || ( $isagent && GuestSupportApp::config()->can_delete_tickets == 'yes' ) )
    {
        global $wp_query;

        $ticket_id = isset( $wp_query->query_vars['id'] ) ? $wp_query->query_vars['id'] : '';
        $ticket_token = isset( $_GET['auth'] ) ? sanitize_text_field( $_GET['auth'] ) : '';
        $return_url = isset( $_GET['return'] ) ? GuestSupportApp::sanitizeBase64( $_GET['return'] ) : '';

        if ( !$ticket_id )
        {
            $ticket_id = isset( $_GET['id'] ) ? sanitize_text_field( $_GET['id'] ) : '';
        }

        if ( !$ticket_id || !$ticket_token )
        {
            $notice = esc_html__("Invalid request.", "guest-support");
            $redirectUrl = GuestSupportApp::refererUrl();
            GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
        }

        if ( !$return_url )
        {
            $return_url = GuestSupportApp::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( $isagent ? '&agent=' . $agent_id_enc : '' );
            $return_url = base64_encode( $return_url );
        }

        GuestSupportApp::deleteTicket( $ticket_id, $ticket_token, $return_url );
    }
    else
    {
        $notice = esc_html__("You're not allowed to Delete Ticket.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
        GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
}
elseif ( $handler == 'deletereply' )
{
    if ( current_user_can( 'administrator' ) || ( $isagent && GuestSupportApp::config()->can_delete_replies == 'yes' ) )
    {
        $ticket_id = isset( $_GET['id'] ) ? sanitize_text_field( $_GET['id'] ) : '';
        $ticket_token = isset( $_GET['auth'] ) ? sanitize_text_field( $_GET['auth'] ) : '';

        $enc_replyid = isset( $_GET['replyid'] ) ? GuestSupportApp::sanitizeBase64( $_GET['replyid'] ) : '';
        $replyid = GuestSupportApp::decrypt( $enc_replyid );

        if ( !$ticket_id || !$replyid || !$ticket_token )
        {
            $notice = esc_html__("Invalid request.", "guest-support");
            $redirectUrl = GuestSupportApp::refererUrl();
            GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
        }

        $return_url = GuestSupportApp::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( $isagent ? '&agent=' . $agent_id_enc : '' );

        GuestSupportApp::deleteReply( $ticket_id, $ticket_token, $replyid, $return_url );
    }
    else
    {
        $notice = esc_html__("You're not allowed to Delete Reply.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
        GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
}
elseif ( $handler == 'deletefile' )
{
    $ticket_id = isset( $_GET['id'] ) ? sanitize_text_field( $_GET['id'] ) : '';
    $ticket_token = isset( $_GET['auth'] ) ? sanitize_text_field( $_GET['auth'] ) : '';

    $enc_fileid = isset( $_GET['fileid'] ) ? GuestSupportApp::sanitizeBase64( $_GET['fileid'] ) : '';
    $fileid = GuestSupportApp::decrypt( $enc_fileid );

    if ( !$ticket_id || !$fileid || !$ticket_token )
    {
        $notice = esc_html__("Invalid request.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
        GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    $return_url = GuestSupportApp::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( $isagent ? '&agent=' . $agent_id_enc : '' );

    GuestSupportApp::deleteFile( $ticket_id, $ticket_token, $fileid, $return_url );
}
elseif ( $handler == 'wpgs_create_ticket_by_agent' )
{
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    $redirectUrl = admin_url( 'admin.php?page=guest-support' );
    if ( !isset( $_POST['_wpgs_create_ticket_by_agent'] ) || !wp_verify_nonce( $_POST['_wpgs_create_ticket_by_agent'], '_wpgs_create_ticket_by_agent_649276492832' ) )
    {
		$notice = esc_html__("Unauthorized request.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    if ( !$ispro )
    {
        $notice = esc_html__("You need PRO addon to use this feature.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    $redirectUrl = admin_url( 'admin.php?page=guest-support&type=create-ticket' );

    $userid = isset( $_POST['userid'] ) ? (int) sanitize_text_field( $_POST['userid'] ) : 0;
    $subject = isset( $_POST['subject'] ) ? sanitize_text_field( $_POST['subject'] ) : '';
    $department = isset( $_POST['department'] ) ? (int) sanitize_text_field( $_POST['department'] ) : 0;
    $message = isset( $_POST['message'] ) ? wp_kses_post( $_POST['message'] ) : '';
    $timezone = '';

    $message = str_replace( '<p><br data-mce-bogus="1"></p>', '', $message );

    if ( !$userid )
    {
        $notice = esc_html__("Error: Please select a registered user.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
    elseif ( !trim( $subject ) )
    {
        $notice = esc_html__("Error: You must enter a Subject for this ticket.", "guest-support");
        GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
    elseif ( !$department )
    {
        $notice = esc_html__("Error: You must select a Department.", "guest-support");
        GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
    elseif ( !trim( $message ) )
    {
        $notice = esc_html__("Error: You must add message to create ticket.", "guest-support");
        GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }
    else
    {
        $data = array(
            'userid' => $userid,
            'subject' => $subject,
            'department' => $department,
            'message' => $message,
            'timezone' => '',
        );
        $db = new GuestSupportAddonDbquery();
        $createTicket = $db->createTicketByAgent( $data );
        if ( $createTicket !== false )
        {
            $notice = esc_html__("New ticket created successfully.", "guest-support");
            GuestSupportApp::setRedirect( $redirectUrl, $notice, 'success' );
        }
        else
        {
            $notice = esc_html__( "Error: Something went wrong and the ticket wasn't created. Reload this page and try again.", "guest-support" );
            GuestSupportApp::setRedirect( $redirectUrl, $notice, 'success' );
        }
    }
}
elseif ( $handler == 'wpgs_subscription_key_form' )
{
    $apikey = isset( $_POST['apikey'] ) ? trim( sanitize_text_field( $_POST['apikey'] ) ) : '';
    $redirectUrl = admin_url( 'admin.php?page=guest-support-subscription-key' );
    update_option( 'guest_support_addon_key', $apikey );
    $notice = esc_html__( "API Key updated successfully.", "guest-support" );
    GuestSupportApp::setRedirect( $redirectUrl, $notice, 'success' );
}
elseif ( $handler == 'ajax' )
{
    $request = isset( $_POST['request'] ) ? sanitize_text_field( $_POST['request'] ) : '';
    if ( in_array( $request, ['get_users', 'edit_reply', 'delete_tickets'] ) )
    {
        require_once WPGS_PLUGIN_DIR . 'includes/library/ajax.php';
    }
    elseif ( $ispro ) {
        require_once WPGS_PLUGIN_ADDON_DIR . 'includes/library/pro-ajax.php';
    }
    else
    {
        $info = esc_html__( 'You need PRO Addon to use this feature.', 'guest-support' );
        echo json_encode($info);
        exit;
    }
}
elseif ( $handler == 'guest_support_login' )
{
    /**
     * Login user
     */
    $redirectUrl = GuestSupportApp::refererUrl();
    // Verify nonce
	if ( !isset( $_POST['_wpgs_login_form'] ) || !wp_verify_nonce( $_POST['_wpgs_login_form'], '_wpgs_login_form_428489363' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    if ( !$ispro )
    {
        $notice = esc_html__("You need PRO addon to use this feature.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

	// Is the login enabled?
	if ( GuestSupportApp::config()->display_login_form !== 'yes' )
	{
		$notice = esc_html__("The login feature is currently disabled.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    $username = isset( $_POST['wpgs_username'] ) ? sanitize_text_field( $_POST['wpgs_username'] ) : '';
    $password = isset( $_POST['wpgs_password'] ) ? sanitize_text_field( $_POST['wpgs_password'] ) : '';
    $rememberme = isset( $_POST['wpgs_rememberme'] ) ? sanitize_text_field( $_POST['wpgs_rememberme'] ) : '';
    $pageurl = isset( $_POST['wpgs_handler_url'] ) ? sanitize_url( $_POST['wpgs_handler_url'] ) : '';

    if ( $pageurl )
    {
        $redirectUrl = site_url( $pageurl );
    }

    if ( !$username || !$password || !$pageurl )
    {
        $notice = esc_html__("Enter your username and password to login.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    $data = array(
        'username' => $username,
        'password' => $password,
        'rememberme' => $rememberme,
        'pageurl' => $pageurl
    );

    GuestSupportAddonApp::loginUser( $data );
}
elseif ( $handler == 'guest_support_register' )
{
    /**
     * Register new user
     */
    $redirectUrl = GuestSupportApp::refererUrl();
    if ( !$ispro )
    {
        $notice = esc_html__("You need PRO addon to use this feature.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

	// Is the registration enabled?
	if ( GuestSupportApp::config()->display_registration_form !== 'yes' )
	{
		$notice = esc_html__("The registration feature is currently disabled.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
		exit;
	}

    $fullname = isset( $_POST['wpgs_r_fullname'] ) ? trim( sanitize_text_field( $_POST['wpgs_r_fullname'] ) ) : '';
    $username = isset( $_POST['wpgs_r_username'] ) ? trim( sanitize_text_field( $_POST['wpgs_r_username'] ) ) : '';
    $email = isset( $_POST['wpgs_r_email'] ) ? trim( sanitize_email( $_POST['wpgs_r_email'] ) ) : '';
    $password = isset( $_POST['wpgs_r_passwords'] ) ? sanitize_text_field( $_POST['wpgs_r_passwords'] ) : '';
    $pageurl = isset( $_POST['wpgs_handler_url'] ) ? sanitize_url( $_POST['wpgs_handler_url'] ) : '';

    if ( $pageurl )
    {
        $redirectUrl = site_url( $pageurl );
    }

    // Verify nonce
	if ( !isset( $_POST['_wpgs_register_form'] ) || !wp_verify_nonce( $_POST['_wpgs_register_form'], '_wpgs_register_form_54937638473' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    // Verify email from session
    $emailfrom_session = GuestSupportApp::getSession( 'wpgs_register_email_verified' );
    $emailto_md5 = md5( $email );

    if ( !$emailfrom_session || ( $emailfrom_session != $emailto_md5 ) )
    {
        $notice = esc_html__("Error: Email verification failed.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    if ( !$fullname || !$email || !$password || !$pageurl )
    {
        $notice = esc_html__("Enter all requested data to create a new account.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    $fullname_parts = explode(" ", $fullname);
    if(count($fullname_parts) > 1) {
        $lastname = array_pop($fullname_parts);
        $firstname = implode(" ", $fullname_parts);
    }
    else
    {
        $firstname = $name;
        $lastname = " ";
    }

    $data = array(
        'display_name' => $fullname,
        'user_login' => $username,
        'user_email' => $email,
        'user_pass' => $password,
        'user_nicename' => $username,
        'first_name' => $firstname,
        'last_name' => $lastname,
        'role' => 'subscriber'
    );

    GuestSupportAddonApp::registerUser( $data );
}
elseif ( $handler == 'guest_support_user_settings' )
{
    /**
     * User Settings
     */
    $redirectUrl = GuestSupportApp::refererUrl();
    if ( !$ispro )
    {
        $notice = esc_html__("You need PRO addon to use this feature.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
    }

    // Verify nonce
	if ( !isset( $_POST['_wpgs_user_settings_form'] ) || !wp_verify_nonce( $_POST['_wpgs_user_settings_form'], '_wpgs_user_settings_form_74929624827' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
        $redirectUrl = GuestSupportApp::refererUrl();
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    $data = filter_var_array( $_POST );

    GuestSupportAddonApp::userSettings( $data );
}
elseif ( $handler == 'add_email_template' || $handler == 'edit_email_template' )
{
    /**
     * Add or edit email template
     */
    // Check if user is in Admin area and can administrator
    GuestSupportApp::verifyAdmin();

    $redirectUrl = admin_url( 'admin.php?page=guest-support-email-templates' );
    // Verify nonce
	if ( !isset( $_POST['_wpnonce_email_template_form'] ) || !wp_verify_nonce( $_POST['_wpnonce_email_template_form'], '_wpnonce_email_template_form_action' ) ) {
		$notice = esc_html__("Invalid request.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    // Get post data
	$type = isset( $_GET['type'] ) ? sanitize_text_field( $_GET['type'] ) : '';
    $template_id = isset( $_GET['template_id'] ) ? sanitize_text_field( $_GET['template_id'] ) : '';
    $title = isset( $_POST['title'] ) ? sanitize_text_field( $_POST['title'] ) : '';
    $template_type = isset( $_POST['template_type'] ) ? sanitize_text_field( $_POST['template_type'] ) : '';
	$subject = isset( $_POST['subject'] ) ? sanitize_text_field( $_POST['subject'] ) : '';
	$template = isset( $_POST['template'] ) ? wp_kses_post( $_POST['template'] ) : '';
	$active = isset( $_POST['active'] ) ? (int) sanitize_text_field( $_POST['active'] ) : 1;

    $redirectUrl = admin_url( 'admin.php?page=guest-support-email-templates&type=edit&template_id=' . $template_id );

    // Template id, title and template content should not empty
	if ( $type == 'edit' && !$template_id )
	{
        $notice = esc_html__("Error: Invalid template id.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}
	if ( !$title )
	{
        $notice = esc_html__("Error: Empty 'Title' not allowed.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}
	if ( !$template )
	{
        $notice = esc_html__("Error: Empty 'Template' not allowed.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
	}

    // Process the udpate
    $data = array(
		'id' => $template_id,
        'title' => $title,
        'type' => $template_type,
        'subject' => $subject,
        'template' => $template,
        'active' => $active
    );

    if ( $type == 'add-new' )
    {
		$redirectUrl = admin_url( 'admin.php?page=guest-support-email-templates' );
		$notice = esc_html__("Error: This action is not allowed.", "guest-support");
		GuestSupportApp::setRedirect( $redirectUrl, $notice, 'error' );
        // GuestSupportAddonApp::addDepartment( $data );
    }
    elseif ( $type == 'edit' && $template_id && $template_type )
    {
        GuestSupportApp::editEmailTemplate( $data );
    }
}
elseif ( $handler == 'askforreview' )
{
	/**
	 * Disable Ask for review
	 */
	$redirectUrl = admin_url( 'admin.php?page=guest-support' );
	GuestSupportApp::disableAskForReview();
	GuestSupportApp::setRedirect( $redirectUrl, '', '' );
}