<?php
/**
 * @package     Guest Support
 * @author      RcaTheme.com <support@rcatheme.com>
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GPLv3
 * @link        https://www.rcatheme.com
 * @copyright   2024 RcaTheme.com, All rights resered.
 */
defined('ABSPATH') or die;

if ( $options['view'] == 'form' )
{
    /**
     * Show thank you message after successfully submitting a new ticket
     */
	$ticketid = isset( $_GET['ticketid'] ) ? sanitize_text_field( $_GET['ticketid'] ) : '';

    if ( GuestSupportApp::getSession('wpgs_create_ticket_success') === true )
    {
		// Reset ticket success option
		GuestSupportApp::createSession( 'wpgs_create_ticket_success', '' );

		// Get and reset ticket success message
		$confirmation_message = GuestSupportApp::getSession('wpgs_create_ticket_success_message');
		GuestSupportApp::createSession( 'wpgs_create_ticket_success_message', '' );

		// Show thank you message
        require_once WPGS_PLUGIN_DIR . 'includes/view-thankyou.php';
    }
    else
    {
        /**
         * Show frontend form
         */
        $formid = (int) $options['id'];

        $db = new GuestSupportDbquery();
        $form = $db->getForm( $formid );
        $config = GuestSupportApp::config();

        if ( !empty( $form ) )
        {
            // Load reCAPTCHA script
            if ( $form->recaptcha && $config->recaptcha_secret_key && $config->recaptcha_site_key )
            {
                wp_enqueue_script( 'guest-support-recaptcha-script' );
            }

            $departments = $db->listFormDepartments( $formid );
            $formFields = unserialize( $form->form_fields );
            if ( !GuestSupportApp::isPro() )
            {
                unset( $formFields['field_department'] );
            }
            require_once WPGS_PLUGIN_DIR . 'includes/view-form.php';
        }
        else
        {
            echo '<p>' . esc_html__( 'Guest Support: No form found to show here. Please add correct shortcode.', 'guest-support' ) . '</p>';
        }
    }
}
elseif ( $options['view'] == 'ticket' )
{
    add_filter( 'wp_robots', function( $robots ) {
        $robots['noindex'] = true;
        $robots['nofollow'] = true;
        $robots['max-image-preview'] = false;
        return $robots;
    } );

    global $wp_query;

    $ticket_id = isset( $wp_query->query_vars['id'] ) ? $wp_query->query_vars['id'] : '';
    // $ticket_id = isset( $_GET['id'] ) ? sanitize_text_field( $_GET['id'] ) : '';
    $ticket_token = isset( $_GET['auth'] ) ? sanitize_text_field( $_GET['auth'] ) : '';

    if ( !$ticket_id || !$ticket_token )
    {
        // require_once WPGS_PLUGIN_DIR . 'includes/view-login-register.php';
        echo '<p>' . esc_html__( "Check your email address for the correct ticket link url to view or reply to your ticket.", "guest-support" ) . '</p>';
    }
    elseif ( GuestSupportApp::getSession('guest_support_deletedticket') == $ticket_id )
    {
        /**
         * Check if returning after Deleted ticket
         */
		GuestSupportApp::createSession('guest_support_deletedticket', '');
        echo wp_kses_post( GuestSupportApp::getNotice() );
    }
    else
    {
        $ticketSubject = GuestSupportApp::getTicketSubject( $ticket_id, $ticket_token );
        $ticket = GuestSupportApp::getTicket( $ticket_id, $ticket_token );

        if ( $ticketSubject === false || $ticket === false )
        {
            echo '<p>' . esc_html__( "No ticket found. Please check your email get the correct ticket link to view or reply to your ticket.", "guest-support" ) . '</p>';
        }
        else
        {
            $isagent = GuestSupportApp::isAgent();
            $agent_id_enc = GuestSupportApp::AgentIdEncrypt();
            $form = GuestSupportApp::getForm( $ticketSubject->form_id );
            $formFields = unserialize( $form->form_fields );
            $ticket_url = GuestSupportApp::getTicketPageUrl() . $ticket_id . '/?auth=' . $ticket_token . ( $isagent ? '&agent=' . $agent_id_enc : '' );
			$params = $form->params ? unserialize( $form->params ) : [];
			$otherTickets_limit = isset( $params['noof_other_tickets'] ) ? (int) $params['noof_other_tickets'] : 10;
            $otherTickets = GuestSupportApp::getOtherTickets( $ticket_id, $ticketSubject->email, $otherTickets_limit );
            require_once WPGS_PLUGIN_DIR . 'includes/view-ticket.php';
        }
    }
}
elseif ( $options['view'] == 'tickets' )
{
    $ispro = GuestSupportApp::isPro();
    if ( $ispro )
    {
        if ( !is_user_logged_in() )
        {
            require_once WPGS_PLUGIN_ADDON_DIR . 'includes/view-login-register.php';
        }
        else
        {
            require_once WPGS_PLUGIN_ADDON_DIR . 'includes/view-tickets.php';
        }
    }
    else
    {
        echo '<p>' . esc_html__( "PRO Addon required to display a list of tickets on the frontend.", "guest-support" ) . '</p>';
    }
}